/* Generated by Cython 0.27.3 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_27_3"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX < 0x030700A0 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject **args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__cprotobuf__internal
#define __PYX_HAVE_API__cprotobuf__internal
#include <string.h>
#include <stdio.h>
#include "pythread.h"
#include <stdint.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "cprotobuf/internal.pyx",
  "stringsource",
  "cprotobuf/utils.pxi",
  "type.pxd",
  "bool.pxd",
  "complex.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer;
struct __pyx_obj_9cprotobuf_8internal_Field;

/* "cprotobuf/utils.pxi":12
 *     char* PyByteArray_AS_STRING(object bytearray)
 * 
 * ctypedef object(*Decoder)(char **pointer, char *end)             # <<<<<<<<<<<<<<
 * ctypedef object(*Encoder)(Field _f, bytearray array, object value)
 * 
 */
typedef PyObject *(*__pyx_t_9cprotobuf_8internal_Decoder)(char **, char *);

/* "cprotobuf/utils.pxi":13
 * 
 * ctypedef object(*Decoder)(char **pointer, char *end)
 * ctypedef object(*Encoder)(Field _f, bytearray array, object value)             # <<<<<<<<<<<<<<
 * 
 * class InternalDecodeError(Exception):
 */
typedef PyObject *(*__pyx_t_9cprotobuf_8internal_Encoder)(struct __pyx_obj_9cprotobuf_8internal_Field *, PyObject *, PyObject *);

/* "cprotobuf/internal.pyx":43
 * }
 * 
 * cdef class RepeatedContainer(list):             # <<<<<<<<<<<<<<
 *     cdef object klass
 * 
 */
struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer {
  PyListObject __pyx_base;
  PyObject *klass;
};


/* "cprotobuf/internal.pyx":54
 *         return obj
 * 
 * cdef class Field(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef public str name
 */
struct __pyx_obj_9cprotobuf_8internal_Field {
  PyObject_HEAD
  struct __pyx_vtabstruct_9cprotobuf_8internal_Field *__pyx_vtab;
  PyObject *name;
  PyObject *type;
  int index;
  int packed;
  int required;
  int repeated;
  PyObject *klass;
  PyObject *__pyx_default;
  unsigned char wire_type;
  __pyx_t_9cprotobuf_8internal_Encoder encoder;
  __pyx_t_9cprotobuf_8internal_Decoder decoder;
};



struct __pyx_vtabstruct_9cprotobuf_8internal_Field {
  PyObject *(*resolve_klass)(struct __pyx_obj_9cprotobuf_8internal_Field *);
  unsigned char (*get_wire_type)(struct __pyx_obj_9cprotobuf_8internal_Field *);
  __pyx_t_9cprotobuf_8internal_Encoder (*get_encoder)(struct __pyx_obj_9cprotobuf_8internal_Field *);
  __pyx_t_9cprotobuf_8internal_Decoder (*get_decoder)(struct __pyx_obj_9cprotobuf_8internal_Field *);
};
static struct __pyx_vtabstruct_9cprotobuf_8internal_Field *__pyx_vtabptr_9cprotobuf_8internal_Field;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o,n,NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg);

/* append.proto */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x);

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args))
                PyErr_SetObject(PyExc_KeyError, args);
            Py_XDECREF(args);
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#else
    #define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunction.proto */
#define __Pyx_CyFunction_USED 1
#include <structmember.h>
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_NewEx(ml, flags, qualname, self, module, globals, code)\
    __Pyx_CyFunction_New(__pyx_CyFunctionType, ml, flags, qualname, self, module, globals, code)
static PyObject *__Pyx_CyFunction_New(PyTypeObject *, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* dict_getitem_default.proto */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value);

/* PyObjectCallMethod2.proto */
static PyObject* __Pyx_PyObject_CallMethod2(PyObject* obj, PyObject* method_name, PyObject* arg1, PyObject* arg2);

/* dict_setdefault.proto */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value, int is_safe_type);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchInspect.proto */
static PyObject* __Pyx_patch_inspect(PyObject* module);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint64_t(uint64_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint32_t(uint32_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int32_t(int32_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int64_t(int64_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_char(unsigned char value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE uint32_t __Pyx_PyInt_As_uint32_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int32_t __Pyx_PyInt_As_int32_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE uint64_t __Pyx_PyInt_As_uint64_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int64_t __Pyx_PyInt_As_int64_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned char __Pyx_PyInt_As_unsigned_char(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_9cprotobuf_8internal_5Field_resolve_klass(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self); /* proto*/
static unsigned char __pyx_f_9cprotobuf_8internal_5Field_get_wire_type(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self); /* proto*/
static __pyx_t_9cprotobuf_8internal_Encoder __pyx_f_9cprotobuf_8internal_5Field_get_encoder(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self); /* proto*/
static __pyx_t_9cprotobuf_8internal_Decoder __pyx_f_9cprotobuf_8internal_5Field_get_decoder(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self); /* proto*/

/* Module declarations from 'cpython.version' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.exc' */

/* Module declarations from 'cpython.module' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'cpython.tuple' */

/* Module declarations from 'cpython.list' */

/* Module declarations from 'cpython.sequence' */

/* Module declarations from 'cpython.mapping' */

/* Module declarations from 'cpython.iterator' */

/* Module declarations from 'cpython.number' */

/* Module declarations from 'cpython.int' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.bool' */
static PyTypeObject *__pyx_ptype_7cpython_4bool_bool = 0;

/* Module declarations from 'cpython.long' */

/* Module declarations from 'cpython.float' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.complex' */
static PyTypeObject *__pyx_ptype_7cpython_7complex_complex = 0;

/* Module declarations from 'cpython.string' */

/* Module declarations from 'cpython.unicode' */

/* Module declarations from 'cpython.dict' */

/* Module declarations from 'cpython.instance' */

/* Module declarations from 'cpython.function' */

/* Module declarations from 'cpython.method' */

/* Module declarations from 'cpython.weakref' */

/* Module declarations from 'cpython.getargs' */

/* Module declarations from 'cpython.pythread' */

/* Module declarations from 'cpython.pystate' */

/* Module declarations from 'cpython.cobject' */

/* Module declarations from 'cpython.oldbuffer' */

/* Module declarations from 'cpython.set' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'cpython.bytes' */

/* Module declarations from 'cpython.pycapsule' */

/* Module declarations from 'cpython' */

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'cprotobuf.internal' */
static PyTypeObject *__pyx_ptype_9cprotobuf_8internal_RepeatedContainer = 0;
static PyTypeObject *__pyx_ptype_9cprotobuf_8internal_Field = 0;
static PyObject *__pyx_v_9cprotobuf_8internal__proto_classes = 0;
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_makeDecodeError(char *, PyObject *); /*proto*/
static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_decode_uint32(char **, char *, uint32_t *); /*proto*/
static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_decode_uint64(char **, char *, uint64_t *); /*proto*/
static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_decode_fixed32(char **, char *, uint32_t *); /*proto*/
static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_decode_fixed64(char **, char *, uint64_t *); /*proto*/
static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_decode_delimited(char **, char *, char **, uint64_t *); /*proto*/
static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_skip_unknown_field(char **, char *, int); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal_decode_uint32(char **, char *); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal_decode_uint64(char **, char *); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal_decode_int64(char **, char *); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal_decode_sint32(char **, char *); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal_decode_sint64(char **, char *); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal_decode_fixed32(char **, char *); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal_decode_fixed64(char **, char *); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal_decode_sfixed32(char **, char *); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal_decode_sfixed64(char **, char *); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal_decode_bytes(char **, char *); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal_decode_string(char **, char *); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal_decode_float(char **, char *); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal_decode_double(char **, char *); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal_decode_bool(char **, char *); /*proto*/
static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_encode_uint32(PyObject *, uint32_t); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_uint32(struct __pyx_obj_9cprotobuf_8internal_Field *, PyObject *, PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_sint32(struct __pyx_obj_9cprotobuf_8internal_Field *, PyObject *, PyObject *); /*proto*/
static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_encode_uint64(PyObject *, uint64_t); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_uint64(struct __pyx_obj_9cprotobuf_8internal_Field *, PyObject *, PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_int64(struct __pyx_obj_9cprotobuf_8internal_Field *, PyObject *, PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_sint64(struct __pyx_obj_9cprotobuf_8internal_Field *, PyObject *, PyObject *); /*proto*/
static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_encode_fixed32(PyObject *, uint32_t); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_fixed32(struct __pyx_obj_9cprotobuf_8internal_Field *, PyObject *, PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_sfixed32(struct __pyx_obj_9cprotobuf_8internal_Field *, PyObject *, PyObject *); /*proto*/
static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_encode_fixed64(PyObject *, uint64_t); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_fixed64(struct __pyx_obj_9cprotobuf_8internal_Field *, PyObject *, PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_sfixed64(struct __pyx_obj_9cprotobuf_8internal_Field *, PyObject *, PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_bytes(struct __pyx_obj_9cprotobuf_8internal_Field *, PyObject *, PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_string(struct __pyx_obj_9cprotobuf_8internal_Field *, PyObject *, PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_bool(struct __pyx_obj_9cprotobuf_8internal_Field *, PyObject *, PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_float(struct __pyx_obj_9cprotobuf_8internal_Field *, PyObject *, PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_double(struct __pyx_obj_9cprotobuf_8internal_Field *, PyObject *, PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_type(PyObject *, unsigned char, uint32_t); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal_get_proto(PyObject *, int __pyx_skip_dispatch); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_subobject(struct __pyx_obj_9cprotobuf_8internal_Field *, PyObject *, PyObject *); /*proto*/
static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_decode_object(PyObject *, char **, char *); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal___pyx_unpickle_RepeatedContainer__set_state(struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "cprotobuf.internal"
extern int __pyx_module_is_main_cprotobuf__internal;
int __pyx_module_is_main_cprotobuf__internal = 0;

/* Implementation of 'cprotobuf.internal' */
static PyObject *__pyx_builtin_object;
static PyObject *__pyx_builtin_KeyError;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_super;
static const char __pyx_k_d[] = "d";
static const char __pyx_k_e[] = "e";
static const char __pyx_k_f[] = "f";
static const char __pyx_k_s[] = "s";
static const char __pyx_k_v[] = "v";
static const char __pyx_k__3[] = "__";
static const char __pyx_k__10[] = "";
static const char __pyx_k_buf[] = "buf";
static const char __pyx_k_cls[] = "cls";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_end[] = "end";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_str[] = "__str__";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_bool[] = "bool";
static const char __pyx_k_buf1[] = "buf1";
static const char __pyx_k_buff[] = "buff";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_enum[] = "enum";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_item[] = "item";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_sort[] = "sort";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_type[] = "type";
static const char __pyx_k_attrs[] = "attrs";
static const char __pyx_k_bases[] = "bases";
static const char __pyx_k_bytes[] = "bytes";
static const char __pyx_k_count[] = "count";
static const char __pyx_k_float[] = "float";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_int32[] = "int32";
static const char __pyx_k_int64[] = "int64";
static const char __pyx_k_items[] = "items";
static const char __pyx_k_start[] = "start";
static const char __pyx_k_super[] = "super";
static const char __pyx_k_utf_8[] = "utf-8";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_append[] = "append";
static const char __pyx_k_decode[] = "decode";
static const char __pyx_k_double[] = "double";
static const char __pyx_k_fields[] = "_fields";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_kwargs[] = "kwargs";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_newcls[] = "newcls";
static const char __pyx_k_object[] = "object";
static const char __pyx_k_offset[] = "offset";
static const char __pyx_k_packed[] = "packed";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_sint32[] = "sint32";
static const char __pyx_k_sint64[] = "sint64";
static const char __pyx_k_string[] = "string";
static const char __pyx_k_todict[] = "todict";
static const char __pyx_k_uint32[] = "uint32";
static const char __pyx_k_uint64[] = "uint64";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_clsname[] = "clsname";
static const char __pyx_k_default[] = "default";
static const char __pyx_k_fixed32[] = "fixed32";
static const char __pyx_k_fixed64[] = "fixed64";
static const char __pyx_k_inspect[] = "inspect";
static const char __pyx_k_isclass[] = "isclass";
static const char __pyx_k_message[] = "message";
static const char __pyx_k_pointer[] = "pointer";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_unicode[] = "__unicode__";
static const char __pyx_k_KeyError[] = "KeyError";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_repeated[] = "repeated";
static const char __pyx_k_required[] = "required";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_sfixed32[] = "sfixed32";
static const char __pyx_k_sfixed64[] = "sfixed64";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_fieldsmap[] = "_fieldsmap";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_setdefault[] = "setdefault";
static const char __pyx_k_startswith[] = "startswith";
static const char __pyx_k_wire_types[] = "wire_types";
static const char __pyx_k_DecodeError[] = "DecodeError";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_ProtoEntity[] = "ProtoEntity";
static const char __pyx_k_encode_data[] = "encode_data";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_invalid_type_s[] = "invalid type %s";
static const char __pyx_k_register_proto[] = "register_proto";
static const char __pyx_k_MetaProtoEntity[] = "MetaProtoEntity";
static const char __pyx_k_ParseFromString[] = "ParseFromString";
static const char __pyx_k_default_objects[] = "default_objects";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_DecodeError___str[] = "DecodeError.__str__";
static const char __pyx_k_ProtoEntity___str[] = "ProtoEntity.__str__";
static const char __pyx_k_SerializeToString[] = "SerializeToString";
static const char __pyx_k_fieldsmap_by_name[] = "_fieldsmap_by_name";
static const char __pyx_k_Count_out_of_bound[] = "Count out of bound.";
static const char __pyx_k_DecodeError___init[] = "DecodeError.__init__";
static const char __pyx_k_ProtoEntity___init[] = "ProtoEntity.__init__";
static const char __pyx_k_ProtoEntity_todict[] = "ProtoEntity.todict";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_cprotobuf_internal[] = "cprotobuf.internal";
static const char __pyx_k_InternalDecodeError[] = "InternalDecodeError";
static const char __pyx_k_Offset_out_of_bound[] = "Offset out of bound.";
static const char __pyx_k_cprotobuf_utils_pxi[] = "cprotobuf/utils.pxi";
static const char __pyx_k_MetaProtoEntity___new[] = "MetaProtoEntity.__new__";
static const char __pyx_k_ProtoEntity___unicode[] = "ProtoEntity.__unicode__";
static const char __pyx_k_cprotobuf_internal_pyx[] = "cprotobuf/internal.pyx";
static const char __pyx_k_duplicate_field_index_s[] = "duplicate field index %s";
static const char __pyx_k_ProtoEntity_ParseFromString[] = "ProtoEntity.ParseFromString";
static const char __pyx_k_ProtoEntity_SerializeToString[] = "ProtoEntity.SerializeToString";
static const char __pyx_k_pyx_unpickle_RepeatedContainer[] = "__pyx_unpickle_RepeatedContainer";
static const char __pyx_k_Can_t_deserialize_type_tag_at_0[] = "Can't deserialize type tag at [{0}] for value";
static const char __pyx_k_Can_t_decode_size_for_value_of_t[] = "Can't decode size for value of type `bytes` at [{0}]";
static const char __pyx_k_Can_t_decode_value_of_sub_messag[] = "Can't decode value of sub message at [{0}]";
static const char __pyx_k_Can_t_decode_value_of_type_bytes[] = "Can't decode value of type `bytes` of size %d at [{0}]";
static const char __pyx_k_Can_t_decode_value_of_type_doubl[] = "Can't decode value of type `double` at [{0}]";
static const char __pyx_k_Can_t_decode_value_of_type_fixed[] = "Can't decode value of type `fixed32` at [{0}]";
static const char __pyx_k_Can_t_decode_value_of_type_float[] = "Can't decode value of type `float` at [{0}]";
static const char __pyx_k_Can_t_decode_value_of_type_int32[] = "Can't decode value of type `int32` at [{0}]";
static const char __pyx_k_Can_t_decode_value_of_type_int64[] = "Can't decode value of type `int64` at [{0}]";
static const char __pyx_k_Can_t_decode_value_of_type_packe[] = "Can't decode value of type `packed` at [{0}]";
static const char __pyx_k_Can_t_decode_value_of_type_sfixe[] = "Can't decode value of type `sfixed32` at [{0}]";
static const char __pyx_k_Can_t_decode_value_of_type_sint3[] = "Can't decode value of type `sint32` at [{0}]";
static const char __pyx_k_Can_t_decode_value_of_type_sint6[] = "Can't decode value of type `sint64` at [{0}]";
static const char __pyx_k_Can_t_decode_value_of_type_strin[] = "Can't decode value of type `string` of size %d at [{0}]";
static const char __pyx_k_Can_t_decode_value_of_type_uint3[] = "Can't decode value of type `uint32` at [{0}]";
static const char __pyx_k_Can_t_decode_value_of_type_uint6[] = "Can't decode value of type `uint64` at [{0}]";
static const char __pyx_k_Can_t_skip_enough_bytes_for_wire[] = "Can't skip enough bytes for wire_type %s at [{0}] for value";
static const char __pyx_k_Incompatible_checksums_s_vs_0x64[] = "Incompatible checksums (%s vs 0x64a34c4 = (klass))";
static const char __pyx_k_MetaProtoEntity___new___locals_l[] = "MetaProtoEntity.__new__.<locals>.<lambda>";
static const char __pyx_k_packed_must_be_used_with_repeate[] = "packed must be used with repeated";
static const char __pyx_k_self_decoder_self_encoder_cannot[] = "self.decoder,self.encoder cannot be converted to a Python object for pickling";
static const char __pyx_k_Can_t_decode_size_for_value_of_t_2[] = "Can't decode size for value of type `string` at [{0}]";
static const char __pyx_k_Can_t_decode_value_of_type_fixed_2[] = "Can't decode value of type `fixed64` at [{0}]";
static const char __pyx_k_Can_t_decode_value_of_type_sfixe_2[] = "Can't decode value of type `sfixed64` at [{0}]";
static PyObject *__pyx_kp_s_Can_t_decode_size_for_value_of_t;
static PyObject *__pyx_kp_s_Can_t_decode_size_for_value_of_t_2;
static PyObject *__pyx_kp_s_Can_t_decode_value_of_sub_messag;
static PyObject *__pyx_kp_s_Can_t_decode_value_of_type_bytes;
static PyObject *__pyx_kp_s_Can_t_decode_value_of_type_doubl;
static PyObject *__pyx_kp_s_Can_t_decode_value_of_type_fixed;
static PyObject *__pyx_kp_s_Can_t_decode_value_of_type_fixed_2;
static PyObject *__pyx_kp_s_Can_t_decode_value_of_type_float;
static PyObject *__pyx_kp_s_Can_t_decode_value_of_type_int32;
static PyObject *__pyx_kp_s_Can_t_decode_value_of_type_int64;
static PyObject *__pyx_kp_s_Can_t_decode_value_of_type_packe;
static PyObject *__pyx_kp_s_Can_t_decode_value_of_type_sfixe;
static PyObject *__pyx_kp_s_Can_t_decode_value_of_type_sfixe_2;
static PyObject *__pyx_kp_s_Can_t_decode_value_of_type_sint3;
static PyObject *__pyx_kp_s_Can_t_decode_value_of_type_sint6;
static PyObject *__pyx_kp_s_Can_t_decode_value_of_type_strin;
static PyObject *__pyx_kp_s_Can_t_decode_value_of_type_uint3;
static PyObject *__pyx_kp_s_Can_t_decode_value_of_type_uint6;
static PyObject *__pyx_kp_s_Can_t_deserialize_type_tag_at_0;
static PyObject *__pyx_kp_s_Can_t_skip_enough_bytes_for_wire;
static PyObject *__pyx_kp_s_Count_out_of_bound;
static PyObject *__pyx_n_s_DecodeError;
static PyObject *__pyx_n_s_DecodeError___init;
static PyObject *__pyx_n_s_DecodeError___str;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x64;
static PyObject *__pyx_n_s_InternalDecodeError;
static PyObject *__pyx_n_s_KeyError;
static PyObject *__pyx_n_s_MetaProtoEntity;
static PyObject *__pyx_n_s_MetaProtoEntity___new;
static PyObject *__pyx_n_s_MetaProtoEntity___new___locals_l;
static PyObject *__pyx_kp_s_Offset_out_of_bound;
static PyObject *__pyx_n_s_ParseFromString;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_ProtoEntity;
static PyObject *__pyx_n_s_ProtoEntity_ParseFromString;
static PyObject *__pyx_n_s_ProtoEntity_SerializeToString;
static PyObject *__pyx_n_s_ProtoEntity___init;
static PyObject *__pyx_n_s_ProtoEntity___str;
static PyObject *__pyx_n_s_ProtoEntity___unicode;
static PyObject *__pyx_n_s_ProtoEntity_todict;
static PyObject *__pyx_n_s_SerializeToString;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_s__10;
static PyObject *__pyx_kp_u__10;
static PyObject *__pyx_n_s__3;
static PyObject *__pyx_n_s_append;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_attrs;
static PyObject *__pyx_n_s_bases;
static PyObject *__pyx_n_s_bool;
static PyObject *__pyx_n_s_buf;
static PyObject *__pyx_n_s_buf1;
static PyObject *__pyx_n_s_buff;
static PyObject *__pyx_n_s_bytes;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_cls;
static PyObject *__pyx_n_s_clsname;
static PyObject *__pyx_n_s_count;
static PyObject *__pyx_n_s_cprotobuf_internal;
static PyObject *__pyx_kp_s_cprotobuf_internal_pyx;
static PyObject *__pyx_kp_s_cprotobuf_utils_pxi;
static PyObject *__pyx_n_s_d;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_s_decode;
static PyObject *__pyx_n_s_default;
static PyObject *__pyx_n_s_default_objects;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_double;
static PyObject *__pyx_kp_s_duplicate_field_index_s;
static PyObject *__pyx_n_s_e;
static PyObject *__pyx_n_s_encode_data;
static PyObject *__pyx_n_s_end;
static PyObject *__pyx_n_s_enum;
static PyObject *__pyx_n_s_f;
static PyObject *__pyx_n_s_fields;
static PyObject *__pyx_n_s_fieldsmap;
static PyObject *__pyx_n_s_fieldsmap_by_name;
static PyObject *__pyx_n_s_fixed32;
static PyObject *__pyx_n_s_fixed64;
static PyObject *__pyx_n_s_float;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_index;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_inspect;
static PyObject *__pyx_n_s_int32;
static PyObject *__pyx_n_s_int64;
static PyObject *__pyx_kp_s_invalid_type_s;
static PyObject *__pyx_n_s_isclass;
static PyObject *__pyx_n_s_item;
static PyObject *__pyx_n_s_items;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_kwargs;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_message;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_newcls;
static PyObject *__pyx_n_s_object;
static PyObject *__pyx_n_s_offset;
static PyObject *__pyx_n_s_packed;
static PyObject *__pyx_kp_s_packed_must_be_used_with_repeate;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_pointer;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_RepeatedContainer;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_register_proto;
static PyObject *__pyx_n_s_repeated;
static PyObject *__pyx_n_s_required;
static PyObject *__pyx_n_s_s;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_kp_s_self_decoder_self_encoder_cannot;
static PyObject *__pyx_n_s_setdefault;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_sfixed32;
static PyObject *__pyx_n_s_sfixed64;
static PyObject *__pyx_n_s_sint32;
static PyObject *__pyx_n_s_sint64;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_sort;
static PyObject *__pyx_n_s_start;
static PyObject *__pyx_n_s_startswith;
static PyObject *__pyx_n_s_str;
static PyObject *__pyx_n_s_string;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_super;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_todict;
static PyObject *__pyx_n_s_type;
static PyObject *__pyx_n_s_uint32;
static PyObject *__pyx_n_s_uint64;
static PyObject *__pyx_n_s_unicode;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_kp_s_utf_8;
static PyObject *__pyx_n_s_v;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_wire_types;
static PyObject *__pyx_pf_9cprotobuf_8internal_11DecodeError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_pointer, PyObject *__pyx_v_message); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_11DecodeError_2__str__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static int __pyx_pf_9cprotobuf_8internal_17RepeatedContainer___init__(struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *__pyx_v_self, PyObject *__pyx_v_cls); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_17RepeatedContainer_2add(struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *__pyx_v_self, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_17RepeatedContainer_4__reduce_cython__(struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_17RepeatedContainer_6__setstate_cython__(struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_9cprotobuf_8internal_5Field___init__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, PyObject *__pyx_v_type, PyObject *__pyx_v_index, PyObject *__pyx_v_required, PyObject *__pyx_v_repeated, PyObject *__pyx_v_packed, PyObject *__pyx_v_default); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_2__get__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, PyObject *__pyx_v_instance, CYTHON_UNUSED PyObject *__pyx_v_type); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_4name___get__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9cprotobuf_8internal_5Field_4name_2__set__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_9cprotobuf_8internal_5Field_4name_4__del__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_5index___get__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9cprotobuf_8internal_5Field_5index_2__set__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_6packed___get__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9cprotobuf_8internal_5Field_6packed_2__set__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_8required___get__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9cprotobuf_8internal_5Field_8required_2__set__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_8repeated___get__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9cprotobuf_8internal_5Field_8repeated_2__set__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_9wire_type___get__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self); /* proto */
static int __pyx_pf_9cprotobuf_8internal_5Field_9wire_type_2__set__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_get_proto(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_2register_proto(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name, PyObject *__pyx_v_cls); /* proto */
static PyObject *__pyx_lambda_funcdef_lambda(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_f); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_15MetaProtoEntity___new__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cls, PyObject *__pyx_v_clsname, PyObject *__pyx_v_bases, PyObject *__pyx_v_attrs); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_11ProtoEntity___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_11ProtoEntity_2SerializeToString(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_11ProtoEntity_4ParseFromString(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_s, int __pyx_v_offset, int __pyx_v_count); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_11ProtoEntity_6__unicode__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_11ProtoEntity_8__str__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_11ProtoEntity_10todict(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_4encode_data(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cls, PyObject *__pyx_v_d); /* proto */
static PyObject *__pyx_pf_9cprotobuf_8internal_6__pyx_unpickle_RepeatedContainer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_9cprotobuf_8internal_RepeatedContainer(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9cprotobuf_8internal_Field(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_float_0_0;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_5;
static PyObject *__pyx_int_105526468;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_codeobj__7;
static PyObject *__pyx_codeobj__9;
static PyObject *__pyx_codeobj__12;
static PyObject *__pyx_codeobj__14;
static PyObject *__pyx_codeobj__17;
static PyObject *__pyx_codeobj__19;
static PyObject *__pyx_codeobj__21;
static PyObject *__pyx_codeobj__23;
static PyObject *__pyx_codeobj__25;
static PyObject *__pyx_codeobj__27;
static PyObject *__pyx_codeobj__29;
static PyObject *__pyx_codeobj__31;

/* "cprotobuf/utils.pxi":18
 *     pass
 * 
 * cdef inline object makeDecodeError(char* pointer, message):             # <<<<<<<<<<<<<<
 *     cdef uint64_t locator = <uint64_t>pointer
 *     return InternalDecodeError(locator, message)
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_makeDecodeError(char *__pyx_v_pointer, PyObject *__pyx_v_message) {
  uint64_t __pyx_v_locator;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("makeDecodeError", 0);

  /* "cprotobuf/utils.pxi":19
 * 
 * cdef inline object makeDecodeError(char* pointer, message):
 *     cdef uint64_t locator = <uint64_t>pointer             # <<<<<<<<<<<<<<
 *     return InternalDecodeError(locator, message)
 * 
 */
  __pyx_v_locator = ((uint64_t)__pyx_v_pointer);

  /* "cprotobuf/utils.pxi":20
 * cdef inline object makeDecodeError(char* pointer, message):
 *     cdef uint64_t locator = <uint64_t>pointer
 *     return InternalDecodeError(locator, message)             # <<<<<<<<<<<<<<
 * 
 * class DecodeError(Exception):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_InternalDecodeError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_uint64_t(__pyx_v_locator); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_3, __pyx_v_message};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 20, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_3, __pyx_v_message};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 20, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 20, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_message);
    __Pyx_GIVEREF(__pyx_v_message);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_message);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 20, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":18
 *     pass
 * 
 * cdef inline object makeDecodeError(char* pointer, message):             # <<<<<<<<<<<<<<
 *     cdef uint64_t locator = <uint64_t>pointer
 *     return InternalDecodeError(locator, message)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cprotobuf.internal.makeDecodeError", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":23
 * 
 * class DecodeError(Exception):
 *     def __init__(self, pointer, message):             # <<<<<<<<<<<<<<
 *         self.pointer = pointer
 *         self.message = message
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_11DecodeError_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9cprotobuf_8internal_11DecodeError_1__init__ = {"__init__", (PyCFunction)__pyx_pw_9cprotobuf_8internal_11DecodeError_1__init__, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9cprotobuf_8internal_11DecodeError_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_pointer = 0;
  PyObject *__pyx_v_message = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_pointer,&__pyx_n_s_message,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pointer)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 1); __PYX_ERR(2, 23, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_message)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 2); __PYX_ERR(2, 23, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(2, 23, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_self = values[0];
    __pyx_v_pointer = values[1];
    __pyx_v_message = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 23, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cprotobuf.internal.DecodeError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9cprotobuf_8internal_11DecodeError___init__(__pyx_self, __pyx_v_self, __pyx_v_pointer, __pyx_v_message);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_11DecodeError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_pointer, PyObject *__pyx_v_message) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cprotobuf/utils.pxi":24
 * class DecodeError(Exception):
 *     def __init__(self, pointer, message):
 *         self.pointer = pointer             # <<<<<<<<<<<<<<
 *         self.message = message
 *     def __str__(self):
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_pointer, __pyx_v_pointer) < 0) __PYX_ERR(2, 24, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":25
 *     def __init__(self, pointer, message):
 *         self.pointer = pointer
 *         self.message = message             # <<<<<<<<<<<<<<
 *     def __str__(self):
 *         return self.message.format(self.pointer)
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_message, __pyx_v_message) < 0) __PYX_ERR(2, 25, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":23
 * 
 * class DecodeError(Exception):
 *     def __init__(self, pointer, message):             # <<<<<<<<<<<<<<
 *         self.pointer = pointer
 *         self.message = message
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.DecodeError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":26
 *         self.pointer = pointer
 *         self.message = message
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message.format(self.pointer)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_11DecodeError_3__str__(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_9cprotobuf_8internal_11DecodeError_3__str__ = {"__str__", (PyCFunction)__pyx_pw_9cprotobuf_8internal_11DecodeError_3__str__, METH_O, 0};
static PyObject *__pyx_pw_9cprotobuf_8internal_11DecodeError_3__str__(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_11DecodeError_2__str__(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_11DecodeError_2__str__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "cprotobuf/utils.pxi":27
 *         self.message = message
 *     def __str__(self):
 *         return self.message.format(self.pointer)             # <<<<<<<<<<<<<<
 * 
 * # }}}
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_message); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_pointer); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 27, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_2};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 27, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_2};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 27, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 27, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
      __pyx_t_2 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 27, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":26
 *         self.pointer = pointer
 *         self.message = message
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message.format(self.pointer)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cprotobuf.internal.DecodeError.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":35
 * # {{{ raw stuff
 * 
 * cdef inline int raw_decode_uint32(char **start, char *end, uint32_t *result) nogil:             # <<<<<<<<<<<<<<
 *     cdef uint32_t value = 0
 *     cdef uint32_t byte
 */

static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_decode_uint32(char **__pyx_v_start, char *__pyx_v_end, uint32_t *__pyx_v_result) {
  uint32_t __pyx_v_value;
  uint32_t __pyx_v_byte;
  char *__pyx_v_pointer;
  int __pyx_v_counter;
  int __pyx_r;
  int __pyx_t_1;

  /* "cprotobuf/utils.pxi":36
 * 
 * cdef inline int raw_decode_uint32(char **start, char *end, uint32_t *result) nogil:
 *     cdef uint32_t value = 0             # <<<<<<<<<<<<<<
 *     cdef uint32_t byte
 *     cdef char *pointer = start[0]
 */
  __pyx_v_value = 0;

  /* "cprotobuf/utils.pxi":38
 *     cdef uint32_t value = 0
 *     cdef uint32_t byte
 *     cdef char *pointer = start[0]             # <<<<<<<<<<<<<<
 *     cdef int counter = 0
 *     while True:
 */
  __pyx_v_pointer = (__pyx_v_start[0]);

  /* "cprotobuf/utils.pxi":39
 *     cdef uint32_t byte
 *     cdef char *pointer = start[0]
 *     cdef int counter = 0             # <<<<<<<<<<<<<<
 *     while True:
 *         if pointer >= end:
 */
  __pyx_v_counter = 0;

  /* "cprotobuf/utils.pxi":40
 *     cdef char *pointer = start[0]
 *     cdef int counter = 0
 *     while True:             # <<<<<<<<<<<<<<
 *         if pointer >= end:
 *             return -1
 */
  while (1) {

    /* "cprotobuf/utils.pxi":41
 *     cdef int counter = 0
 *     while True:
 *         if pointer >= end:             # <<<<<<<<<<<<<<
 *             return -1
 *         byte = pointer[0]
 */
    __pyx_t_1 = ((__pyx_v_pointer >= __pyx_v_end) != 0);
    if (__pyx_t_1) {

      /* "cprotobuf/utils.pxi":42
 *     while True:
 *         if pointer >= end:
 *             return -1             # <<<<<<<<<<<<<<
 *         byte = pointer[0]
 *         value |= (byte & 0x7f) << counter
 */
      __pyx_r = -1;
      goto __pyx_L0;

      /* "cprotobuf/utils.pxi":41
 *     cdef int counter = 0
 *     while True:
 *         if pointer >= end:             # <<<<<<<<<<<<<<
 *             return -1
 *         byte = pointer[0]
 */
    }

    /* "cprotobuf/utils.pxi":43
 *         if pointer >= end:
 *             return -1
 *         byte = pointer[0]             # <<<<<<<<<<<<<<
 *         value |= (byte & 0x7f) << counter
 *         counter+=7
 */
    __pyx_v_byte = (__pyx_v_pointer[0]);

    /* "cprotobuf/utils.pxi":44
 *             return -1
 *         byte = pointer[0]
 *         value |= (byte & 0x7f) << counter             # <<<<<<<<<<<<<<
 *         counter+=7
 *         pointer+=1
 */
    __pyx_v_value = (__pyx_v_value | ((__pyx_v_byte & 0x7f) << __pyx_v_counter));

    /* "cprotobuf/utils.pxi":45
 *         byte = pointer[0]
 *         value |= (byte & 0x7f) << counter
 *         counter+=7             # <<<<<<<<<<<<<<
 *         pointer+=1
 *         if byte & 0x80 == 0:
 */
    __pyx_v_counter = (__pyx_v_counter + 7);

    /* "cprotobuf/utils.pxi":46
 *         value |= (byte & 0x7f) << counter
 *         counter+=7
 *         pointer+=1             # <<<<<<<<<<<<<<
 *         if byte & 0x80 == 0:
 *             break
 */
    __pyx_v_pointer = (__pyx_v_pointer + 1);

    /* "cprotobuf/utils.pxi":47
 *         counter+=7
 *         pointer+=1
 *         if byte & 0x80 == 0:             # <<<<<<<<<<<<<<
 *             break
 *     start[0] = pointer
 */
    __pyx_t_1 = (((__pyx_v_byte & 0x80) == 0) != 0);
    if (__pyx_t_1) {

      /* "cprotobuf/utils.pxi":48
 *         pointer+=1
 *         if byte & 0x80 == 0:
 *             break             # <<<<<<<<<<<<<<
 *     start[0] = pointer
 *     result[0] = value
 */
      goto __pyx_L4_break;

      /* "cprotobuf/utils.pxi":47
 *         counter+=7
 *         pointer+=1
 *         if byte & 0x80 == 0:             # <<<<<<<<<<<<<<
 *             break
 *     start[0] = pointer
 */
    }
  }
  __pyx_L4_break:;

  /* "cprotobuf/utils.pxi":49
 *         if byte & 0x80 == 0:
 *             break
 *     start[0] = pointer             # <<<<<<<<<<<<<<
 *     result[0] = value
 *     return 0
 */
  (__pyx_v_start[0]) = __pyx_v_pointer;

  /* "cprotobuf/utils.pxi":50
 *             break
 *     start[0] = pointer
 *     result[0] = value             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  (__pyx_v_result[0]) = __pyx_v_value;

  /* "cprotobuf/utils.pxi":51
 *     start[0] = pointer
 *     result[0] = value
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline int raw_decode_uint64(char **start, char *end, uint64_t *result) nogil:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":35
 * # {{{ raw stuff
 * 
 * cdef inline int raw_decode_uint32(char **start, char *end, uint32_t *result) nogil:             # <<<<<<<<<<<<<<
 *     cdef uint32_t value = 0
 *     cdef uint32_t byte
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":53
 *     return 0
 * 
 * cdef inline int raw_decode_uint64(char **start, char *end, uint64_t *result) nogil:             # <<<<<<<<<<<<<<
 *     cdef uint64_t value = 0
 *     cdef uint64_t byte
 */

static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_decode_uint64(char **__pyx_v_start, char *__pyx_v_end, uint64_t *__pyx_v_result) {
  uint64_t __pyx_v_value;
  uint64_t __pyx_v_byte;
  char *__pyx_v_pointer;
  int __pyx_v_counter;
  int __pyx_r;
  int __pyx_t_1;

  /* "cprotobuf/utils.pxi":54
 * 
 * cdef inline int raw_decode_uint64(char **start, char *end, uint64_t *result) nogil:
 *     cdef uint64_t value = 0             # <<<<<<<<<<<<<<
 *     cdef uint64_t byte
 *     cdef char *pointer = start[0]
 */
  __pyx_v_value = 0;

  /* "cprotobuf/utils.pxi":56
 *     cdef uint64_t value = 0
 *     cdef uint64_t byte
 *     cdef char *pointer = start[0]             # <<<<<<<<<<<<<<
 *     cdef int counter = 0
 * 
 */
  __pyx_v_pointer = (__pyx_v_start[0]);

  /* "cprotobuf/utils.pxi":57
 *     cdef uint64_t byte
 *     cdef char *pointer = start[0]
 *     cdef int counter = 0             # <<<<<<<<<<<<<<
 * 
 *     while True:
 */
  __pyx_v_counter = 0;

  /* "cprotobuf/utils.pxi":59
 *     cdef int counter = 0
 * 
 *     while True:             # <<<<<<<<<<<<<<
 *         if pointer >= end:
 *             return -1
 */
  while (1) {

    /* "cprotobuf/utils.pxi":60
 * 
 *     while True:
 *         if pointer >= end:             # <<<<<<<<<<<<<<
 *             return -1
 *         byte = pointer[0]
 */
    __pyx_t_1 = ((__pyx_v_pointer >= __pyx_v_end) != 0);
    if (__pyx_t_1) {

      /* "cprotobuf/utils.pxi":61
 *     while True:
 *         if pointer >= end:
 *             return -1             # <<<<<<<<<<<<<<
 *         byte = pointer[0]
 *         value |= (byte & 0x7f) << counter
 */
      __pyx_r = -1;
      goto __pyx_L0;

      /* "cprotobuf/utils.pxi":60
 * 
 *     while True:
 *         if pointer >= end:             # <<<<<<<<<<<<<<
 *             return -1
 *         byte = pointer[0]
 */
    }

    /* "cprotobuf/utils.pxi":62
 *         if pointer >= end:
 *             return -1
 *         byte = pointer[0]             # <<<<<<<<<<<<<<
 *         value |= (byte & 0x7f) << counter
 *         counter+=7
 */
    __pyx_v_byte = (__pyx_v_pointer[0]);

    /* "cprotobuf/utils.pxi":63
 *             return -1
 *         byte = pointer[0]
 *         value |= (byte & 0x7f) << counter             # <<<<<<<<<<<<<<
 *         counter+=7
 *         pointer+=1
 */
    __pyx_v_value = (__pyx_v_value | ((__pyx_v_byte & 0x7f) << __pyx_v_counter));

    /* "cprotobuf/utils.pxi":64
 *         byte = pointer[0]
 *         value |= (byte & 0x7f) << counter
 *         counter+=7             # <<<<<<<<<<<<<<
 *         pointer+=1
 *         if byte & 0x80 == 0:
 */
    __pyx_v_counter = (__pyx_v_counter + 7);

    /* "cprotobuf/utils.pxi":65
 *         value |= (byte & 0x7f) << counter
 *         counter+=7
 *         pointer+=1             # <<<<<<<<<<<<<<
 *         if byte & 0x80 == 0:
 *             break
 */
    __pyx_v_pointer = (__pyx_v_pointer + 1);

    /* "cprotobuf/utils.pxi":66
 *         counter+=7
 *         pointer+=1
 *         if byte & 0x80 == 0:             # <<<<<<<<<<<<<<
 *             break
 *     start[0] = pointer
 */
    __pyx_t_1 = (((__pyx_v_byte & 0x80) == 0) != 0);
    if (__pyx_t_1) {

      /* "cprotobuf/utils.pxi":67
 *         pointer+=1
 *         if byte & 0x80 == 0:
 *             break             # <<<<<<<<<<<<<<
 *     start[0] = pointer
 *     result[0] = value
 */
      goto __pyx_L4_break;

      /* "cprotobuf/utils.pxi":66
 *         counter+=7
 *         pointer+=1
 *         if byte & 0x80 == 0:             # <<<<<<<<<<<<<<
 *             break
 *     start[0] = pointer
 */
    }
  }
  __pyx_L4_break:;

  /* "cprotobuf/utils.pxi":68
 *         if byte & 0x80 == 0:
 *             break
 *     start[0] = pointer             # <<<<<<<<<<<<<<
 *     result[0] = value
 *     return 0
 */
  (__pyx_v_start[0]) = __pyx_v_pointer;

  /* "cprotobuf/utils.pxi":69
 *             break
 *     start[0] = pointer
 *     result[0] = value             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  (__pyx_v_result[0]) = __pyx_v_value;

  /* "cprotobuf/utils.pxi":70
 *     start[0] = pointer
 *     result[0] = value
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline int raw_decode_fixed32(char **pointer, char *end, uint32_t *result) nogil:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":53
 *     return 0
 * 
 * cdef inline int raw_decode_uint64(char **start, char *end, uint64_t *result) nogil:             # <<<<<<<<<<<<<<
 *     cdef uint64_t value = 0
 *     cdef uint64_t byte
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":72
 *     return 0
 * 
 * cdef inline int raw_decode_fixed32(char **pointer, char *end, uint32_t *result) nogil:             # <<<<<<<<<<<<<<
 *     cdef uint32_t value = 0
 *     cdef char *start = pointer[0]
 */

static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_decode_fixed32(char **__pyx_v_pointer, char *__pyx_v_end, uint32_t *__pyx_v_result) {
  uint32_t __pyx_v_value;
  char *__pyx_v_start;
  int __pyx_v_i;
  int __pyx_r;
  int __pyx_t_1;

  /* "cprotobuf/utils.pxi":73
 * 
 * cdef inline int raw_decode_fixed32(char **pointer, char *end, uint32_t *result) nogil:
 *     cdef uint32_t value = 0             # <<<<<<<<<<<<<<
 *     cdef char *start = pointer[0]
 *     cdef int i
 */
  __pyx_v_value = 0;

  /* "cprotobuf/utils.pxi":74
 * cdef inline int raw_decode_fixed32(char **pointer, char *end, uint32_t *result) nogil:
 *     cdef uint32_t value = 0
 *     cdef char *start = pointer[0]             # <<<<<<<<<<<<<<
 *     cdef int i
 * 
 */
  __pyx_v_start = (__pyx_v_pointer[0]);

  /* "cprotobuf/utils.pxi":77
 *     cdef int i
 * 
 *     for i from 0 <= i < 4:             # <<<<<<<<<<<<<<
 *         if start == end:
 *             return -1
 */
  for (__pyx_v_i = 0; __pyx_v_i < 4; __pyx_v_i++) {

    /* "cprotobuf/utils.pxi":78
 * 
 *     for i from 0 <= i < 4:
 *         if start == end:             # <<<<<<<<<<<<<<
 *             return -1
 *         value |= <unsigned char>start[0] << (i * 8)
 */
    __pyx_t_1 = ((__pyx_v_start == __pyx_v_end) != 0);
    if (__pyx_t_1) {

      /* "cprotobuf/utils.pxi":79
 *     for i from 0 <= i < 4:
 *         if start == end:
 *             return -1             # <<<<<<<<<<<<<<
 *         value |= <unsigned char>start[0] << (i * 8)
 *         start += 1
 */
      __pyx_r = -1;
      goto __pyx_L0;

      /* "cprotobuf/utils.pxi":78
 * 
 *     for i from 0 <= i < 4:
 *         if start == end:             # <<<<<<<<<<<<<<
 *             return -1
 *         value |= <unsigned char>start[0] << (i * 8)
 */
    }

    /* "cprotobuf/utils.pxi":80
 *         if start == end:
 *             return -1
 *         value |= <unsigned char>start[0] << (i * 8)             # <<<<<<<<<<<<<<
 *         start += 1
 *     pointer[0] = start
 */
    __pyx_v_value = (__pyx_v_value | (((unsigned char)(__pyx_v_start[0])) << (__pyx_v_i * 8)));

    /* "cprotobuf/utils.pxi":81
 *             return -1
 *         value |= <unsigned char>start[0] << (i * 8)
 *         start += 1             # <<<<<<<<<<<<<<
 *     pointer[0] = start
 *     result[0] = value
 */
    __pyx_v_start = (__pyx_v_start + 1);
  }

  /* "cprotobuf/utils.pxi":82
 *         value |= <unsigned char>start[0] << (i * 8)
 *         start += 1
 *     pointer[0] = start             # <<<<<<<<<<<<<<
 *     result[0] = value
 *     return 0
 */
  (__pyx_v_pointer[0]) = __pyx_v_start;

  /* "cprotobuf/utils.pxi":83
 *         start += 1
 *     pointer[0] = start
 *     result[0] = value             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  (__pyx_v_result[0]) = __pyx_v_value;

  /* "cprotobuf/utils.pxi":84
 *     pointer[0] = start
 *     result[0] = value
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline int raw_decode_fixed64(char **pointer, char *end, uint64_t *result) nogil:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":72
 *     return 0
 * 
 * cdef inline int raw_decode_fixed32(char **pointer, char *end, uint32_t *result) nogil:             # <<<<<<<<<<<<<<
 *     cdef uint32_t value = 0
 *     cdef char *start = pointer[0]
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":86
 *     return 0
 * 
 * cdef inline int raw_decode_fixed64(char **pointer, char *end, uint64_t *result) nogil:             # <<<<<<<<<<<<<<
 *     cdef uint64_t value = 0
 *     cdef char *start = pointer[0]
 */

static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_decode_fixed64(char **__pyx_v_pointer, char *__pyx_v_end, uint64_t *__pyx_v_result) {
  uint64_t __pyx_v_value;
  char *__pyx_v_start;
  uint64_t __pyx_v_temp;
  int __pyx_v_i;
  int __pyx_r;
  int __pyx_t_1;

  /* "cprotobuf/utils.pxi":87
 * 
 * cdef inline int raw_decode_fixed64(char **pointer, char *end, uint64_t *result) nogil:
 *     cdef uint64_t value = 0             # <<<<<<<<<<<<<<
 *     cdef char *start = pointer[0]
 *     cdef uint64_t temp = 0
 */
  __pyx_v_value = 0;

  /* "cprotobuf/utils.pxi":88
 * cdef inline int raw_decode_fixed64(char **pointer, char *end, uint64_t *result) nogil:
 *     cdef uint64_t value = 0
 *     cdef char *start = pointer[0]             # <<<<<<<<<<<<<<
 *     cdef uint64_t temp = 0
 *     cdef int i
 */
  __pyx_v_start = (__pyx_v_pointer[0]);

  /* "cprotobuf/utils.pxi":89
 *     cdef uint64_t value = 0
 *     cdef char *start = pointer[0]
 *     cdef uint64_t temp = 0             # <<<<<<<<<<<<<<
 *     cdef int i
 *     for i from 0 <= i < 8:
 */
  __pyx_v_temp = 0;

  /* "cprotobuf/utils.pxi":91
 *     cdef uint64_t temp = 0
 *     cdef int i
 *     for i from 0 <= i < 8:             # <<<<<<<<<<<<<<
 *         if start == end:
 *             return -1
 */
  for (__pyx_v_i = 0; __pyx_v_i < 8; __pyx_v_i++) {

    /* "cprotobuf/utils.pxi":92
 *     cdef int i
 *     for i from 0 <= i < 8:
 *         if start == end:             # <<<<<<<<<<<<<<
 *             return -1
 *         temp = <unsigned char>start[0]
 */
    __pyx_t_1 = ((__pyx_v_start == __pyx_v_end) != 0);
    if (__pyx_t_1) {

      /* "cprotobuf/utils.pxi":93
 *     for i from 0 <= i < 8:
 *         if start == end:
 *             return -1             # <<<<<<<<<<<<<<
 *         temp = <unsigned char>start[0]
 *         value |= temp << (i * 8)
 */
      __pyx_r = -1;
      goto __pyx_L0;

      /* "cprotobuf/utils.pxi":92
 *     cdef int i
 *     for i from 0 <= i < 8:
 *         if start == end:             # <<<<<<<<<<<<<<
 *             return -1
 *         temp = <unsigned char>start[0]
 */
    }

    /* "cprotobuf/utils.pxi":94
 *         if start == end:
 *             return -1
 *         temp = <unsigned char>start[0]             # <<<<<<<<<<<<<<
 *         value |= temp << (i * 8)
 *         start += 1
 */
    __pyx_v_temp = ((unsigned char)(__pyx_v_start[0]));

    /* "cprotobuf/utils.pxi":95
 *             return -1
 *         temp = <unsigned char>start[0]
 *         value |= temp << (i * 8)             # <<<<<<<<<<<<<<
 *         start += 1
 *     pointer[0] = start
 */
    __pyx_v_value = (__pyx_v_value | (__pyx_v_temp << (__pyx_v_i * 8)));

    /* "cprotobuf/utils.pxi":96
 *         temp = <unsigned char>start[0]
 *         value |= temp << (i * 8)
 *         start += 1             # <<<<<<<<<<<<<<
 *     pointer[0] = start
 *     result[0] = value
 */
    __pyx_v_start = (__pyx_v_start + 1);
  }

  /* "cprotobuf/utils.pxi":97
 *         value |= temp << (i * 8)
 *         start += 1
 *     pointer[0] = start             # <<<<<<<<<<<<<<
 *     result[0] = value
 *     return 0
 */
  (__pyx_v_pointer[0]) = __pyx_v_start;

  /* "cprotobuf/utils.pxi":98
 *         start += 1
 *     pointer[0] = start
 *     result[0] = value             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  (__pyx_v_result[0]) = __pyx_v_value;

  /* "cprotobuf/utils.pxi":99
 *     pointer[0] = start
 *     result[0] = value
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline int raw_decode_delimited(char **pointer, char *end, char **result, uint64_t *size) nogil:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":86
 *     return 0
 * 
 * cdef inline int raw_decode_fixed64(char **pointer, char *end, uint64_t *result) nogil:             # <<<<<<<<<<<<<<
 *     cdef uint64_t value = 0
 *     cdef char *start = pointer[0]
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":101
 *     return 0
 * 
 * cdef inline int raw_decode_delimited(char **pointer, char *end, char **result, uint64_t *size) nogil:             # <<<<<<<<<<<<<<
 *     if raw_decode_uint64(pointer, end, size):
 *         return -1
 */

static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_decode_delimited(char **__pyx_v_pointer, char *__pyx_v_end, char **__pyx_v_result, uint64_t *__pyx_v_size) {
  char *__pyx_v_start;
  int __pyx_r;
  int __pyx_t_1;

  /* "cprotobuf/utils.pxi":102
 * 
 * cdef inline int raw_decode_delimited(char **pointer, char *end, char **result, uint64_t *size) nogil:
 *     if raw_decode_uint64(pointer, end, size):             # <<<<<<<<<<<<<<
 *         return -1
 * 
 */
  __pyx_t_1 = (__pyx_f_9cprotobuf_8internal_raw_decode_uint64(__pyx_v_pointer, __pyx_v_end, __pyx_v_size) != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/utils.pxi":103
 * cdef inline int raw_decode_delimited(char **pointer, char *end, char **result, uint64_t *size) nogil:
 *     if raw_decode_uint64(pointer, end, size):
 *         return -1             # <<<<<<<<<<<<<<
 * 
 *     cdef char* start = pointer[0]
 */
    __pyx_r = -1;
    goto __pyx_L0;

    /* "cprotobuf/utils.pxi":102
 * 
 * cdef inline int raw_decode_delimited(char **pointer, char *end, char **result, uint64_t *size) nogil:
 *     if raw_decode_uint64(pointer, end, size):             # <<<<<<<<<<<<<<
 *         return -1
 * 
 */
  }

  /* "cprotobuf/utils.pxi":105
 *         return -1
 * 
 *     cdef char* start = pointer[0]             # <<<<<<<<<<<<<<
 *     if start+size[0] > end:
 *         return -2
 */
  __pyx_v_start = (__pyx_v_pointer[0]);

  /* "cprotobuf/utils.pxi":106
 * 
 *     cdef char* start = pointer[0]
 *     if start+size[0] > end:             # <<<<<<<<<<<<<<
 *         return -2
 * 
 */
  __pyx_t_1 = (((__pyx_v_start + (__pyx_v_size[0])) > __pyx_v_end) != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/utils.pxi":107
 *     cdef char* start = pointer[0]
 *     if start+size[0] > end:
 *         return -2             # <<<<<<<<<<<<<<
 * 
 *     result[0] = start
 */
    __pyx_r = -2;
    goto __pyx_L0;

    /* "cprotobuf/utils.pxi":106
 * 
 *     cdef char* start = pointer[0]
 *     if start+size[0] > end:             # <<<<<<<<<<<<<<
 *         return -2
 * 
 */
  }

  /* "cprotobuf/utils.pxi":109
 *         return -2
 * 
 *     result[0] = start             # <<<<<<<<<<<<<<
 *     pointer[0] = start+size[0]
 *     return 0
 */
  (__pyx_v_result[0]) = __pyx_v_start;

  /* "cprotobuf/utils.pxi":110
 * 
 *     result[0] = start
 *     pointer[0] = start+size[0]             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  (__pyx_v_pointer[0]) = (__pyx_v_start + (__pyx_v_size[0]));

  /* "cprotobuf/utils.pxi":111
 *     result[0] = start
 *     pointer[0] = start+size[0]
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline int skip_unknown_field(char **pointer, char *end, int wtype) nogil:
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":101
 *     return 0
 * 
 * cdef inline int raw_decode_delimited(char **pointer, char *end, char **result, uint64_t *size) nogil:             # <<<<<<<<<<<<<<
 *     if raw_decode_uint64(pointer, end, size):
 *         return -1
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":113
 *     return 0
 * 
 * cdef inline int skip_unknown_field(char **pointer, char *end, int wtype) nogil:             # <<<<<<<<<<<<<<
 *     cdef uint32_t size
 *     cdef char* start
 */

static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_skip_unknown_field(char **__pyx_v_pointer, char *__pyx_v_end, int __pyx_v_wtype) {
  uint32_t __pyx_v_size;
  char *__pyx_v_start;
  int __pyx_r;
  int __pyx_t_1;
  long __pyx_t_2;

  /* "cprotobuf/utils.pxi":116
 *     cdef uint32_t size
 *     cdef char* start
 *     if wtype == 0:             # <<<<<<<<<<<<<<
 *         start = pointer[0]
 *         while True:
 */
  switch (__pyx_v_wtype) {
    case 0:

    /* "cprotobuf/utils.pxi":117
 *     cdef char* start
 *     if wtype == 0:
 *         start = pointer[0]             # <<<<<<<<<<<<<<
 *         while True:
 *             if start >= end:
 */
    __pyx_v_start = (__pyx_v_pointer[0]);

    /* "cprotobuf/utils.pxi":118
 *     if wtype == 0:
 *         start = pointer[0]
 *         while True:             # <<<<<<<<<<<<<<
 *             if start >= end:
 *                 return -1
 */
    while (1) {

      /* "cprotobuf/utils.pxi":119
 *         start = pointer[0]
 *         while True:
 *             if start >= end:             # <<<<<<<<<<<<<<
 *                 return -1
 *             if start[0] & 0x80 == 0:
 */
      __pyx_t_1 = ((__pyx_v_start >= __pyx_v_end) != 0);
      if (__pyx_t_1) {

        /* "cprotobuf/utils.pxi":120
 *         while True:
 *             if start >= end:
 *                 return -1             # <<<<<<<<<<<<<<
 *             if start[0] & 0x80 == 0:
 *                 break
 */
        __pyx_r = -1;
        goto __pyx_L0;

        /* "cprotobuf/utils.pxi":119
 *         start = pointer[0]
 *         while True:
 *             if start >= end:             # <<<<<<<<<<<<<<
 *                 return -1
 *             if start[0] & 0x80 == 0:
 */
      }

      /* "cprotobuf/utils.pxi":121
 *             if start >= end:
 *                 return -1
 *             if start[0] & 0x80 == 0:             # <<<<<<<<<<<<<<
 *                 break
 *             start += 1
 */
      __pyx_t_1 = ((((__pyx_v_start[0]) & 0x80) == 0) != 0);
      if (__pyx_t_1) {

        /* "cprotobuf/utils.pxi":122
 *                 return -1
 *             if start[0] & 0x80 == 0:
 *                 break             # <<<<<<<<<<<<<<
 *             start += 1
 *         pointer[0] = start + 1
 */
        goto __pyx_L4_break;

        /* "cprotobuf/utils.pxi":121
 *             if start >= end:
 *                 return -1
 *             if start[0] & 0x80 == 0:             # <<<<<<<<<<<<<<
 *                 break
 *             start += 1
 */
      }

      /* "cprotobuf/utils.pxi":123
 *             if start[0] & 0x80 == 0:
 *                 break
 *             start += 1             # <<<<<<<<<<<<<<
 *         pointer[0] = start + 1
 *     elif wtype == 1:
 */
      __pyx_v_start = (__pyx_v_start + 1);
    }
    __pyx_L4_break:;

    /* "cprotobuf/utils.pxi":124
 *                 break
 *             start += 1
 *         pointer[0] = start + 1             # <<<<<<<<<<<<<<
 *     elif wtype == 1:
 *         pointer[0] += 8
 */
    (__pyx_v_pointer[0]) = (__pyx_v_start + 1);

    /* "cprotobuf/utils.pxi":116
 *     cdef uint32_t size
 *     cdef char* start
 *     if wtype == 0:             # <<<<<<<<<<<<<<
 *         start = pointer[0]
 *         while True:
 */
    break;

    /* "cprotobuf/utils.pxi":125
 *             start += 1
 *         pointer[0] = start + 1
 *     elif wtype == 1:             # <<<<<<<<<<<<<<
 *         pointer[0] += 8
 *     elif wtype == 2:
 */
    case 1:

    /* "cprotobuf/utils.pxi":126
 *         pointer[0] = start + 1
 *     elif wtype == 1:
 *         pointer[0] += 8             # <<<<<<<<<<<<<<
 *     elif wtype == 2:
 *         if raw_decode_uint32(pointer, end, &size):
 */
    __pyx_t_2 = 0;
    (__pyx_v_pointer[__pyx_t_2]) = ((__pyx_v_pointer[__pyx_t_2]) + 8);

    /* "cprotobuf/utils.pxi":125
 *             start += 1
 *         pointer[0] = start + 1
 *     elif wtype == 1:             # <<<<<<<<<<<<<<
 *         pointer[0] += 8
 *     elif wtype == 2:
 */
    break;

    /* "cprotobuf/utils.pxi":127
 *     elif wtype == 1:
 *         pointer[0] += 8
 *     elif wtype == 2:             # <<<<<<<<<<<<<<
 *         if raw_decode_uint32(pointer, end, &size):
 *             return -1
 */
    case 2:

    /* "cprotobuf/utils.pxi":128
 *         pointer[0] += 8
 *     elif wtype == 2:
 *         if raw_decode_uint32(pointer, end, &size):             # <<<<<<<<<<<<<<
 *             return -1
 *         if pointer[0]+size > end:
 */
    __pyx_t_1 = (__pyx_f_9cprotobuf_8internal_raw_decode_uint32(__pyx_v_pointer, __pyx_v_end, (&__pyx_v_size)) != 0);
    if (__pyx_t_1) {

      /* "cprotobuf/utils.pxi":129
 *     elif wtype == 2:
 *         if raw_decode_uint32(pointer, end, &size):
 *             return -1             # <<<<<<<<<<<<<<
 *         if pointer[0]+size > end:
 *             return -1
 */
      __pyx_r = -1;
      goto __pyx_L0;

      /* "cprotobuf/utils.pxi":128
 *         pointer[0] += 8
 *     elif wtype == 2:
 *         if raw_decode_uint32(pointer, end, &size):             # <<<<<<<<<<<<<<
 *             return -1
 *         if pointer[0]+size > end:
 */
    }

    /* "cprotobuf/utils.pxi":130
 *         if raw_decode_uint32(pointer, end, &size):
 *             return -1
 *         if pointer[0]+size > end:             # <<<<<<<<<<<<<<
 *             return -1
 *         pointer[0] += size
 */
    __pyx_t_1 = ((((__pyx_v_pointer[0]) + __pyx_v_size) > __pyx_v_end) != 0);
    if (__pyx_t_1) {

      /* "cprotobuf/utils.pxi":131
 *             return -1
 *         if pointer[0]+size > end:
 *             return -1             # <<<<<<<<<<<<<<
 *         pointer[0] += size
 *     elif wtype == 5:
 */
      __pyx_r = -1;
      goto __pyx_L0;

      /* "cprotobuf/utils.pxi":130
 *         if raw_decode_uint32(pointer, end, &size):
 *             return -1
 *         if pointer[0]+size > end:             # <<<<<<<<<<<<<<
 *             return -1
 *         pointer[0] += size
 */
    }

    /* "cprotobuf/utils.pxi":132
 *         if pointer[0]+size > end:
 *             return -1
 *         pointer[0] += size             # <<<<<<<<<<<<<<
 *     elif wtype == 5:
 *         pointer[0] += 4
 */
    __pyx_t_2 = 0;
    (__pyx_v_pointer[__pyx_t_2]) = ((__pyx_v_pointer[__pyx_t_2]) + __pyx_v_size);

    /* "cprotobuf/utils.pxi":127
 *     elif wtype == 1:
 *         pointer[0] += 8
 *     elif wtype == 2:             # <<<<<<<<<<<<<<
 *         if raw_decode_uint32(pointer, end, &size):
 *             return -1
 */
    break;

    /* "cprotobuf/utils.pxi":133
 *             return -1
 *         pointer[0] += size
 *     elif wtype == 5:             # <<<<<<<<<<<<<<
 *         pointer[0] += 4
 *     else:
 */
    case 5:

    /* "cprotobuf/utils.pxi":134
 *         pointer[0] += size
 *     elif wtype == 5:
 *         pointer[0] += 4             # <<<<<<<<<<<<<<
 *     else:
 *         return -1
 */
    __pyx_t_2 = 0;
    (__pyx_v_pointer[__pyx_t_2]) = ((__pyx_v_pointer[__pyx_t_2]) + 4);

    /* "cprotobuf/utils.pxi":133
 *             return -1
 *         pointer[0] += size
 *     elif wtype == 5:             # <<<<<<<<<<<<<<
 *         pointer[0] += 4
 *     else:
 */
    break;
    default:

    /* "cprotobuf/utils.pxi":136
 *         pointer[0] += 4
 *     else:
 *         return -1             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
    __pyx_r = -1;
    goto __pyx_L0;
    break;
  }

  /* "cprotobuf/utils.pxi":137
 *     else:
 *         return -1
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * # }}}
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":113
 *     return 0
 * 
 * cdef inline int skip_unknown_field(char **pointer, char *end, int wtype) nogil:             # <<<<<<<<<<<<<<
 *     cdef uint32_t size
 *     cdef char* start
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":141
 * # }}}
 * 
 * cdef object decode_uint32(char **pointer, char *end):             # <<<<<<<<<<<<<<
 *     cdef uint32_t result
 *     if raw_decode_uint32(pointer, end, &result):
 */

static PyObject *__pyx_f_9cprotobuf_8internal_decode_uint32(char **__pyx_v_pointer, char *__pyx_v_end) {
  uint32_t __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("decode_uint32", 0);

  /* "cprotobuf/utils.pxi":143
 * cdef object decode_uint32(char **pointer, char *end):
 *     cdef uint32_t result
 *     if raw_decode_uint32(pointer, end, &result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `uint32` at [{0}]")
 * 
 */
  __pyx_t_1 = (__pyx_f_9cprotobuf_8internal_raw_decode_uint32(__pyx_v_pointer, __pyx_v_end, (&__pyx_v_result)) != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/utils.pxi":144
 *     cdef uint32_t result
 *     if raw_decode_uint32(pointer, end, &result):
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `uint32` at [{0}]")             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
    __pyx_t_2 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_decode_value_of_type_uint3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 144, __pyx_L1_error)

    /* "cprotobuf/utils.pxi":143
 * cdef object decode_uint32(char **pointer, char *end):
 *     cdef uint32_t result
 *     if raw_decode_uint32(pointer, end, &result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `uint32` at [{0}]")
 * 
 */
  }

  /* "cprotobuf/utils.pxi":146
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `uint32` at [{0}]")
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * cdef object decode_uint64(char **pointer, char *end):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_uint32_t(__pyx_v_result); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":141
 * # }}}
 * 
 * cdef object decode_uint32(char **pointer, char *end):             # <<<<<<<<<<<<<<
 *     cdef uint32_t result
 *     if raw_decode_uint32(pointer, end, &result):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cprotobuf.internal.decode_uint32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":148
 *     return result
 * 
 * cdef object decode_uint64(char **pointer, char *end):             # <<<<<<<<<<<<<<
 *     cdef uint64_t result
 *     if raw_decode_uint64(pointer, end, &result):
 */

static PyObject *__pyx_f_9cprotobuf_8internal_decode_uint64(char **__pyx_v_pointer, char *__pyx_v_end) {
  uint64_t __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("decode_uint64", 0);

  /* "cprotobuf/utils.pxi":150
 * cdef object decode_uint64(char **pointer, char *end):
 *     cdef uint64_t result
 *     if raw_decode_uint64(pointer, end, &result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `uint64` at [{0}]")
 * 
 */
  __pyx_t_1 = (__pyx_f_9cprotobuf_8internal_raw_decode_uint64(__pyx_v_pointer, __pyx_v_end, (&__pyx_v_result)) != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/utils.pxi":151
 *     cdef uint64_t result
 *     if raw_decode_uint64(pointer, end, &result):
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `uint64` at [{0}]")             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
    __pyx_t_2 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_decode_value_of_type_uint6); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 151, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 151, __pyx_L1_error)

    /* "cprotobuf/utils.pxi":150
 * cdef object decode_uint64(char **pointer, char *end):
 *     cdef uint64_t result
 *     if raw_decode_uint64(pointer, end, &result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `uint64` at [{0}]")
 * 
 */
  }

  /* "cprotobuf/utils.pxi":153
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `uint64` at [{0}]")
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * cdef object decode_int32(char **pointer, char *end, ):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_uint64_t(__pyx_v_result); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":148
 *     return result
 * 
 * cdef object decode_uint64(char **pointer, char *end):             # <<<<<<<<<<<<<<
 *     cdef uint64_t result
 *     if raw_decode_uint64(pointer, end, &result):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cprotobuf.internal.decode_uint64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":155
 *     return result
 * 
 * cdef object decode_int32(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef int32_t result
 *     if raw_decode_uint32(pointer, end, <uint32_t*>&result):
 */

static PyObject *__pyx_f_9cprotobuf_8internal_decode_int32(char **__pyx_v_pointer, char *__pyx_v_end) {
  int32_t __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("decode_int32", 0);

  /* "cprotobuf/utils.pxi":157
 * cdef object decode_int32(char **pointer, char *end, ):
 *     cdef int32_t result
 *     if raw_decode_uint32(pointer, end, <uint32_t*>&result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `int32` at [{0}]")
 * 
 */
  __pyx_t_1 = (__pyx_f_9cprotobuf_8internal_raw_decode_uint32(__pyx_v_pointer, __pyx_v_end, ((uint32_t *)(&__pyx_v_result))) != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/utils.pxi":158
 *     cdef int32_t result
 *     if raw_decode_uint32(pointer, end, <uint32_t*>&result):
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `int32` at [{0}]")             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
    __pyx_t_2 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_decode_value_of_type_int32); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 158, __pyx_L1_error)

    /* "cprotobuf/utils.pxi":157
 * cdef object decode_int32(char **pointer, char *end, ):
 *     cdef int32_t result
 *     if raw_decode_uint32(pointer, end, <uint32_t*>&result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `int32` at [{0}]")
 * 
 */
  }

  /* "cprotobuf/utils.pxi":160
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `int32` at [{0}]")
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * cdef object decode_int64(char **pointer, char *end, ):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int32_t(__pyx_v_result); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":155
 *     return result
 * 
 * cdef object decode_int32(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef int32_t result
 *     if raw_decode_uint32(pointer, end, <uint32_t*>&result):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cprotobuf.internal.decode_int32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":162
 *     return result
 * 
 * cdef object decode_int64(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef int64_t result
 *     if raw_decode_uint64(pointer, end, <uint64_t*>&result):
 */

static PyObject *__pyx_f_9cprotobuf_8internal_decode_int64(char **__pyx_v_pointer, char *__pyx_v_end) {
  int64_t __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("decode_int64", 0);

  /* "cprotobuf/utils.pxi":164
 * cdef object decode_int64(char **pointer, char *end, ):
 *     cdef int64_t result
 *     if raw_decode_uint64(pointer, end, <uint64_t*>&result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `int64` at [{0}]")
 * 
 */
  __pyx_t_1 = (__pyx_f_9cprotobuf_8internal_raw_decode_uint64(__pyx_v_pointer, __pyx_v_end, ((uint64_t *)(&__pyx_v_result))) != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/utils.pxi":165
 *     cdef int64_t result
 *     if raw_decode_uint64(pointer, end, <uint64_t*>&result):
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `int64` at [{0}]")             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
    __pyx_t_2 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_decode_value_of_type_int64); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 165, __pyx_L1_error)

    /* "cprotobuf/utils.pxi":164
 * cdef object decode_int64(char **pointer, char *end, ):
 *     cdef int64_t result
 *     if raw_decode_uint64(pointer, end, <uint64_t*>&result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `int64` at [{0}]")
 * 
 */
  }

  /* "cprotobuf/utils.pxi":167
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `int64` at [{0}]")
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * cdef object decode_sint32(char **pointer, char *end, ):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int64_t(__pyx_v_result); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":162
 *     return result
 * 
 * cdef object decode_int64(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef int64_t result
 *     if raw_decode_uint64(pointer, end, <uint64_t*>&result):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cprotobuf.internal.decode_int64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":169
 *     return result
 * 
 * cdef object decode_sint32(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef uint32_t result
 *     if raw_decode_uint32(pointer, end, &result):
 */

static PyObject *__pyx_f_9cprotobuf_8internal_decode_sint32(char **__pyx_v_pointer, char *__pyx_v_end) {
  uint32_t __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("decode_sint32", 0);

  /* "cprotobuf/utils.pxi":171
 * cdef object decode_sint32(char **pointer, char *end, ):
 *     cdef uint32_t result
 *     if raw_decode_uint32(pointer, end, &result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `sint32` at [{0}]")
 * 
 */
  __pyx_t_1 = (__pyx_f_9cprotobuf_8internal_raw_decode_uint32(__pyx_v_pointer, __pyx_v_end, (&__pyx_v_result)) != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/utils.pxi":172
 *     cdef uint32_t result
 *     if raw_decode_uint32(pointer, end, &result):
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `sint32` at [{0}]")             # <<<<<<<<<<<<<<
 * 
 *     return <int32_t>((result >> 1) ^ (-<int32_t>(result & 1)))
 */
    __pyx_t_2 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_decode_value_of_type_sint3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 172, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 172, __pyx_L1_error)

    /* "cprotobuf/utils.pxi":171
 * cdef object decode_sint32(char **pointer, char *end, ):
 *     cdef uint32_t result
 *     if raw_decode_uint32(pointer, end, &result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `sint32` at [{0}]")
 * 
 */
  }

  /* "cprotobuf/utils.pxi":174
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `sint32` at [{0}]")
 * 
 *     return <int32_t>((result >> 1) ^ (-<int32_t>(result & 1)))             # <<<<<<<<<<<<<<
 * 
 * cdef object decode_sint64(char **pointer, char *end, ):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int32_t(((int32_t)((__pyx_v_result >> 1) ^ (-((int32_t)(__pyx_v_result & 1)))))); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":169
 *     return result
 * 
 * cdef object decode_sint32(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef uint32_t result
 *     if raw_decode_uint32(pointer, end, &result):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cprotobuf.internal.decode_sint32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":176
 *     return <int32_t>((result >> 1) ^ (-<int32_t>(result & 1)))
 * 
 * cdef object decode_sint64(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef uint64_t un
 *     if raw_decode_uint64(pointer, end, &un):
 */

static PyObject *__pyx_f_9cprotobuf_8internal_decode_sint64(char **__pyx_v_pointer, char *__pyx_v_end) {
  uint64_t __pyx_v_un;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("decode_sint64", 0);

  /* "cprotobuf/utils.pxi":178
 * cdef object decode_sint64(char **pointer, char *end, ):
 *     cdef uint64_t un
 *     if raw_decode_uint64(pointer, end, &un):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `sint64` at [{0}]")
 * 
 */
  __pyx_t_1 = (__pyx_f_9cprotobuf_8internal_raw_decode_uint64(__pyx_v_pointer, __pyx_v_end, (&__pyx_v_un)) != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/utils.pxi":179
 *     cdef uint64_t un
 *     if raw_decode_uint64(pointer, end, &un):
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `sint64` at [{0}]")             # <<<<<<<<<<<<<<
 * 
 *     return <int64_t>((un >> 1) ^ (-<int64_t>(un & 1)))
 */
    __pyx_t_2 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_decode_value_of_type_sint6); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 179, __pyx_L1_error)

    /* "cprotobuf/utils.pxi":178
 * cdef object decode_sint64(char **pointer, char *end, ):
 *     cdef uint64_t un
 *     if raw_decode_uint64(pointer, end, &un):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `sint64` at [{0}]")
 * 
 */
  }

  /* "cprotobuf/utils.pxi":181
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `sint64` at [{0}]")
 * 
 *     return <int64_t>((un >> 1) ^ (-<int64_t>(un & 1)))             # <<<<<<<<<<<<<<
 * 
 * cdef object decode_fixed32(char **pointer, char *end, ):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int64_t(((int64_t)((__pyx_v_un >> 1) ^ (-((int64_t)(__pyx_v_un & 1)))))); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":176
 *     return <int32_t>((result >> 1) ^ (-<int32_t>(result & 1)))
 * 
 * cdef object decode_sint64(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef uint64_t un
 *     if raw_decode_uint64(pointer, end, &un):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cprotobuf.internal.decode_sint64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":183
 *     return <int64_t>((un >> 1) ^ (-<int64_t>(un & 1)))
 * 
 * cdef object decode_fixed32(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef uint32_t result
 *     if raw_decode_fixed32(pointer, end, &result):
 */

static PyObject *__pyx_f_9cprotobuf_8internal_decode_fixed32(char **__pyx_v_pointer, char *__pyx_v_end) {
  uint32_t __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("decode_fixed32", 0);

  /* "cprotobuf/utils.pxi":185
 * cdef object decode_fixed32(char **pointer, char *end, ):
 *     cdef uint32_t result
 *     if raw_decode_fixed32(pointer, end, &result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `fixed32` at [{0}]")
 * 
 */
  __pyx_t_1 = (__pyx_f_9cprotobuf_8internal_raw_decode_fixed32(__pyx_v_pointer, __pyx_v_end, (&__pyx_v_result)) != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/utils.pxi":186
 *     cdef uint32_t result
 *     if raw_decode_fixed32(pointer, end, &result):
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `fixed32` at [{0}]")             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
    __pyx_t_2 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_decode_value_of_type_fixed); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 186, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 186, __pyx_L1_error)

    /* "cprotobuf/utils.pxi":185
 * cdef object decode_fixed32(char **pointer, char *end, ):
 *     cdef uint32_t result
 *     if raw_decode_fixed32(pointer, end, &result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `fixed32` at [{0}]")
 * 
 */
  }

  /* "cprotobuf/utils.pxi":188
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `fixed32` at [{0}]")
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * cdef object decode_fixed64(char **pointer, char *end, ):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_uint32_t(__pyx_v_result); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":183
 *     return <int64_t>((un >> 1) ^ (-<int64_t>(un & 1)))
 * 
 * cdef object decode_fixed32(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef uint32_t result
 *     if raw_decode_fixed32(pointer, end, &result):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cprotobuf.internal.decode_fixed32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":190
 *     return result
 * 
 * cdef object decode_fixed64(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef uint64_t result
 *     if raw_decode_fixed64(pointer, end, &result):
 */

static PyObject *__pyx_f_9cprotobuf_8internal_decode_fixed64(char **__pyx_v_pointer, char *__pyx_v_end) {
  uint64_t __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("decode_fixed64", 0);

  /* "cprotobuf/utils.pxi":192
 * cdef object decode_fixed64(char **pointer, char *end, ):
 *     cdef uint64_t result
 *     if raw_decode_fixed64(pointer, end, &result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `fixed64` at [{0}]")
 * 
 */
  __pyx_t_1 = (__pyx_f_9cprotobuf_8internal_raw_decode_fixed64(__pyx_v_pointer, __pyx_v_end, (&__pyx_v_result)) != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/utils.pxi":193
 *     cdef uint64_t result
 *     if raw_decode_fixed64(pointer, end, &result):
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `fixed64` at [{0}]")             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
    __pyx_t_2 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_decode_value_of_type_fixed_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 193, __pyx_L1_error)

    /* "cprotobuf/utils.pxi":192
 * cdef object decode_fixed64(char **pointer, char *end, ):
 *     cdef uint64_t result
 *     if raw_decode_fixed64(pointer, end, &result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `fixed64` at [{0}]")
 * 
 */
  }

  /* "cprotobuf/utils.pxi":195
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `fixed64` at [{0}]")
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * cdef object decode_sfixed32(char **pointer, char *end, ):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_uint64_t(__pyx_v_result); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":190
 *     return result
 * 
 * cdef object decode_fixed64(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef uint64_t result
 *     if raw_decode_fixed64(pointer, end, &result):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cprotobuf.internal.decode_fixed64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":197
 *     return result
 * 
 * cdef object decode_sfixed32(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef int32_t result
 *     if raw_decode_fixed32(pointer, end, <uint32_t*>&result):
 */

static PyObject *__pyx_f_9cprotobuf_8internal_decode_sfixed32(char **__pyx_v_pointer, char *__pyx_v_end) {
  int32_t __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("decode_sfixed32", 0);

  /* "cprotobuf/utils.pxi":199
 * cdef object decode_sfixed32(char **pointer, char *end, ):
 *     cdef int32_t result
 *     if raw_decode_fixed32(pointer, end, <uint32_t*>&result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `sfixed32` at [{0}]")
 * 
 */
  __pyx_t_1 = (__pyx_f_9cprotobuf_8internal_raw_decode_fixed32(__pyx_v_pointer, __pyx_v_end, ((uint32_t *)(&__pyx_v_result))) != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/utils.pxi":200
 *     cdef int32_t result
 *     if raw_decode_fixed32(pointer, end, <uint32_t*>&result):
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `sfixed32` at [{0}]")             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
    __pyx_t_2 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_decode_value_of_type_sfixe); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 200, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 200, __pyx_L1_error)

    /* "cprotobuf/utils.pxi":199
 * cdef object decode_sfixed32(char **pointer, char *end, ):
 *     cdef int32_t result
 *     if raw_decode_fixed32(pointer, end, <uint32_t*>&result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `sfixed32` at [{0}]")
 * 
 */
  }

  /* "cprotobuf/utils.pxi":202
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `sfixed32` at [{0}]")
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * cdef object decode_sfixed64(char **pointer, char *end, ):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int32_t(__pyx_v_result); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":197
 *     return result
 * 
 * cdef object decode_sfixed32(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef int32_t result
 *     if raw_decode_fixed32(pointer, end, <uint32_t*>&result):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cprotobuf.internal.decode_sfixed32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":204
 *     return result
 * 
 * cdef object decode_sfixed64(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef int64_t result
 *     if raw_decode_fixed64(pointer, end, <uint64_t*>&result):
 */

static PyObject *__pyx_f_9cprotobuf_8internal_decode_sfixed64(char **__pyx_v_pointer, char *__pyx_v_end) {
  int64_t __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("decode_sfixed64", 0);

  /* "cprotobuf/utils.pxi":206
 * cdef object decode_sfixed64(char **pointer, char *end, ):
 *     cdef int64_t result
 *     if raw_decode_fixed64(pointer, end, <uint64_t*>&result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `sfixed64` at [{0}]")
 * 
 */
  __pyx_t_1 = (__pyx_f_9cprotobuf_8internal_raw_decode_fixed64(__pyx_v_pointer, __pyx_v_end, ((uint64_t *)(&__pyx_v_result))) != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/utils.pxi":207
 *     cdef int64_t result
 *     if raw_decode_fixed64(pointer, end, <uint64_t*>&result):
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `sfixed64` at [{0}]")             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
    __pyx_t_2 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_decode_value_of_type_sfixe_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 207, __pyx_L1_error)

    /* "cprotobuf/utils.pxi":206
 * cdef object decode_sfixed64(char **pointer, char *end, ):
 *     cdef int64_t result
 *     if raw_decode_fixed64(pointer, end, <uint64_t*>&result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `sfixed64` at [{0}]")
 * 
 */
  }

  /* "cprotobuf/utils.pxi":209
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `sfixed64` at [{0}]")
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * cdef object decode_bytes(char **pointer, char *end, ):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int64_t(__pyx_v_result); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":204
 *     return result
 * 
 * cdef object decode_sfixed64(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef int64_t result
 *     if raw_decode_fixed64(pointer, end, <uint64_t*>&result):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cprotobuf.internal.decode_sfixed64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":211
 *     return result
 * 
 * cdef object decode_bytes(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef char *result
 *     cdef uint64_t size
 */

static PyObject *__pyx_f_9cprotobuf_8internal_decode_bytes(char **__pyx_v_pointer, char *__pyx_v_end) {
  char *__pyx_v_result;
  uint64_t __pyx_v_size;
  int __pyx_v_ret;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("decode_bytes", 0);

  /* "cprotobuf/utils.pxi":214
 *     cdef char *result
 *     cdef uint64_t size
 *     cdef int ret = raw_decode_delimited(pointer, end, &result, &size)             # <<<<<<<<<<<<<<
 *     if ret==0:
 *         return PyBytes_FromStringAndSize(result, size)
 */
  __pyx_v_ret = __pyx_f_9cprotobuf_8internal_raw_decode_delimited(__pyx_v_pointer, __pyx_v_end, (&__pyx_v_result), (&__pyx_v_size));

  /* "cprotobuf/utils.pxi":215
 *     cdef uint64_t size
 *     cdef int ret = raw_decode_delimited(pointer, end, &result, &size)
 *     if ret==0:             # <<<<<<<<<<<<<<
 *         return PyBytes_FromStringAndSize(result, size)
 * 
 */
  __pyx_t_1 = ((__pyx_v_ret == 0) != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/utils.pxi":216
 *     cdef int ret = raw_decode_delimited(pointer, end, &result, &size)
 *     if ret==0:
 *         return PyBytes_FromStringAndSize(result, size)             # <<<<<<<<<<<<<<
 * 
 *     if ret == -1:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = PyBytes_FromStringAndSize(__pyx_v_result, __pyx_v_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 216, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "cprotobuf/utils.pxi":215
 *     cdef uint64_t size
 *     cdef int ret = raw_decode_delimited(pointer, end, &result, &size)
 *     if ret==0:             # <<<<<<<<<<<<<<
 *         return PyBytes_FromStringAndSize(result, size)
 * 
 */
  }

  /* "cprotobuf/utils.pxi":218
 *         return PyBytes_FromStringAndSize(result, size)
 * 
 *     if ret == -1:             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode size for value of type `bytes` at [{0}]")
 *     elif ret == -2:
 */
  switch (__pyx_v_ret) {
    case -1L:

    /* "cprotobuf/utils.pxi":219
 * 
 *     if ret == -1:
 *         raise makeDecodeError(pointer[0], "Can't decode size for value of type `bytes` at [{0}]")             # <<<<<<<<<<<<<<
 *     elif ret == -2:
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `bytes` of size %d at [{0}]" % size)
 */
    __pyx_t_2 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_decode_size_for_value_of_t); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 219, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 219, __pyx_L1_error)

    /* "cprotobuf/utils.pxi":218
 *         return PyBytes_FromStringAndSize(result, size)
 * 
 *     if ret == -1:             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode size for value of type `bytes` at [{0}]")
 *     elif ret == -2:
 */
    break;

    /* "cprotobuf/utils.pxi":220
 *     if ret == -1:
 *         raise makeDecodeError(pointer[0], "Can't decode size for value of type `bytes` at [{0}]")
 *     elif ret == -2:             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `bytes` of size %d at [{0}]" % size)
 * 
 */
    case -2L:

    /* "cprotobuf/utils.pxi":221
 *         raise makeDecodeError(pointer[0], "Can't decode size for value of type `bytes` at [{0}]")
 *     elif ret == -2:
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `bytes` of size %d at [{0}]" % size)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_2 = __Pyx_PyInt_From_uint64_t(__pyx_v_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 221, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Can_t_decode_value_of_type_bytes, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 221, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 221, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 221, __pyx_L1_error)

    /* "cprotobuf/utils.pxi":220
 *     if ret == -1:
 *         raise makeDecodeError(pointer[0], "Can't decode size for value of type `bytes` at [{0}]")
 *     elif ret == -2:             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `bytes` of size %d at [{0}]" % size)
 * 
 */
    break;
    default: break;
  }

  /* "cprotobuf/utils.pxi":211
 *     return result
 * 
 * cdef object decode_bytes(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef char *result
 *     cdef uint64_t size
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cprotobuf.internal.decode_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":224
 * 
 * 
 * cdef object decode_string(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef char *result
 *     cdef uint64_t size
 */

static PyObject *__pyx_f_9cprotobuf_8internal_decode_string(char **__pyx_v_pointer, char *__pyx_v_end) {
  char *__pyx_v_result;
  uint64_t __pyx_v_size;
  int __pyx_v_ret;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("decode_string", 0);

  /* "cprotobuf/utils.pxi":227
 *     cdef char *result
 *     cdef uint64_t size
 *     cdef int ret = raw_decode_delimited(pointer, end, &result, &size)             # <<<<<<<<<<<<<<
 *     if ret==0:
 *         return PyUnicode_FromStringAndSize(result, size)
 */
  __pyx_v_ret = __pyx_f_9cprotobuf_8internal_raw_decode_delimited(__pyx_v_pointer, __pyx_v_end, (&__pyx_v_result), (&__pyx_v_size));

  /* "cprotobuf/utils.pxi":228
 *     cdef uint64_t size
 *     cdef int ret = raw_decode_delimited(pointer, end, &result, &size)
 *     if ret==0:             # <<<<<<<<<<<<<<
 *         return PyUnicode_FromStringAndSize(result, size)
 * 
 */
  __pyx_t_1 = ((__pyx_v_ret == 0) != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/utils.pxi":229
 *     cdef int ret = raw_decode_delimited(pointer, end, &result, &size)
 *     if ret==0:
 *         return PyUnicode_FromStringAndSize(result, size)             # <<<<<<<<<<<<<<
 * 
 *     if ret == -1:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = PyUnicode_FromStringAndSize(__pyx_v_result, __pyx_v_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "cprotobuf/utils.pxi":228
 *     cdef uint64_t size
 *     cdef int ret = raw_decode_delimited(pointer, end, &result, &size)
 *     if ret==0:             # <<<<<<<<<<<<<<
 *         return PyUnicode_FromStringAndSize(result, size)
 * 
 */
  }

  /* "cprotobuf/utils.pxi":231
 *         return PyUnicode_FromStringAndSize(result, size)
 * 
 *     if ret == -1:             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode size for value of type `string` at [{0}]")
 *     elif ret == -2:
 */
  switch (__pyx_v_ret) {
    case -1L:

    /* "cprotobuf/utils.pxi":232
 * 
 *     if ret == -1:
 *         raise makeDecodeError(pointer[0], "Can't decode size for value of type `string` at [{0}]")             # <<<<<<<<<<<<<<
 *     elif ret == -2:
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `string` of size %d at [{0}]" % size)
 */
    __pyx_t_2 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_decode_size_for_value_of_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 232, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 232, __pyx_L1_error)

    /* "cprotobuf/utils.pxi":231
 *         return PyUnicode_FromStringAndSize(result, size)
 * 
 *     if ret == -1:             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode size for value of type `string` at [{0}]")
 *     elif ret == -2:
 */
    break;

    /* "cprotobuf/utils.pxi":233
 *     if ret == -1:
 *         raise makeDecodeError(pointer[0], "Can't decode size for value of type `string` at [{0}]")
 *     elif ret == -2:             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `string` of size %d at [{0}]" % size)
 * 
 */
    case -2L:

    /* "cprotobuf/utils.pxi":234
 *         raise makeDecodeError(pointer[0], "Can't decode size for value of type `string` at [{0}]")
 *     elif ret == -2:
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `string` of size %d at [{0}]" % size)             # <<<<<<<<<<<<<<
 * 
 * cdef object decode_float(char **pointer, char *end, ):
 */
    __pyx_t_2 = __Pyx_PyInt_From_uint64_t(__pyx_v_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 234, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Can_t_decode_value_of_type_strin, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 234, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 234, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 234, __pyx_L1_error)

    /* "cprotobuf/utils.pxi":233
 *     if ret == -1:
 *         raise makeDecodeError(pointer[0], "Can't decode size for value of type `string` at [{0}]")
 *     elif ret == -2:             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `string` of size %d at [{0}]" % size)
 * 
 */
    break;
    default: break;
  }

  /* "cprotobuf/utils.pxi":224
 * 
 * 
 * cdef object decode_string(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef char *result
 *     cdef uint64_t size
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cprotobuf.internal.decode_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":236
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `string` of size %d at [{0}]" % size)
 * 
 * cdef object decode_float(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef float result
 *     if raw_decode_fixed32(pointer, end, <uint32_t*>&result):
 */

static PyObject *__pyx_f_9cprotobuf_8internal_decode_float(char **__pyx_v_pointer, char *__pyx_v_end) {
  float __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("decode_float", 0);

  /* "cprotobuf/utils.pxi":238
 * cdef object decode_float(char **pointer, char *end, ):
 *     cdef float result
 *     if raw_decode_fixed32(pointer, end, <uint32_t*>&result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `float` at [{0}]")
 * 
 */
  __pyx_t_1 = (__pyx_f_9cprotobuf_8internal_raw_decode_fixed32(__pyx_v_pointer, __pyx_v_end, ((uint32_t *)(&__pyx_v_result))) != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/utils.pxi":239
 *     cdef float result
 *     if raw_decode_fixed32(pointer, end, <uint32_t*>&result):
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `float` at [{0}]")             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
    __pyx_t_2 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_decode_value_of_type_float); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 239, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 239, __pyx_L1_error)

    /* "cprotobuf/utils.pxi":238
 * cdef object decode_float(char **pointer, char *end, ):
 *     cdef float result
 *     if raw_decode_fixed32(pointer, end, <uint32_t*>&result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `float` at [{0}]")
 * 
 */
  }

  /* "cprotobuf/utils.pxi":241
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `float` at [{0}]")
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * cdef object decode_double(char **pointer, char *end, ):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_result); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":236
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `string` of size %d at [{0}]" % size)
 * 
 * cdef object decode_float(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef float result
 *     if raw_decode_fixed32(pointer, end, <uint32_t*>&result):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cprotobuf.internal.decode_float", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":243
 *     return result
 * 
 * cdef object decode_double(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef double result
 *     if raw_decode_fixed64(pointer, end, <uint64_t*>&result):
 */

static PyObject *__pyx_f_9cprotobuf_8internal_decode_double(char **__pyx_v_pointer, char *__pyx_v_end) {
  double __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("decode_double", 0);

  /* "cprotobuf/utils.pxi":245
 * cdef object decode_double(char **pointer, char *end, ):
 *     cdef double result
 *     if raw_decode_fixed64(pointer, end, <uint64_t*>&result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `double` at [{0}]")
 * 
 */
  __pyx_t_1 = (__pyx_f_9cprotobuf_8internal_raw_decode_fixed64(__pyx_v_pointer, __pyx_v_end, ((uint64_t *)(&__pyx_v_result))) != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/utils.pxi":246
 *     cdef double result
 *     if raw_decode_fixed64(pointer, end, <uint64_t*>&result):
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `double` at [{0}]")             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
    __pyx_t_2 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_decode_value_of_type_doubl); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 246, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(2, 246, __pyx_L1_error)

    /* "cprotobuf/utils.pxi":245
 * cdef object decode_double(char **pointer, char *end, ):
 *     cdef double result
 *     if raw_decode_fixed64(pointer, end, <uint64_t*>&result):             # <<<<<<<<<<<<<<
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `double` at [{0}]")
 * 
 */
  }

  /* "cprotobuf/utils.pxi":248
 *         raise makeDecodeError(pointer[0], "Can't decode value of type `double` at [{0}]")
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * cdef object decode_bool(char **pointer, char *end, ):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_result); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":243
 *     return result
 * 
 * cdef object decode_double(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef double result
 *     if raw_decode_fixed64(pointer, end, <uint64_t*>&result):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cprotobuf.internal.decode_double", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":250
 *     return result
 * 
 * cdef object decode_bool(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef char* start = pointer[0]
 *     pointer[0] = start + 1
 */

static PyObject *__pyx_f_9cprotobuf_8internal_decode_bool(char **__pyx_v_pointer, CYTHON_UNUSED char *__pyx_v_end) {
  char *__pyx_v_start;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("decode_bool", 0);

  /* "cprotobuf/utils.pxi":251
 * 
 * cdef object decode_bool(char **pointer, char *end, ):
 *     cdef char* start = pointer[0]             # <<<<<<<<<<<<<<
 *     pointer[0] = start + 1
 * 
 */
  __pyx_v_start = (__pyx_v_pointer[0]);

  /* "cprotobuf/utils.pxi":252
 * cdef object decode_bool(char **pointer, char *end, ):
 *     cdef char* start = pointer[0]
 *     pointer[0] = start + 1             # <<<<<<<<<<<<<<
 * 
 *     return <bint>start[0]
 */
  (__pyx_v_pointer[0]) = (__pyx_v_start + 1);

  /* "cprotobuf/utils.pxi":254
 *     pointer[0] = start + 1
 * 
 *     return <bint>start[0]             # <<<<<<<<<<<<<<
 * 
 * # }}}
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(((__pyx_v_start[0]) != 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":250
 *     return result
 * 
 * cdef object decode_bool(char **pointer, char *end, ):             # <<<<<<<<<<<<<<
 *     cdef char* start = pointer[0]
 *     pointer[0] = start + 1
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cprotobuf.internal.decode_bool", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":260
 * # {{{ encoding
 * 
 * cdef inline int raw_encode_uint32(bytearray array, uint32_t n) except -1:             # <<<<<<<<<<<<<<
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 */

static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_encode_uint32(PyObject *__pyx_v_array, uint32_t __pyx_v_n) {
  unsigned short __pyx_v_rem;
  Py_ssize_t __pyx_v_size;
  char *__pyx_v_buff;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("raw_encode_uint32", 0);

  /* "cprotobuf/utils.pxi":262
 * cdef inline int raw_encode_uint32(bytearray array, uint32_t n) except -1:
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)             # <<<<<<<<<<<<<<
 *     PyByteArray_Resize(array, size + 10)
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 */
  __pyx_v_size = PyByteArray_GET_SIZE(__pyx_v_array);

  /* "cprotobuf/utils.pxi":263
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 *     PyByteArray_Resize(array, size + 10)             # <<<<<<<<<<<<<<
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 * 
 */
  __pyx_t_1 = PyByteArray_Resize(__pyx_v_array, (__pyx_v_size + 10)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 263, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":264
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 *     PyByteArray_Resize(array, size + 10)
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size             # <<<<<<<<<<<<<<
 * 
 *     if 0!=n:
 */
  __pyx_v_buff = (PyByteArray_AS_STRING(__pyx_v_array) + __pyx_v_size);

  /* "cprotobuf/utils.pxi":266
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 * 
 *     if 0!=n:             # <<<<<<<<<<<<<<
 *         while True:
 *             rem = <char>(n & 0x7f)
 */
  __pyx_t_2 = ((0 != __pyx_v_n) != 0);
  if (__pyx_t_2) {

    /* "cprotobuf/utils.pxi":267
 * 
 *     if 0!=n:
 *         while True:             # <<<<<<<<<<<<<<
 *             rem = <char>(n & 0x7f)
 *             n = n>>7
 */
    while (1) {

      /* "cprotobuf/utils.pxi":268
 *     if 0!=n:
 *         while True:
 *             rem = <char>(n & 0x7f)             # <<<<<<<<<<<<<<
 *             n = n>>7
 *             if 0==n:
 */
      __pyx_v_rem = ((char)(__pyx_v_n & 0x7f));

      /* "cprotobuf/utils.pxi":269
 *         while True:
 *             rem = <char>(n & 0x7f)
 *             n = n>>7             # <<<<<<<<<<<<<<
 *             if 0==n:
 *                 buff[0] = <char> rem
 */
      __pyx_v_n = (__pyx_v_n >> 7);

      /* "cprotobuf/utils.pxi":270
 *             rem = <char>(n & 0x7f)
 *             n = n>>7
 *             if 0==n:             # <<<<<<<<<<<<<<
 *                 buff[0] = <char> rem
 *                 buff+=1
 */
      __pyx_t_2 = ((0 == __pyx_v_n) != 0);
      if (__pyx_t_2) {

        /* "cprotobuf/utils.pxi":271
 *             n = n>>7
 *             if 0==n:
 *                 buff[0] = <char> rem             # <<<<<<<<<<<<<<
 *                 buff+=1
 *                 break
 */
        (__pyx_v_buff[0]) = ((char)__pyx_v_rem);

        /* "cprotobuf/utils.pxi":272
 *             if 0==n:
 *                 buff[0] = <char> rem
 *                 buff+=1             # <<<<<<<<<<<<<<
 *                 break
 *             else:
 */
        __pyx_v_buff = (__pyx_v_buff + 1);

        /* "cprotobuf/utils.pxi":273
 *                 buff[0] = <char> rem
 *                 buff+=1
 *                 break             # <<<<<<<<<<<<<<
 *             else:
 *                 rem = rem | 0x80
 */
        goto __pyx_L5_break;

        /* "cprotobuf/utils.pxi":270
 *             rem = <char>(n & 0x7f)
 *             n = n>>7
 *             if 0==n:             # <<<<<<<<<<<<<<
 *                 buff[0] = <char> rem
 *                 buff+=1
 */
      }

      /* "cprotobuf/utils.pxi":275
 *                 break
 *             else:
 *                 rem = rem | 0x80             # <<<<<<<<<<<<<<
 *                 buff[0] = <char> rem
 *                 buff+=1
 */
      /*else*/ {
        __pyx_v_rem = (__pyx_v_rem | 0x80);

        /* "cprotobuf/utils.pxi":276
 *             else:
 *                 rem = rem | 0x80
 *                 buff[0] = <char> rem             # <<<<<<<<<<<<<<
 *                 buff+=1
 *     else:
 */
        (__pyx_v_buff[0]) = ((char)__pyx_v_rem);

        /* "cprotobuf/utils.pxi":277
 *                 rem = rem | 0x80
 *                 buff[0] = <char> rem
 *                 buff+=1             # <<<<<<<<<<<<<<
 *     else:
 *         buff[0] = '\0'
 */
        __pyx_v_buff = (__pyx_v_buff + 1);
      }
    }
    __pyx_L5_break:;

    /* "cprotobuf/utils.pxi":266
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 * 
 *     if 0!=n:             # <<<<<<<<<<<<<<
 *         while True:
 *             rem = <char>(n & 0x7f)
 */
    goto __pyx_L3;
  }

  /* "cprotobuf/utils.pxi":279
 *                 buff+=1
 *     else:
 *         buff[0] = '\0'             # <<<<<<<<<<<<<<
 *         buff+=1
 * 
 */
  /*else*/ {
    (__pyx_v_buff[0]) = '\x00';

    /* "cprotobuf/utils.pxi":280
 *     else:
 *         buff[0] = '\0'
 *         buff+=1             # <<<<<<<<<<<<<<
 * 
 *     PyByteArray_Resize(array, buff - PyByteArray_AS_STRING(array))
 */
    __pyx_v_buff = (__pyx_v_buff + 1);
  }
  __pyx_L3:;

  /* "cprotobuf/utils.pxi":282
 *         buff+=1
 * 
 *     PyByteArray_Resize(array, buff - PyByteArray_AS_STRING(array))             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_1 = PyByteArray_Resize(__pyx_v_array, (__pyx_v_buff - PyByteArray_AS_STRING(__pyx_v_array))); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 282, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":283
 * 
 *     PyByteArray_Resize(array, buff - PyByteArray_AS_STRING(array))
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_uint32(Field _f, bytearray array, object value):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":260
 * # {{{ encoding
 * 
 * cdef inline int raw_encode_uint32(bytearray array, uint32_t n) except -1:             # <<<<<<<<<<<<<<
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.raw_encode_uint32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":285
 *     return 0
 * 
 * cdef inline encode_uint32(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     raw_encode_uint32(array, value)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_uint32(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v__f, PyObject *__pyx_v_array, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  uint32_t __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("encode_uint32", 0);

  /* "cprotobuf/utils.pxi":286
 * 
 * cdef inline encode_uint32(Field _f, bytearray array, object value):
 *     raw_encode_uint32(array, value)             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_int32(Field _f, bytearray array, object value):
 */
  __pyx_t_1 = __Pyx_PyInt_As_uint32_t(__pyx_v_value); if (unlikely((__pyx_t_1 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(2, 286, __pyx_L1_error)
  __pyx_t_2 = __pyx_f_9cprotobuf_8internal_raw_encode_uint32(__pyx_v_array, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 286, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":285
 *     return 0
 * 
 * cdef inline encode_uint32(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     raw_encode_uint32(array, value)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.encode_uint32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":288
 *     raw_encode_uint32(array, value)
 * 
 * cdef inline encode_int32(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef int32_t n = value
 *     raw_encode_uint32(array, <uint32_t>n)
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_int32(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v__f, PyObject *__pyx_v_array, PyObject *__pyx_v_value) {
  int32_t __pyx_v_n;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int32_t __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("encode_int32", 0);

  /* "cprotobuf/utils.pxi":289
 * 
 * cdef inline encode_int32(Field _f, bytearray array, object value):
 *     cdef int32_t n = value             # <<<<<<<<<<<<<<
 *     raw_encode_uint32(array, <uint32_t>n)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_As_int32_t(__pyx_v_value); if (unlikely((__pyx_t_1 == ((int32_t)-1)) && PyErr_Occurred())) __PYX_ERR(2, 289, __pyx_L1_error)
  __pyx_v_n = __pyx_t_1;

  /* "cprotobuf/utils.pxi":290
 * cdef inline encode_int32(Field _f, bytearray array, object value):
 *     cdef int32_t n = value
 *     raw_encode_uint32(array, <uint32_t>n)             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_sint32(Field _f, bytearray array, object value):
 */
  __pyx_t_2 = __pyx_f_9cprotobuf_8internal_raw_encode_uint32(__pyx_v_array, ((uint32_t)__pyx_v_n)); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 290, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":288
 *     raw_encode_uint32(array, value)
 * 
 * cdef inline encode_int32(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef int32_t n = value
 *     raw_encode_uint32(array, <uint32_t>n)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.encode_int32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":292
 *     raw_encode_uint32(array, <uint32_t>n)
 * 
 * cdef inline encode_sint32(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef int32_t n = value
 *     cdef uint32_t un = (n << 1) ^ (n >> 31)
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_sint32(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v__f, PyObject *__pyx_v_array, PyObject *__pyx_v_value) {
  int32_t __pyx_v_n;
  uint32_t __pyx_v_un;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int32_t __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("encode_sint32", 0);

  /* "cprotobuf/utils.pxi":293
 * 
 * cdef inline encode_sint32(Field _f, bytearray array, object value):
 *     cdef int32_t n = value             # <<<<<<<<<<<<<<
 *     cdef uint32_t un = (n << 1) ^ (n >> 31)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_As_int32_t(__pyx_v_value); if (unlikely((__pyx_t_1 == ((int32_t)-1)) && PyErr_Occurred())) __PYX_ERR(2, 293, __pyx_L1_error)
  __pyx_v_n = __pyx_t_1;

  /* "cprotobuf/utils.pxi":294
 * cdef inline encode_sint32(Field _f, bytearray array, object value):
 *     cdef int32_t n = value
 *     cdef uint32_t un = (n << 1) ^ (n >> 31)             # <<<<<<<<<<<<<<
 * 
 *     raw_encode_uint32(array, un)
 */
  __pyx_v_un = ((__pyx_v_n << 1) ^ (__pyx_v_n >> 31));

  /* "cprotobuf/utils.pxi":296
 *     cdef uint32_t un = (n << 1) ^ (n >> 31)
 * 
 *     raw_encode_uint32(array, un)             # <<<<<<<<<<<<<<
 * 
 * cdef inline int raw_encode_uint64(bytearray array, uint64_t n) except -1:
 */
  __pyx_t_2 = __pyx_f_9cprotobuf_8internal_raw_encode_uint32(__pyx_v_array, __pyx_v_un); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 296, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":292
 *     raw_encode_uint32(array, <uint32_t>n)
 * 
 * cdef inline encode_sint32(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef int32_t n = value
 *     cdef uint32_t un = (n << 1) ^ (n >> 31)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.encode_sint32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":298
 *     raw_encode_uint32(array, un)
 * 
 * cdef inline int raw_encode_uint64(bytearray array, uint64_t n) except -1:             # <<<<<<<<<<<<<<
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 */

static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_encode_uint64(PyObject *__pyx_v_array, uint64_t __pyx_v_n) {
  unsigned short __pyx_v_rem;
  Py_ssize_t __pyx_v_size;
  char *__pyx_v_buff;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("raw_encode_uint64", 0);

  /* "cprotobuf/utils.pxi":300
 * cdef inline int raw_encode_uint64(bytearray array, uint64_t n) except -1:
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)             # <<<<<<<<<<<<<<
 *     PyByteArray_Resize(array, size + 20)
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 */
  __pyx_v_size = PyByteArray_GET_SIZE(__pyx_v_array);

  /* "cprotobuf/utils.pxi":301
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 *     PyByteArray_Resize(array, size + 20)             # <<<<<<<<<<<<<<
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 * 
 */
  __pyx_t_1 = PyByteArray_Resize(__pyx_v_array, (__pyx_v_size + 20)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 301, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":302
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 *     PyByteArray_Resize(array, size + 20)
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size             # <<<<<<<<<<<<<<
 * 
 *     if 0!=n:
 */
  __pyx_v_buff = (PyByteArray_AS_STRING(__pyx_v_array) + __pyx_v_size);

  /* "cprotobuf/utils.pxi":304
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 * 
 *     if 0!=n:             # <<<<<<<<<<<<<<
 *         while True:
 *             rem = <char>(n & 0x7f)
 */
  __pyx_t_2 = ((0 != __pyx_v_n) != 0);
  if (__pyx_t_2) {

    /* "cprotobuf/utils.pxi":305
 * 
 *     if 0!=n:
 *         while True:             # <<<<<<<<<<<<<<
 *             rem = <char>(n & 0x7f)
 *             n = n>>7
 */
    while (1) {

      /* "cprotobuf/utils.pxi":306
 *     if 0!=n:
 *         while True:
 *             rem = <char>(n & 0x7f)             # <<<<<<<<<<<<<<
 *             n = n>>7
 *             if 0==n:
 */
      __pyx_v_rem = ((char)(__pyx_v_n & 0x7f));

      /* "cprotobuf/utils.pxi":307
 *         while True:
 *             rem = <char>(n & 0x7f)
 *             n = n>>7             # <<<<<<<<<<<<<<
 *             if 0==n:
 *                 buff[0] = <char> rem
 */
      __pyx_v_n = (__pyx_v_n >> 7);

      /* "cprotobuf/utils.pxi":308
 *             rem = <char>(n & 0x7f)
 *             n = n>>7
 *             if 0==n:             # <<<<<<<<<<<<<<
 *                 buff[0] = <char> rem
 *                 buff+=1
 */
      __pyx_t_2 = ((0 == __pyx_v_n) != 0);
      if (__pyx_t_2) {

        /* "cprotobuf/utils.pxi":309
 *             n = n>>7
 *             if 0==n:
 *                 buff[0] = <char> rem             # <<<<<<<<<<<<<<
 *                 buff+=1
 *                 break
 */
        (__pyx_v_buff[0]) = ((char)__pyx_v_rem);

        /* "cprotobuf/utils.pxi":310
 *             if 0==n:
 *                 buff[0] = <char> rem
 *                 buff+=1             # <<<<<<<<<<<<<<
 *                 break
 *             else:
 */
        __pyx_v_buff = (__pyx_v_buff + 1);

        /* "cprotobuf/utils.pxi":311
 *                 buff[0] = <char> rem
 *                 buff+=1
 *                 break             # <<<<<<<<<<<<<<
 *             else:
 *                 rem = rem | 0x80
 */
        goto __pyx_L5_break;

        /* "cprotobuf/utils.pxi":308
 *             rem = <char>(n & 0x7f)
 *             n = n>>7
 *             if 0==n:             # <<<<<<<<<<<<<<
 *                 buff[0] = <char> rem
 *                 buff+=1
 */
      }

      /* "cprotobuf/utils.pxi":313
 *                 break
 *             else:
 *                 rem = rem | 0x80             # <<<<<<<<<<<<<<
 *                 buff[0] = <char> rem
 *                 buff+=1
 */
      /*else*/ {
        __pyx_v_rem = (__pyx_v_rem | 0x80);

        /* "cprotobuf/utils.pxi":314
 *             else:
 *                 rem = rem | 0x80
 *                 buff[0] = <char> rem             # <<<<<<<<<<<<<<
 *                 buff+=1
 *     else:
 */
        (__pyx_v_buff[0]) = ((char)__pyx_v_rem);

        /* "cprotobuf/utils.pxi":315
 *                 rem = rem | 0x80
 *                 buff[0] = <char> rem
 *                 buff+=1             # <<<<<<<<<<<<<<
 *     else:
 *         buff[0] = '\0'
 */
        __pyx_v_buff = (__pyx_v_buff + 1);
      }
    }
    __pyx_L5_break:;

    /* "cprotobuf/utils.pxi":304
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 * 
 *     if 0!=n:             # <<<<<<<<<<<<<<
 *         while True:
 *             rem = <char>(n & 0x7f)
 */
    goto __pyx_L3;
  }

  /* "cprotobuf/utils.pxi":317
 *                 buff+=1
 *     else:
 *         buff[0] = '\0'             # <<<<<<<<<<<<<<
 *         buff+=1
 *     PyByteArray_Resize(array, buff - PyByteArray_AS_STRING(array))
 */
  /*else*/ {
    (__pyx_v_buff[0]) = '\x00';

    /* "cprotobuf/utils.pxi":318
 *     else:
 *         buff[0] = '\0'
 *         buff+=1             # <<<<<<<<<<<<<<
 *     PyByteArray_Resize(array, buff - PyByteArray_AS_STRING(array))
 *     return 0
 */
    __pyx_v_buff = (__pyx_v_buff + 1);
  }
  __pyx_L3:;

  /* "cprotobuf/utils.pxi":319
 *         buff[0] = '\0'
 *         buff+=1
 *     PyByteArray_Resize(array, buff - PyByteArray_AS_STRING(array))             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_1 = PyByteArray_Resize(__pyx_v_array, (__pyx_v_buff - PyByteArray_AS_STRING(__pyx_v_array))); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 319, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":320
 *         buff+=1
 *     PyByteArray_Resize(array, buff - PyByteArray_AS_STRING(array))
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_uint64(Field _f, bytearray array, object value):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":298
 *     raw_encode_uint32(array, un)
 * 
 * cdef inline int raw_encode_uint64(bytearray array, uint64_t n) except -1:             # <<<<<<<<<<<<<<
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.raw_encode_uint64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":322
 *     return 0
 * 
 * cdef inline encode_uint64(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     raw_encode_uint64(array, value)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_uint64(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v__f, PyObject *__pyx_v_array, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  uint64_t __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("encode_uint64", 0);

  /* "cprotobuf/utils.pxi":323
 * 
 * cdef inline encode_uint64(Field _f, bytearray array, object value):
 *     raw_encode_uint64(array, value)             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_int64(Field _f, bytearray array, object value):
 */
  __pyx_t_1 = __Pyx_PyInt_As_uint64_t(__pyx_v_value); if (unlikely((__pyx_t_1 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(2, 323, __pyx_L1_error)
  __pyx_t_2 = __pyx_f_9cprotobuf_8internal_raw_encode_uint64(__pyx_v_array, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 323, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":322
 *     return 0
 * 
 * cdef inline encode_uint64(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     raw_encode_uint64(array, value)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.encode_uint64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":325
 *     raw_encode_uint64(array, value)
 * 
 * cdef inline encode_int64(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef int64_t n = value
 *     raw_encode_uint64(array, <uint64_t>n)
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_int64(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v__f, PyObject *__pyx_v_array, PyObject *__pyx_v_value) {
  int64_t __pyx_v_n;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int64_t __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("encode_int64", 0);

  /* "cprotobuf/utils.pxi":326
 * 
 * cdef inline encode_int64(Field _f, bytearray array, object value):
 *     cdef int64_t n = value             # <<<<<<<<<<<<<<
 *     raw_encode_uint64(array, <uint64_t>n)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_As_int64_t(__pyx_v_value); if (unlikely((__pyx_t_1 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(2, 326, __pyx_L1_error)
  __pyx_v_n = __pyx_t_1;

  /* "cprotobuf/utils.pxi":327
 * cdef inline encode_int64(Field _f, bytearray array, object value):
 *     cdef int64_t n = value
 *     raw_encode_uint64(array, <uint64_t>n)             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_sint64(Field _f, bytearray array, object value):
 */
  __pyx_t_2 = __pyx_f_9cprotobuf_8internal_raw_encode_uint64(__pyx_v_array, ((uint64_t)__pyx_v_n)); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 327, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":325
 *     raw_encode_uint64(array, value)
 * 
 * cdef inline encode_int64(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef int64_t n = value
 *     raw_encode_uint64(array, <uint64_t>n)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.encode_int64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":329
 *     raw_encode_uint64(array, <uint64_t>n)
 * 
 * cdef inline encode_sint64(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef int64_t n = value
 *     cdef uint64_t un = (n<<1) ^ (n>>63)
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_sint64(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v__f, PyObject *__pyx_v_array, PyObject *__pyx_v_value) {
  int64_t __pyx_v_n;
  uint64_t __pyx_v_un;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int64_t __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("encode_sint64", 0);

  /* "cprotobuf/utils.pxi":330
 * 
 * cdef inline encode_sint64(Field _f, bytearray array, object value):
 *     cdef int64_t n = value             # <<<<<<<<<<<<<<
 *     cdef uint64_t un = (n<<1) ^ (n>>63)
 *     raw_encode_uint64(array, un)
 */
  __pyx_t_1 = __Pyx_PyInt_As_int64_t(__pyx_v_value); if (unlikely((__pyx_t_1 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(2, 330, __pyx_L1_error)
  __pyx_v_n = __pyx_t_1;

  /* "cprotobuf/utils.pxi":331
 * cdef inline encode_sint64(Field _f, bytearray array, object value):
 *     cdef int64_t n = value
 *     cdef uint64_t un = (n<<1) ^ (n>>63)             # <<<<<<<<<<<<<<
 *     raw_encode_uint64(array, un)
 * 
 */
  __pyx_v_un = ((__pyx_v_n << 1) ^ (__pyx_v_n >> 63));

  /* "cprotobuf/utils.pxi":332
 *     cdef int64_t n = value
 *     cdef uint64_t un = (n<<1) ^ (n>>63)
 *     raw_encode_uint64(array, un)             # <<<<<<<<<<<<<<
 * 
 * cdef inline int raw_encode_fixed32(bytearray array, uint32_t n) except -1:
 */
  __pyx_t_2 = __pyx_f_9cprotobuf_8internal_raw_encode_uint64(__pyx_v_array, __pyx_v_un); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 332, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":329
 *     raw_encode_uint64(array, <uint64_t>n)
 * 
 * cdef inline encode_sint64(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef int64_t n = value
 *     cdef uint64_t un = (n<<1) ^ (n>>63)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.encode_sint64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":334
 *     raw_encode_uint64(array, un)
 * 
 * cdef inline int raw_encode_fixed32(bytearray array, uint32_t n) except -1:             # <<<<<<<<<<<<<<
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 */

static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_encode_fixed32(PyObject *__pyx_v_array, uint32_t __pyx_v_n) {
  unsigned short __pyx_v_rem;
  Py_ssize_t __pyx_v_size;
  char *__pyx_v_buff;
  CYTHON_UNUSED int __pyx_v_i;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("raw_encode_fixed32", 0);

  /* "cprotobuf/utils.pxi":336
 * cdef inline int raw_encode_fixed32(bytearray array, uint32_t n) except -1:
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)             # <<<<<<<<<<<<<<
 *     PyByteArray_Resize(array, size + 4)
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 */
  __pyx_v_size = PyByteArray_GET_SIZE(__pyx_v_array);

  /* "cprotobuf/utils.pxi":337
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 *     PyByteArray_Resize(array, size + 4)             # <<<<<<<<<<<<<<
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 *     cdef int i
 */
  __pyx_t_1 = PyByteArray_Resize(__pyx_v_array, (__pyx_v_size + 4)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 337, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":338
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 *     PyByteArray_Resize(array, size + 4)
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size             # <<<<<<<<<<<<<<
 *     cdef int i
 * 
 */
  __pyx_v_buff = (PyByteArray_AS_STRING(__pyx_v_array) + __pyx_v_size);

  /* "cprotobuf/utils.pxi":341
 *     cdef int i
 * 
 *     for i from 0 <= i < 4:             # <<<<<<<<<<<<<<
 *         rem = n & 0xff
 *         n = n >> 8
 */
  for (__pyx_v_i = 0; __pyx_v_i < 4; __pyx_v_i++) {

    /* "cprotobuf/utils.pxi":342
 * 
 *     for i from 0 <= i < 4:
 *         rem = n & 0xff             # <<<<<<<<<<<<<<
 *         n = n >> 8
 *         buff[0] = <char> rem
 */
    __pyx_v_rem = (__pyx_v_n & 0xff);

    /* "cprotobuf/utils.pxi":343
 *     for i from 0 <= i < 4:
 *         rem = n & 0xff
 *         n = n >> 8             # <<<<<<<<<<<<<<
 *         buff[0] = <char> rem
 *         buff += 1
 */
    __pyx_v_n = (__pyx_v_n >> 8);

    /* "cprotobuf/utils.pxi":344
 *         rem = n & 0xff
 *         n = n >> 8
 *         buff[0] = <char> rem             # <<<<<<<<<<<<<<
 *         buff += 1
 * 
 */
    (__pyx_v_buff[0]) = ((char)__pyx_v_rem);

    /* "cprotobuf/utils.pxi":345
 *         n = n >> 8
 *         buff[0] = <char> rem
 *         buff += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    __pyx_v_buff = (__pyx_v_buff + 1);
  }

  /* "cprotobuf/utils.pxi":347
 *         buff += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_fixed32(Field _f, bytearray array, object value):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":334
 *     raw_encode_uint64(array, un)
 * 
 * cdef inline int raw_encode_fixed32(bytearray array, uint32_t n) except -1:             # <<<<<<<<<<<<<<
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.raw_encode_fixed32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":349
 *     return 0
 * 
 * cdef inline encode_fixed32(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     raw_encode_fixed32(array, value)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_fixed32(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v__f, PyObject *__pyx_v_array, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  uint32_t __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("encode_fixed32", 0);

  /* "cprotobuf/utils.pxi":350
 * 
 * cdef inline encode_fixed32(Field _f, bytearray array, object value):
 *     raw_encode_fixed32(array, value)             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_sfixed32(Field _f, bytearray array, object value):
 */
  __pyx_t_1 = __Pyx_PyInt_As_uint32_t(__pyx_v_value); if (unlikely((__pyx_t_1 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(2, 350, __pyx_L1_error)
  __pyx_t_2 = __pyx_f_9cprotobuf_8internal_raw_encode_fixed32(__pyx_v_array, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 350, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":349
 *     return 0
 * 
 * cdef inline encode_fixed32(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     raw_encode_fixed32(array, value)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.encode_fixed32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":352
 *     raw_encode_fixed32(array, value)
 * 
 * cdef inline encode_sfixed32(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef int32_t n = value
 *     cdef unsigned short int rem
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_sfixed32(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v__f, PyObject *__pyx_v_array, PyObject *__pyx_v_value) {
  int32_t __pyx_v_n;
  unsigned short __pyx_v_rem;
  Py_ssize_t __pyx_v_size;
  char *__pyx_v_buff;
  CYTHON_UNUSED int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int32_t __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("encode_sfixed32", 0);

  /* "cprotobuf/utils.pxi":353
 * 
 * cdef inline encode_sfixed32(Field _f, bytearray array, object value):
 *     cdef int32_t n = value             # <<<<<<<<<<<<<<
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 */
  __pyx_t_1 = __Pyx_PyInt_As_int32_t(__pyx_v_value); if (unlikely((__pyx_t_1 == ((int32_t)-1)) && PyErr_Occurred())) __PYX_ERR(2, 353, __pyx_L1_error)
  __pyx_v_n = __pyx_t_1;

  /* "cprotobuf/utils.pxi":355
 *     cdef int32_t n = value
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)             # <<<<<<<<<<<<<<
 *     PyByteArray_Resize(array, size + 4)
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 */
  __pyx_v_size = PyByteArray_GET_SIZE(__pyx_v_array);

  /* "cprotobuf/utils.pxi":356
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 *     PyByteArray_Resize(array, size + 4)             # <<<<<<<<<<<<<<
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 *     cdef int i
 */
  __pyx_t_2 = PyByteArray_Resize(__pyx_v_array, (__pyx_v_size + 4)); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 356, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":357
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 *     PyByteArray_Resize(array, size + 4)
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size             # <<<<<<<<<<<<<<
 *     cdef int i
 * 
 */
  __pyx_v_buff = (PyByteArray_AS_STRING(__pyx_v_array) + __pyx_v_size);

  /* "cprotobuf/utils.pxi":360
 *     cdef int i
 * 
 *     for i from 0 <= i < 4:             # <<<<<<<<<<<<<<
 *         rem = n & 0xff
 *         n = n >> 8
 */
  for (__pyx_v_i = 0; __pyx_v_i < 4; __pyx_v_i++) {

    /* "cprotobuf/utils.pxi":361
 * 
 *     for i from 0 <= i < 4:
 *         rem = n & 0xff             # <<<<<<<<<<<<<<
 *         n = n >> 8
 *         buff[0] = <char> rem
 */
    __pyx_v_rem = (__pyx_v_n & 0xff);

    /* "cprotobuf/utils.pxi":362
 *     for i from 0 <= i < 4:
 *         rem = n & 0xff
 *         n = n >> 8             # <<<<<<<<<<<<<<
 *         buff[0] = <char> rem
 *         buff += 1
 */
    __pyx_v_n = (__pyx_v_n >> 8);

    /* "cprotobuf/utils.pxi":363
 *         rem = n & 0xff
 *         n = n >> 8
 *         buff[0] = <char> rem             # <<<<<<<<<<<<<<
 *         buff += 1
 * 
 */
    (__pyx_v_buff[0]) = ((char)__pyx_v_rem);

    /* "cprotobuf/utils.pxi":364
 *         n = n >> 8
 *         buff[0] = <char> rem
 *         buff += 1             # <<<<<<<<<<<<<<
 * 
 * cdef inline int raw_encode_fixed64(bytearray array, uint64_t n) except -1:
 */
    __pyx_v_buff = (__pyx_v_buff + 1);
  }

  /* "cprotobuf/utils.pxi":352
 *     raw_encode_fixed32(array, value)
 * 
 * cdef inline encode_sfixed32(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef int32_t n = value
 *     cdef unsigned short int rem
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.encode_sfixed32", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":366
 *         buff += 1
 * 
 * cdef inline int raw_encode_fixed64(bytearray array, uint64_t n) except -1:             # <<<<<<<<<<<<<<
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 */

static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_raw_encode_fixed64(PyObject *__pyx_v_array, uint64_t __pyx_v_n) {
  unsigned short __pyx_v_rem;
  Py_ssize_t __pyx_v_size;
  char *__pyx_v_buff;
  CYTHON_UNUSED int __pyx_v_i;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("raw_encode_fixed64", 0);

  /* "cprotobuf/utils.pxi":368
 * cdef inline int raw_encode_fixed64(bytearray array, uint64_t n) except -1:
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)             # <<<<<<<<<<<<<<
 *     PyByteArray_Resize(array, size + 8)
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 */
  __pyx_v_size = PyByteArray_GET_SIZE(__pyx_v_array);

  /* "cprotobuf/utils.pxi":369
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 *     PyByteArray_Resize(array, size + 8)             # <<<<<<<<<<<<<<
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 *     cdef int i
 */
  __pyx_t_1 = PyByteArray_Resize(__pyx_v_array, (__pyx_v_size + 8)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 369, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":370
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 *     PyByteArray_Resize(array, size + 8)
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size             # <<<<<<<<<<<<<<
 *     cdef int i
 * 
 */
  __pyx_v_buff = (PyByteArray_AS_STRING(__pyx_v_array) + __pyx_v_size);

  /* "cprotobuf/utils.pxi":373
 *     cdef int i
 * 
 *     for i from 0 <= i < 8:             # <<<<<<<<<<<<<<
 *         rem = n & 0xff
 *         n = n >> 8
 */
  for (__pyx_v_i = 0; __pyx_v_i < 8; __pyx_v_i++) {

    /* "cprotobuf/utils.pxi":374
 * 
 *     for i from 0 <= i < 8:
 *         rem = n & 0xff             # <<<<<<<<<<<<<<
 *         n = n >> 8
 *         buff[0] = <char> rem
 */
    __pyx_v_rem = (__pyx_v_n & 0xff);

    /* "cprotobuf/utils.pxi":375
 *     for i from 0 <= i < 8:
 *         rem = n & 0xff
 *         n = n >> 8             # <<<<<<<<<<<<<<
 *         buff[0] = <char> rem
 *         buff += 1
 */
    __pyx_v_n = (__pyx_v_n >> 8);

    /* "cprotobuf/utils.pxi":376
 *         rem = n & 0xff
 *         n = n >> 8
 *         buff[0] = <char> rem             # <<<<<<<<<<<<<<
 *         buff += 1
 * 
 */
    (__pyx_v_buff[0]) = ((char)__pyx_v_rem);

    /* "cprotobuf/utils.pxi":377
 *         n = n >> 8
 *         buff[0] = <char> rem
 *         buff += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    __pyx_v_buff = (__pyx_v_buff + 1);
  }

  /* "cprotobuf/utils.pxi":379
 *         buff += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_fixed64(Field _f, bytearray array, object value):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "cprotobuf/utils.pxi":366
 *         buff += 1
 * 
 * cdef inline int raw_encode_fixed64(bytearray array, uint64_t n) except -1:             # <<<<<<<<<<<<<<
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.raw_encode_fixed64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":381
 *     return 0
 * 
 * cdef inline encode_fixed64(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     raw_encode_fixed64(array, value)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_fixed64(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v__f, PyObject *__pyx_v_array, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  uint64_t __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("encode_fixed64", 0);

  /* "cprotobuf/utils.pxi":382
 * 
 * cdef inline encode_fixed64(Field _f, bytearray array, object value):
 *     raw_encode_fixed64(array, value)             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_sfixed64(Field _f, bytearray array, object value):
 */
  __pyx_t_1 = __Pyx_PyInt_As_uint64_t(__pyx_v_value); if (unlikely((__pyx_t_1 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(2, 382, __pyx_L1_error)
  __pyx_t_2 = __pyx_f_9cprotobuf_8internal_raw_encode_fixed64(__pyx_v_array, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 382, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":381
 *     return 0
 * 
 * cdef inline encode_fixed64(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     raw_encode_fixed64(array, value)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.encode_fixed64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":384
 *     raw_encode_fixed64(array, value)
 * 
 * cdef inline encode_sfixed64(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef int64_t n = value
 *     cdef unsigned short int rem
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_sfixed64(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v__f, PyObject *__pyx_v_array, PyObject *__pyx_v_value) {
  int64_t __pyx_v_n;
  unsigned short __pyx_v_rem;
  Py_ssize_t __pyx_v_size;
  char *__pyx_v_buff;
  CYTHON_UNUSED int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int64_t __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("encode_sfixed64", 0);

  /* "cprotobuf/utils.pxi":385
 * 
 * cdef inline encode_sfixed64(Field _f, bytearray array, object value):
 *     cdef int64_t n = value             # <<<<<<<<<<<<<<
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 */
  __pyx_t_1 = __Pyx_PyInt_As_int64_t(__pyx_v_value); if (unlikely((__pyx_t_1 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(2, 385, __pyx_L1_error)
  __pyx_v_n = __pyx_t_1;

  /* "cprotobuf/utils.pxi":387
 *     cdef int64_t n = value
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)             # <<<<<<<<<<<<<<
 *     PyByteArray_Resize(array, size + 8)
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 */
  __pyx_v_size = PyByteArray_GET_SIZE(__pyx_v_array);

  /* "cprotobuf/utils.pxi":388
 *     cdef unsigned short int rem
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 *     PyByteArray_Resize(array, size + 8)             # <<<<<<<<<<<<<<
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 *     cdef int i
 */
  __pyx_t_2 = PyByteArray_Resize(__pyx_v_array, (__pyx_v_size + 8)); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 388, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":389
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 *     PyByteArray_Resize(array, size + 8)
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size             # <<<<<<<<<<<<<<
 *     cdef int i
 * 
 */
  __pyx_v_buff = (PyByteArray_AS_STRING(__pyx_v_array) + __pyx_v_size);

  /* "cprotobuf/utils.pxi":392
 *     cdef int i
 * 
 *     for i from 0 <= i < 8:             # <<<<<<<<<<<<<<
 *         rem = n & 0xff
 *         n = n >> 8
 */
  for (__pyx_v_i = 0; __pyx_v_i < 8; __pyx_v_i++) {

    /* "cprotobuf/utils.pxi":393
 * 
 *     for i from 0 <= i < 8:
 *         rem = n & 0xff             # <<<<<<<<<<<<<<
 *         n = n >> 8
 *         buff[0] = <char> rem
 */
    __pyx_v_rem = (__pyx_v_n & 0xff);

    /* "cprotobuf/utils.pxi":394
 *     for i from 0 <= i < 8:
 *         rem = n & 0xff
 *         n = n >> 8             # <<<<<<<<<<<<<<
 *         buff[0] = <char> rem
 *         buff += 1
 */
    __pyx_v_n = (__pyx_v_n >> 8);

    /* "cprotobuf/utils.pxi":395
 *         rem = n & 0xff
 *         n = n >> 8
 *         buff[0] = <char> rem             # <<<<<<<<<<<<<<
 *         buff += 1
 * 
 */
    (__pyx_v_buff[0]) = ((char)__pyx_v_rem);

    /* "cprotobuf/utils.pxi":396
 *         n = n >> 8
 *         buff[0] = <char> rem
 *         buff += 1             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_bytes(Field _f, bytearray array, object n):
 */
    __pyx_v_buff = (__pyx_v_buff + 1);
  }

  /* "cprotobuf/utils.pxi":384
 *     raw_encode_fixed64(array, value)
 * 
 * cdef inline encode_sfixed64(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef int64_t n = value
 *     cdef unsigned short int rem
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.encode_sfixed64", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":398
 *         buff += 1
 * 
 * cdef inline encode_bytes(Field _f, bytearray array, object n):             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t len = PySequence_Length(n)
 *     raw_encode_uint64(array, len)
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_bytes(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v__f, PyObject *__pyx_v_array, PyObject *__pyx_v_n) {
  Py_ssize_t __pyx_v_len;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("encode_bytes", 0);

  /* "cprotobuf/utils.pxi":399
 * 
 * cdef inline encode_bytes(Field _f, bytearray array, object n):
 *     cdef Py_ssize_t len = PySequence_Length(n)             # <<<<<<<<<<<<<<
 *     raw_encode_uint64(array, len)
 *     PySequence_InPlaceConcat(array, n)
 */
  __pyx_t_1 = PySequence_Length(__pyx_v_n); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1L))) __PYX_ERR(2, 399, __pyx_L1_error)
  __pyx_v_len = __pyx_t_1;

  /* "cprotobuf/utils.pxi":400
 * cdef inline encode_bytes(Field _f, bytearray array, object n):
 *     cdef Py_ssize_t len = PySequence_Length(n)
 *     raw_encode_uint64(array, len)             # <<<<<<<<<<<<<<
 *     PySequence_InPlaceConcat(array, n)
 * 
 */
  __pyx_t_2 = __pyx_f_9cprotobuf_8internal_raw_encode_uint64(__pyx_v_array, __pyx_v_len); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 400, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":401
 *     cdef Py_ssize_t len = PySequence_Length(n)
 *     raw_encode_uint64(array, len)
 *     PySequence_InPlaceConcat(array, n)             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_string(Field _f, bytearray array, object n):
 */
  __pyx_t_3 = PySequence_InPlaceConcat(__pyx_v_array, __pyx_v_n); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cprotobuf/utils.pxi":398
 *         buff += 1
 * 
 * cdef inline encode_bytes(Field _f, bytearray array, object n):             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t len = PySequence_Length(n)
 *     raw_encode_uint64(array, len)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cprotobuf.internal.encode_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":403
 *     PySequence_InPlaceConcat(array, n)
 * 
 * cdef inline encode_string(Field _f, bytearray array, object n):             # <<<<<<<<<<<<<<
 *     if isinstance(n, unicode):
 *         n = PyUnicode_AsUTF8String(n)
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_string(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v__f, PyObject *__pyx_v_array, PyObject *__pyx_v_n) {
  Py_ssize_t __pyx_v_len;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("encode_string", 0);
  __Pyx_INCREF(__pyx_v_n);

  /* "cprotobuf/utils.pxi":404
 * 
 * cdef inline encode_string(Field _f, bytearray array, object n):
 *     if isinstance(n, unicode):             # <<<<<<<<<<<<<<
 *         n = PyUnicode_AsUTF8String(n)
 *     cdef Py_ssize_t len = PySequence_Length(n)
 */
  __pyx_t_1 = PyUnicode_Check(__pyx_v_n); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "cprotobuf/utils.pxi":405
 * cdef inline encode_string(Field _f, bytearray array, object n):
 *     if isinstance(n, unicode):
 *         n = PyUnicode_AsUTF8String(n)             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t len = PySequence_Length(n)
 *     raw_encode_uint64(array, len)
 */
    __pyx_t_3 = PyUnicode_AsUTF8String(__pyx_v_n); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 405, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_n, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "cprotobuf/utils.pxi":404
 * 
 * cdef inline encode_string(Field _f, bytearray array, object n):
 *     if isinstance(n, unicode):             # <<<<<<<<<<<<<<
 *         n = PyUnicode_AsUTF8String(n)
 *     cdef Py_ssize_t len = PySequence_Length(n)
 */
  }

  /* "cprotobuf/utils.pxi":406
 *     if isinstance(n, unicode):
 *         n = PyUnicode_AsUTF8String(n)
 *     cdef Py_ssize_t len = PySequence_Length(n)             # <<<<<<<<<<<<<<
 *     raw_encode_uint64(array, len)
 *     PySequence_InPlaceConcat(array, n)
 */
  __pyx_t_4 = PySequence_Length(__pyx_v_n); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1L))) __PYX_ERR(2, 406, __pyx_L1_error)
  __pyx_v_len = __pyx_t_4;

  /* "cprotobuf/utils.pxi":407
 *         n = PyUnicode_AsUTF8String(n)
 *     cdef Py_ssize_t len = PySequence_Length(n)
 *     raw_encode_uint64(array, len)             # <<<<<<<<<<<<<<
 *     PySequence_InPlaceConcat(array, n)
 * 
 */
  __pyx_t_5 = __pyx_f_9cprotobuf_8internal_raw_encode_uint64(__pyx_v_array, __pyx_v_len); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 407, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":408
 *     cdef Py_ssize_t len = PySequence_Length(n)
 *     raw_encode_uint64(array, len)
 *     PySequence_InPlaceConcat(array, n)             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_bool(Field _f, bytearray array, object value):
 */
  __pyx_t_3 = PySequence_InPlaceConcat(__pyx_v_array, __pyx_v_n); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cprotobuf/utils.pxi":403
 *     PySequence_InPlaceConcat(array, n)
 * 
 * cdef inline encode_string(Field _f, bytearray array, object n):             # <<<<<<<<<<<<<<
 *     if isinstance(n, unicode):
 *         n = PyUnicode_AsUTF8String(n)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cprotobuf.internal.encode_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_n);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":410
 *     PySequence_InPlaceConcat(array, n)
 * 
 * cdef inline encode_bool(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef bint b = value
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_bool(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v__f, PyObject *__pyx_v_array, PyObject *__pyx_v_value) {
  int __pyx_v_b;
  Py_ssize_t __pyx_v_size;
  char *__pyx_v_buff;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("encode_bool", 0);

  /* "cprotobuf/utils.pxi":411
 * 
 * cdef inline encode_bool(Field _f, bytearray array, object value):
 *     cdef bint b = value             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 *     PyByteArray_Resize(array, size + 1)
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 411, __pyx_L1_error)
  __pyx_v_b = __pyx_t_1;

  /* "cprotobuf/utils.pxi":412
 * cdef inline encode_bool(Field _f, bytearray array, object value):
 *     cdef bint b = value
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)             # <<<<<<<<<<<<<<
 *     PyByteArray_Resize(array, size + 1)
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 */
  __pyx_v_size = PyByteArray_GET_SIZE(__pyx_v_array);

  /* "cprotobuf/utils.pxi":413
 *     cdef bint b = value
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 *     PyByteArray_Resize(array, size + 1)             # <<<<<<<<<<<<<<
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 *     buff[0] = b
 */
  __pyx_t_2 = PyByteArray_Resize(__pyx_v_array, (__pyx_v_size + 1)); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 413, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":414
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 *     PyByteArray_Resize(array, size + 1)
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size             # <<<<<<<<<<<<<<
 *     buff[0] = b
 * 
 */
  __pyx_v_buff = (PyByteArray_AS_STRING(__pyx_v_array) + __pyx_v_size);

  /* "cprotobuf/utils.pxi":415
 *     PyByteArray_Resize(array, size + 1)
 *     cdef char *buff = PyByteArray_AS_STRING(array) + size
 *     buff[0] = b             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_float(Field _f, bytearray array, object value):
 */
  (__pyx_v_buff[0]) = __pyx_v_b;

  /* "cprotobuf/utils.pxi":410
 *     PySequence_InPlaceConcat(array, n)
 * 
 * cdef inline encode_bool(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef bint b = value
 *     cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.encode_bool", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":417
 *     buff[0] = b
 * 
 * cdef inline encode_float(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef float f = value
 *     raw_encode_fixed32(array, (<uint32_t*>&f)[0])
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_float(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v__f, PyObject *__pyx_v_array, PyObject *__pyx_v_value) {
  float __pyx_v_f;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  float __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("encode_float", 0);

  /* "cprotobuf/utils.pxi":418
 * 
 * cdef inline encode_float(Field _f, bytearray array, object value):
 *     cdef float f = value             # <<<<<<<<<<<<<<
 *     raw_encode_fixed32(array, (<uint32_t*>&f)[0])
 * 
 */
  __pyx_t_1 = __pyx_PyFloat_AsFloat(__pyx_v_value); if (unlikely((__pyx_t_1 == (float)-1) && PyErr_Occurred())) __PYX_ERR(2, 418, __pyx_L1_error)
  __pyx_v_f = __pyx_t_1;

  /* "cprotobuf/utils.pxi":419
 * cdef inline encode_float(Field _f, bytearray array, object value):
 *     cdef float f = value
 *     raw_encode_fixed32(array, (<uint32_t*>&f)[0])             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_double(Field _f, bytearray array, object value):
 */
  __pyx_t_2 = __pyx_f_9cprotobuf_8internal_raw_encode_fixed32(__pyx_v_array, (((uint32_t *)(&__pyx_v_f))[0])); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 419, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":417
 *     buff[0] = b
 * 
 * cdef inline encode_float(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef float f = value
 *     raw_encode_fixed32(array, (<uint32_t*>&f)[0])
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.encode_float", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":421
 *     raw_encode_fixed32(array, (<uint32_t*>&f)[0])
 * 
 * cdef inline encode_double(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef double d = value
 *     raw_encode_fixed64(array, (<uint64_t*>&d)[0])
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_double(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v__f, PyObject *__pyx_v_array, PyObject *__pyx_v_value) {
  double __pyx_v_d;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("encode_double", 0);

  /* "cprotobuf/utils.pxi":422
 * 
 * cdef inline encode_double(Field _f, bytearray array, object value):
 *     cdef double d = value             # <<<<<<<<<<<<<<
 *     raw_encode_fixed64(array, (<uint64_t*>&d)[0])
 * 
 */
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 422, __pyx_L1_error)
  __pyx_v_d = __pyx_t_1;

  /* "cprotobuf/utils.pxi":423
 * cdef inline encode_double(Field _f, bytearray array, object value):
 *     cdef double d = value
 *     raw_encode_fixed64(array, (<uint64_t*>&d)[0])             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_type(bytearray array, unsigned char wire_type, uint32_t index):
 */
  __pyx_t_2 = __pyx_f_9cprotobuf_8internal_raw_encode_fixed64(__pyx_v_array, (((uint64_t *)(&__pyx_v_d))[0])); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(2, 423, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":421
 *     raw_encode_fixed32(array, (<uint32_t*>&f)[0])
 * 
 * cdef inline encode_double(Field _f, bytearray array, object value):             # <<<<<<<<<<<<<<
 *     cdef double d = value
 *     raw_encode_fixed64(array, (<uint64_t*>&d)[0])
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.encode_double", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/utils.pxi":425
 *     raw_encode_fixed64(array, (<uint64_t*>&d)[0])
 * 
 * cdef inline encode_type(bytearray array, unsigned char wire_type, uint32_t index):             # <<<<<<<<<<<<<<
 *     raw_encode_uint32(array, index<<3|wire_type)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_type(PyObject *__pyx_v_array, unsigned char __pyx_v_wire_type, uint32_t __pyx_v_index) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("encode_type", 0);

  /* "cprotobuf/utils.pxi":426
 * 
 * cdef inline encode_type(bytearray array, unsigned char wire_type, uint32_t index):
 *     raw_encode_uint32(array, index<<3|wire_type)             # <<<<<<<<<<<<<<
 * 
 * # }}}
 */
  __pyx_t_1 = __pyx_f_9cprotobuf_8internal_raw_encode_uint32(__pyx_v_array, ((__pyx_v_index << 3) | __pyx_v_wire_type)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 426, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":425
 *     raw_encode_fixed64(array, (<uint64_t*>&d)[0])
 * 
 * cdef inline encode_type(bytearray array, unsigned char wire_type, uint32_t index):             # <<<<<<<<<<<<<<
 *     raw_encode_uint32(array, index<<3|wire_type)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.encode_type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":46
 *     cdef object klass
 * 
 *     def __init__(self, cls):             # <<<<<<<<<<<<<<
 *         self.klass = cls
 * 
 */

/* Python wrapper */
static int __pyx_pw_9cprotobuf_8internal_17RepeatedContainer_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9cprotobuf_8internal_17RepeatedContainer_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_cls = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_cls,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_cls)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 46, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_cls = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 46, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cprotobuf.internal.RepeatedContainer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9cprotobuf_8internal_17RepeatedContainer___init__(((struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *)__pyx_v_self), __pyx_v_cls);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9cprotobuf_8internal_17RepeatedContainer___init__(struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *__pyx_v_self, PyObject *__pyx_v_cls) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cprotobuf/internal.pyx":47
 * 
 *     def __init__(self, cls):
 *         self.klass = cls             # <<<<<<<<<<<<<<
 * 
 *     def add(self, **kwargs):
 */
  __Pyx_INCREF(__pyx_v_cls);
  __Pyx_GIVEREF(__pyx_v_cls);
  __Pyx_GOTREF(__pyx_v_self->klass);
  __Pyx_DECREF(__pyx_v_self->klass);
  __pyx_v_self->klass = __pyx_v_cls;

  /* "cprotobuf/internal.pyx":46
 *     cdef object klass
 * 
 *     def __init__(self, cls):             # <<<<<<<<<<<<<<
 *         self.klass = cls
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":49
 *         self.klass = cls
 * 
 *     def add(self, **kwargs):             # <<<<<<<<<<<<<<
 *         obj = self.klass(**kwargs)
 *         self.append(obj)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_17RepeatedContainer_3add(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9cprotobuf_8internal_17RepeatedContainer_3add(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_kwargs = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("add", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return NULL;}
  if (__pyx_kwds && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "add", 1))) return NULL;
  if (__pyx_kwds) {
    __pyx_v_kwargs = PyDict_Copy(__pyx_kwds); if (unlikely(!__pyx_v_kwargs)) return NULL;
    __Pyx_GOTREF(__pyx_v_kwargs);
  } else {
    __pyx_v_kwargs = NULL;
  }
  __pyx_r = __pyx_pf_9cprotobuf_8internal_17RepeatedContainer_2add(((struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *)__pyx_v_self), __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_17RepeatedContainer_2add(struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *__pyx_v_self, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_v_obj = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("add", 0);

  /* "cprotobuf/internal.pyx":50
 * 
 *     def add(self, **kwargs):
 *         obj = self.klass(**kwargs)             # <<<<<<<<<<<<<<
 *         self.append(obj)
 *         return obj
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_v_self->klass, __pyx_empty_tuple, __pyx_v_kwargs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_obj = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":51
 *     def add(self, **kwargs):
 *         obj = self.klass(**kwargs)
 *         self.append(obj)             # <<<<<<<<<<<<<<
 *         return obj
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_Append(((PyObject *)__pyx_v_self), __pyx_v_obj); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 51, __pyx_L1_error)

  /* "cprotobuf/internal.pyx":52
 *         obj = self.klass(**kwargs)
 *         self.append(obj)
 *         return obj             # <<<<<<<<<<<<<<
 * 
 * cdef class Field(object):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_obj);
  __pyx_r = __pyx_v_obj;
  goto __pyx_L0;

  /* "cprotobuf/internal.pyx":49
 *         self.klass = cls
 * 
 *     def add(self, **kwargs):             # <<<<<<<<<<<<<<
 *         obj = self.klass(**kwargs)
 *         self.append(obj)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cprotobuf.internal.RepeatedContainer.add", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.klass,)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_17RepeatedContainer_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9cprotobuf_8internal_17RepeatedContainer_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_17RepeatedContainer_4__reduce_cython__(((struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_17RepeatedContainer_4__reduce_cython__(struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = (self.klass,)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->klass);
  __Pyx_GIVEREF(__pyx_v_self->klass);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->klass);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = (self.klass,)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":5
 *     state = (self.klass,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.klass is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = (self.klass,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = self.klass is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_RepeatedContainer, (type(self), 0x64a34c4, None), state
 */
  /*else*/ {
    __pyx_t_3 = (__pyx_v_self->klass != Py_None);
    __pyx_v_use_setstate = __pyx_t_3;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.klass is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_RepeatedContainer, (type(self), 0x64a34c4, None), state
 *     else:
 */
  __pyx_t_3 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":11
 *         use_setstate = self.klass is not None
 *     if use_setstate:
 *         return __pyx_unpickle_RepeatedContainer, (type(self), 0x64a34c4, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_RepeatedContainer, (type(self), 0x64a34c4, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_RepeatedContainer); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self));
    __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_4type_type), __pyx_t_1, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5);
    __Pyx_INCREF(__pyx_int_105526468);
    __Pyx_GIVEREF(__pyx_int_105526468);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_105526468);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
    __pyx_t_5 = 0;
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.klass is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_RepeatedContainer, (type(self), 0x64a34c4, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle_RepeatedContainer, (type(self), 0x64a34c4, None), state
 *     else:
 *         return __pyx_unpickle_RepeatedContainer, (type(self), 0x64a34c4, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_RepeatedContainer__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_RepeatedContainer); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self));
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_4type_type), __pyx_t_1, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4);
    __Pyx_INCREF(__pyx_int_105526468);
    __Pyx_GIVEREF(__pyx_int_105526468);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_105526468);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.klass,)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cprotobuf.internal.RepeatedContainer.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_RepeatedContainer, (type(self), 0x64a34c4, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_RepeatedContainer__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_17RepeatedContainer_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9cprotobuf_8internal_17RepeatedContainer_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_17RepeatedContainer_6__setstate_cython__(((struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_17RepeatedContainer_6__setstate_cython__(struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle_RepeatedContainer, (type(self), 0x64a34c4, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_RepeatedContainer__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_9cprotobuf_8internal___pyx_unpickle_RepeatedContainer__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_RepeatedContainer, (type(self), 0x64a34c4, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_RepeatedContainer__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cprotobuf.internal.RepeatedContainer.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":69
 *     cdef Decoder decoder
 * 
 *     def __init__(self, type, index, required=True, repeated=False, packed=False, default=None):             # <<<<<<<<<<<<<<
 *         assert type in wire_types or isinstance(type, (str, unicode)) or issubclass(type, ProtoEntity), 'invalid type %s' % type
 * 
 */

/* Python wrapper */
static int __pyx_pw_9cprotobuf_8internal_5Field_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9cprotobuf_8internal_5Field_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_type = 0;
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_required = 0;
  PyObject *__pyx_v_repeated = 0;
  PyObject *__pyx_v_packed = 0;
  PyObject *__pyx_v_default = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_type,&__pyx_n_s_index,&__pyx_n_s_required,&__pyx_n_s_repeated,&__pyx_n_s_packed,&__pyx_n_s_default,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    values[2] = ((PyObject *)Py_True);
    values[3] = ((PyObject *)Py_False);
    values[4] = ((PyObject *)Py_False);
    values[5] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 6, 1); __PYX_ERR(0, 69, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_required);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_repeated);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_packed);
          if (value) { values[4] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_default);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 69, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_type = values[0];
    __pyx_v_index = values[1];
    __pyx_v_required = values[2];
    __pyx_v_repeated = values[3];
    __pyx_v_packed = values[4];
    __pyx_v_default = values[5];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 69, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cprotobuf.internal.Field.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field___init__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self), __pyx_v_type, __pyx_v_index, __pyx_v_required, __pyx_v_repeated, __pyx_v_packed, __pyx_v_default);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9cprotobuf_8internal_5Field___init__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, PyObject *__pyx_v_type, PyObject *__pyx_v_index, PyObject *__pyx_v_required, PyObject *__pyx_v_repeated, PyObject *__pyx_v_packed, PyObject *__pyx_v_default) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cprotobuf/internal.pyx":70
 * 
 *     def __init__(self, type, index, required=True, repeated=False, packed=False, default=None):
 *         assert type in wire_types or isinstance(type, (str, unicode)) or issubclass(type, ProtoEntity), 'invalid type %s' % type             # <<<<<<<<<<<<<<
 * 
 *         self.type = type
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_wire_types); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 70, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = (__Pyx_PySequence_ContainsTF(__pyx_v_type, __pyx_t_2, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 70, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_4 = (__pyx_t_3 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_1 = __pyx_t_4;
      goto __pyx_L3_bool_binop_done;
    }
    __pyx_t_3 = PyString_Check(__pyx_v_type); 
    __pyx_t_5 = (__pyx_t_3 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_5 = PyUnicode_Check(__pyx_v_type); 
    __pyx_t_3 = (__pyx_t_5 != 0);
    __pyx_t_4 = __pyx_t_3;
    __pyx_L6_bool_binop_done:;
    __pyx_t_3 = (__pyx_t_4 != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_1 = __pyx_t_3;
      goto __pyx_L3_bool_binop_done;
    }
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_ProtoEntity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 70, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyObject_IsSubclass(__pyx_v_type, __pyx_t_2); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 70, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_4 = (__pyx_t_3 != 0);
    __pyx_t_1 = __pyx_t_4;
    __pyx_L3_bool_binop_done:;
    if (unlikely(!__pyx_t_1)) {
      __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_invalid_type_s, __pyx_v_type); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 70, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      PyErr_SetObject(PyExc_AssertionError, __pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 70, __pyx_L1_error)
    }
  }
  #endif

  /* "cprotobuf/internal.pyx":72
 *         assert type in wire_types or isinstance(type, (str, unicode)) or issubclass(type, ProtoEntity), 'invalid type %s' % type
 * 
 *         self.type = type             # <<<<<<<<<<<<<<
 *         self.index = index
 *         self.required = required
 */
  __Pyx_INCREF(__pyx_v_type);
  __Pyx_GIVEREF(__pyx_v_type);
  __Pyx_GOTREF(__pyx_v_self->type);
  __Pyx_DECREF(__pyx_v_self->type);
  __pyx_v_self->type = __pyx_v_type;

  /* "cprotobuf/internal.pyx":73
 * 
 *         self.type = type
 *         self.index = index             # <<<<<<<<<<<<<<
 *         self.required = required
 *         self.repeated = repeated
 */
  __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_v_index); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 73, __pyx_L1_error)
  __pyx_v_self->index = __pyx_t_6;

  /* "cprotobuf/internal.pyx":74
 *         self.type = type
 *         self.index = index
 *         self.required = required             # <<<<<<<<<<<<<<
 *         self.repeated = repeated
 *         self.packed = packed
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_required); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 74, __pyx_L1_error)
  __pyx_v_self->required = __pyx_t_1;

  /* "cprotobuf/internal.pyx":75
 *         self.index = index
 *         self.required = required
 *         self.repeated = repeated             # <<<<<<<<<<<<<<
 *         self.packed = packed
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_repeated); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 75, __pyx_L1_error)
  __pyx_v_self->repeated = __pyx_t_1;

  /* "cprotobuf/internal.pyx":76
 *         self.required = required
 *         self.repeated = repeated
 *         self.packed = packed             # <<<<<<<<<<<<<<
 * 
 *         if inspect.isclass(self.type) and issubclass(self.type, ProtoEntity):
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_packed); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 76, __pyx_L1_error)
  __pyx_v_self->packed = __pyx_t_1;

  /* "cprotobuf/internal.pyx":78
 *         self.packed = packed
 * 
 *         if inspect.isclass(self.type) and issubclass(self.type, ProtoEntity):             # <<<<<<<<<<<<<<
 *             self.klass = self.type
 *         elif self.type not in default_objects:
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_inspect); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_isclass); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
    }
  }
  if (!__pyx_t_7) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_v_self->type); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_8)) {
      PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_v_self->type};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
      PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_v_self->type};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    {
      __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 78, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
      __Pyx_INCREF(__pyx_v_self->type);
      __Pyx_GIVEREF(__pyx_v_self->type);
      PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_v_self->type);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_4) {
  } else {
    __pyx_t_1 = __pyx_t_4;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = __pyx_v_self->type;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_ProtoEntity); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = PyObject_IsSubclass(__pyx_t_2, __pyx_t_8); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_t_4 != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":79
 * 
 *         if inspect.isclass(self.type) and issubclass(self.type, ProtoEntity):
 *             self.klass = self.type             # <<<<<<<<<<<<<<
 *         elif self.type not in default_objects:
 *             # sub message specified as name
 */
    __pyx_t_8 = __pyx_v_self->type;
    __Pyx_INCREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_8);
    __Pyx_GOTREF(__pyx_v_self->klass);
    __Pyx_DECREF(__pyx_v_self->klass);
    __pyx_v_self->klass = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "cprotobuf/internal.pyx":78
 *         self.packed = packed
 * 
 *         if inspect.isclass(self.type) and issubclass(self.type, ProtoEntity):             # <<<<<<<<<<<<<<
 *             self.klass = self.type
 *         elif self.type not in default_objects:
 */
    goto __pyx_L8;
  }

  /* "cprotobuf/internal.pyx":80
 *         if inspect.isclass(self.type) and issubclass(self.type, ProtoEntity):
 *             self.klass = self.type
 *         elif self.type not in default_objects:             # <<<<<<<<<<<<<<
 *             # sub message specified as name
 *             self.klass = self.type
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_default_objects); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_1 = (__Pyx_PySequence_ContainsTF(__pyx_v_self->type, __pyx_t_8, Py_NE)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {

    /* "cprotobuf/internal.pyx":82
 *         elif self.type not in default_objects:
 *             # sub message specified as name
 *             self.klass = self.type             # <<<<<<<<<<<<<<
 *         else:
 *             self.klass = None
 */
    __pyx_t_8 = __pyx_v_self->type;
    __Pyx_INCREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_8);
    __Pyx_GOTREF(__pyx_v_self->klass);
    __Pyx_DECREF(__pyx_v_self->klass);
    __pyx_v_self->klass = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "cprotobuf/internal.pyx":80
 *         if inspect.isclass(self.type) and issubclass(self.type, ProtoEntity):
 *             self.klass = self.type
 *         elif self.type not in default_objects:             # <<<<<<<<<<<<<<
 *             # sub message specified as name
 *             self.klass = self.type
 */
    goto __pyx_L8;
  }

  /* "cprotobuf/internal.pyx":84
 *             self.klass = self.type
 *         else:
 *             self.klass = None             # <<<<<<<<<<<<<<
 * 
 *         self.default = default or default_objects.get(self.type)
 */
  /*else*/ {
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->klass);
    __Pyx_DECREF(__pyx_v_self->klass);
    __pyx_v_self->klass = Py_None;
  }
  __pyx_L8:;

  /* "cprotobuf/internal.pyx":86
 *             self.klass = None
 * 
 *         self.default = default or default_objects.get(self.type)             # <<<<<<<<<<<<<<
 * 
 *         self.wire_type = self.get_wire_type()
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_default); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 86, __pyx_L1_error)
  if (!__pyx_t_3) {
  } else {
    __Pyx_INCREF(__pyx_v_default);
    __pyx_t_8 = __pyx_v_default;
    goto __pyx_L11_bool_binop_done;
  }
  __pyx_t_9 = __Pyx_GetModuleGlobalName(__pyx_n_s_default_objects); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_get); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
    }
  }
  if (!__pyx_t_9) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_self->type); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 86, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_7)) {
      PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_v_self->type};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 86, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
      PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_v_self->type};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 86, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    {
      __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 86, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
      __Pyx_INCREF(__pyx_v_self->type);
      __Pyx_GIVEREF(__pyx_v_self->type);
      PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_v_self->type);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_10, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 86, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_8 = __pyx_t_2;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_L11_bool_binop_done:;
  __Pyx_GIVEREF(__pyx_t_8);
  __Pyx_GOTREF(__pyx_v_self->__pyx_default);
  __Pyx_DECREF(__pyx_v_self->__pyx_default);
  __pyx_v_self->__pyx_default = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "cprotobuf/internal.pyx":88
 *         self.default = default or default_objects.get(self.type)
 * 
 *         self.wire_type = self.get_wire_type()             # <<<<<<<<<<<<<<
 *         self.encoder = self.get_encoder()
 *         self.decoder = self.get_decoder()
 */
  __pyx_v_self->wire_type = ((struct __pyx_vtabstruct_9cprotobuf_8internal_Field *)__pyx_v_self->__pyx_vtab)->get_wire_type(__pyx_v_self);

  /* "cprotobuf/internal.pyx":89
 * 
 *         self.wire_type = self.get_wire_type()
 *         self.encoder = self.get_encoder()             # <<<<<<<<<<<<<<
 *         self.decoder = self.get_decoder()
 * 
 */
  __pyx_v_self->encoder = ((struct __pyx_vtabstruct_9cprotobuf_8internal_Field *)__pyx_v_self->__pyx_vtab)->get_encoder(__pyx_v_self);

  /* "cprotobuf/internal.pyx":90
 *         self.wire_type = self.get_wire_type()
 *         self.encoder = self.get_encoder()
 *         self.decoder = self.get_decoder()             # <<<<<<<<<<<<<<
 * 
 *         if self.packed:
 */
  __pyx_v_self->decoder = ((struct __pyx_vtabstruct_9cprotobuf_8internal_Field *)__pyx_v_self->__pyx_vtab)->get_decoder(__pyx_v_self);

  /* "cprotobuf/internal.pyx":92
 *         self.decoder = self.get_decoder()
 * 
 *         if self.packed:             # <<<<<<<<<<<<<<
 *             assert self.repeated, 'packed must be used with repeated'
 * 
 */
  __pyx_t_3 = (__pyx_v_self->packed != 0);
  if (__pyx_t_3) {

    /* "cprotobuf/internal.pyx":93
 * 
 *         if self.packed:
 *             assert self.repeated, 'packed must be used with repeated'             # <<<<<<<<<<<<<<
 * 
 *     cdef resolve_klass(self):
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!(__pyx_v_self->repeated != 0))) {
        PyErr_SetObject(PyExc_AssertionError, __pyx_kp_s_packed_must_be_used_with_repeate);
        __PYX_ERR(0, 93, __pyx_L1_error)
      }
    }
    #endif

    /* "cprotobuf/internal.pyx":92
 *         self.decoder = self.get_decoder()
 * 
 *         if self.packed:             # <<<<<<<<<<<<<<
 *             assert self.repeated, 'packed must be used with repeated'
 * 
 */
  }

  /* "cprotobuf/internal.pyx":69
 *     cdef Decoder decoder
 * 
 *     def __init__(self, type, index, required=True, repeated=False, packed=False, default=None):             # <<<<<<<<<<<<<<
 *         assert type in wire_types or isinstance(type, (str, unicode)) or issubclass(type, ProtoEntity), 'invalid type %s' % type
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("cprotobuf.internal.Field.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":95
 *             assert self.repeated, 'packed must be used with repeated'
 * 
 *     cdef resolve_klass(self):             # <<<<<<<<<<<<<<
 *         if self.klass and isinstance(self.klass, (str, unicode)):
 *             self.klass = get_proto(self.klass)
 */

static PyObject *__pyx_f_9cprotobuf_8internal_5Field_resolve_klass(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("resolve_klass", 0);

  /* "cprotobuf/internal.pyx":96
 * 
 *     cdef resolve_klass(self):
 *         if self.klass and isinstance(self.klass, (str, unicode)):             # <<<<<<<<<<<<<<
 *             self.klass = get_proto(self.klass)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_self->klass); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 96, __pyx_L1_error)
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __pyx_v_self->klass;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = PyString_Check(__pyx_t_3); 
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (!__pyx_t_5) {
  } else {
    __pyx_t_2 = __pyx_t_5;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_3 = __pyx_v_self->klass;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_5 = PyUnicode_Check(__pyx_t_3); 
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4 = (__pyx_t_5 != 0);
  __pyx_t_2 = __pyx_t_4;
  __pyx_L6_bool_binop_done:;
  __pyx_t_4 = (__pyx_t_2 != 0);
  __pyx_t_1 = __pyx_t_4;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":97
 *     cdef resolve_klass(self):
 *         if self.klass and isinstance(self.klass, (str, unicode)):
 *             self.klass = get_proto(self.klass)             # <<<<<<<<<<<<<<
 * 
 *     def __get__(self, instance, type):
 */
    __pyx_t_3 = __pyx_v_self->klass;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_6 = __pyx_f_9cprotobuf_8internal_get_proto(__pyx_t_3, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 97, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GIVEREF(__pyx_t_6);
    __Pyx_GOTREF(__pyx_v_self->klass);
    __Pyx_DECREF(__pyx_v_self->klass);
    __pyx_v_self->klass = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "cprotobuf/internal.pyx":96
 * 
 *     cdef resolve_klass(self):
 *         if self.klass and isinstance(self.klass, (str, unicode)):             # <<<<<<<<<<<<<<
 *             self.klass = get_proto(self.klass)
 * 
 */
  }

  /* "cprotobuf/internal.pyx":95
 *             assert self.repeated, 'packed must be used with repeated'
 * 
 *     cdef resolve_klass(self):             # <<<<<<<<<<<<<<
 *         if self.klass and isinstance(self.klass, (str, unicode)):
 *             self.klass = get_proto(self.klass)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cprotobuf.internal.Field.resolve_klass", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":99
 *             self.klass = get_proto(self.klass)
 * 
 *     def __get__(self, instance, type):             # <<<<<<<<<<<<<<
 *         if not instance:
 *             return self
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_3__get__(PyObject *__pyx_v_self, PyObject *__pyx_v_instance, PyObject *__pyx_v_type); /*proto*/
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_3__get__(PyObject *__pyx_v_self, PyObject *__pyx_v_instance, PyObject *__pyx_v_type) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field_2__get__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self), ((PyObject *)__pyx_v_instance), ((PyObject *)__pyx_v_type));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_2__get__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, PyObject *__pyx_v_instance, CYTHON_UNUSED PyObject *__pyx_v_type) {
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "cprotobuf/internal.pyx":100
 * 
 *     def __get__(self, instance, type):
 *         if not instance:             # <<<<<<<<<<<<<<
 *             return self
 *         value = None
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_instance); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 100, __pyx_L1_error)
  __pyx_t_2 = ((!__pyx_t_1) != 0);
  if (__pyx_t_2) {

    /* "cprotobuf/internal.pyx":101
 *     def __get__(self, instance, type):
 *         if not instance:
 *             return self             # <<<<<<<<<<<<<<
 *         value = None
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __pyx_r = ((PyObject *)__pyx_v_self);
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":100
 * 
 *     def __get__(self, instance, type):
 *         if not instance:             # <<<<<<<<<<<<<<
 *             return self
 *         value = None
 */
  }

  /* "cprotobuf/internal.pyx":102
 *         if not instance:
 *             return self
 *         value = None             # <<<<<<<<<<<<<<
 * 
 *         if self.repeated:
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_value = Py_None;

  /* "cprotobuf/internal.pyx":104
 *         value = None
 * 
 *         if self.repeated:             # <<<<<<<<<<<<<<
 *             if self.klass:
 *                 self.resolve_klass()
 */
  __pyx_t_2 = (__pyx_v_self->repeated != 0);
  if (__pyx_t_2) {

    /* "cprotobuf/internal.pyx":105
 * 
 *         if self.repeated:
 *             if self.klass:             # <<<<<<<<<<<<<<
 *                 self.resolve_klass()
 *                 value = RepeatedContainer(self.klass)
 */
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_self->klass); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 105, __pyx_L1_error)
    if (__pyx_t_2) {

      /* "cprotobuf/internal.pyx":106
 *         if self.repeated:
 *             if self.klass:
 *                 self.resolve_klass()             # <<<<<<<<<<<<<<
 *                 value = RepeatedContainer(self.klass)
 *             else:
 */
      __pyx_t_3 = ((struct __pyx_vtabstruct_9cprotobuf_8internal_Field *)__pyx_v_self->__pyx_vtab)->resolve_klass(__pyx_v_self); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 106, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "cprotobuf/internal.pyx":107
 *             if self.klass:
 *                 self.resolve_klass()
 *                 value = RepeatedContainer(self.klass)             # <<<<<<<<<<<<<<
 *             else:
 *                 value = []
 */
      __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 107, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_v_self->klass);
      __Pyx_GIVEREF(__pyx_v_self->klass);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->klass);
      __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9cprotobuf_8internal_RepeatedContainer), __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 107, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "cprotobuf/internal.pyx":105
 * 
 *         if self.repeated:
 *             if self.klass:             # <<<<<<<<<<<<<<
 *                 self.resolve_klass()
 *                 value = RepeatedContainer(self.klass)
 */
      goto __pyx_L5;
    }

    /* "cprotobuf/internal.pyx":109
 *                 value = RepeatedContainer(self.klass)
 *             else:
 *                 value = []             # <<<<<<<<<<<<<<
 *             setattr(instance, self.name, value)
 *         elif self.klass:
 */
    /*else*/ {
      __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 109, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_4);
      __pyx_t_4 = 0;
    }
    __pyx_L5:;

    /* "cprotobuf/internal.pyx":110
 *             else:
 *                 value = []
 *             setattr(instance, self.name, value)             # <<<<<<<<<<<<<<
 *         elif self.klass:
 *             self.resolve_klass()
 */
    __pyx_t_4 = __pyx_v_self->name;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = PyObject_SetAttr(__pyx_v_instance, __pyx_t_4, __pyx_v_value); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 110, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "cprotobuf/internal.pyx":104
 *         value = None
 * 
 *         if self.repeated:             # <<<<<<<<<<<<<<
 *             if self.klass:
 *                 self.resolve_klass()
 */
    goto __pyx_L4;
  }

  /* "cprotobuf/internal.pyx":111
 *                 value = []
 *             setattr(instance, self.name, value)
 *         elif self.klass:             # <<<<<<<<<<<<<<
 *             self.resolve_klass()
 *             value = self.klass()
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_self->klass); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 111, __pyx_L1_error)
  if (__pyx_t_2) {

    /* "cprotobuf/internal.pyx":112
 *             setattr(instance, self.name, value)
 *         elif self.klass:
 *             self.resolve_klass()             # <<<<<<<<<<<<<<
 *             value = self.klass()
 *             setattr(instance, self.name, value)
 */
    __pyx_t_4 = ((struct __pyx_vtabstruct_9cprotobuf_8internal_Field *)__pyx_v_self->__pyx_vtab)->resolve_klass(__pyx_v_self); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "cprotobuf/internal.pyx":113
 *         elif self.klass:
 *             self.resolve_klass()
 *             value = self.klass()             # <<<<<<<<<<<<<<
 *             setattr(instance, self.name, value)
 *         else:
 */
    __Pyx_INCREF(__pyx_v_self->klass);
    __pyx_t_3 = __pyx_v_self->klass; __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_6) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 113, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else {
      __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 113, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "cprotobuf/internal.pyx":114
 *             self.resolve_klass()
 *             value = self.klass()
 *             setattr(instance, self.name, value)             # <<<<<<<<<<<<<<
 *         else:
 *             value = self.default
 */
    __pyx_t_4 = __pyx_v_self->name;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = PyObject_SetAttr(__pyx_v_instance, __pyx_t_4, __pyx_v_value); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 114, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "cprotobuf/internal.pyx":111
 *                 value = []
 *             setattr(instance, self.name, value)
 *         elif self.klass:             # <<<<<<<<<<<<<<
 *             self.resolve_klass()
 *             value = self.klass()
 */
    goto __pyx_L4;
  }

  /* "cprotobuf/internal.pyx":116
 *             setattr(instance, self.name, value)
 *         else:
 *             value = self.default             # <<<<<<<<<<<<<<
 *         return value
 * 
 */
  /*else*/ {
    __pyx_t_4 = __pyx_v_self->__pyx_default;
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_DECREF_SET(__pyx_v_value, __pyx_t_4);
    __pyx_t_4 = 0;
  }
  __pyx_L4:;

  /* "cprotobuf/internal.pyx":117
 *         else:
 *             value = self.default
 *         return value             # <<<<<<<<<<<<<<
 * 
 *     cdef unsigned char get_wire_type(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_value);
  __pyx_r = __pyx_v_value;
  goto __pyx_L0;

  /* "cprotobuf/internal.pyx":99
 *             self.klass = get_proto(self.klass)
 * 
 *     def __get__(self, instance, type):             # <<<<<<<<<<<<<<
 *         if not instance:
 *             return self
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cprotobuf.internal.Field.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":119
 *         return value
 * 
 *     cdef unsigned char get_wire_type(self):             # <<<<<<<<<<<<<<
 *         if self.packed:
 *             return 2
 */

static unsigned char __pyx_f_9cprotobuf_8internal_5Field_get_wire_type(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self) {
  unsigned char __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  unsigned char __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("get_wire_type", 0);

  /* "cprotobuf/internal.pyx":120
 * 
 *     cdef unsigned char get_wire_type(self):
 *         if self.packed:             # <<<<<<<<<<<<<<
 *             return 2
 *         try:
 */
  __pyx_t_1 = (__pyx_v_self->packed != 0);
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":121
 *     cdef unsigned char get_wire_type(self):
 *         if self.packed:
 *             return 2             # <<<<<<<<<<<<<<
 *         try:
 *             return wire_types[self.type]
 */
    __pyx_r = 2;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":120
 * 
 *     cdef unsigned char get_wire_type(self):
 *         if self.packed:             # <<<<<<<<<<<<<<
 *             return 2
 *         try:
 */
  }

  /* "cprotobuf/internal.pyx":122
 *         if self.packed:
 *             return 2
 *         try:             # <<<<<<<<<<<<<<
 *             return wire_types[self.type]
 *         except KeyError:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    /*try:*/ {

      /* "cprotobuf/internal.pyx":123
 *             return 2
 *         try:
 *             return wire_types[self.type]             # <<<<<<<<<<<<<<
 *         except KeyError:
 *             return 2
 */
      __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_wire_types); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 123, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = PyObject_GetItem(__pyx_t_5, __pyx_v_self->type); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 123, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_7 = __Pyx_PyInt_As_unsigned_char(__pyx_t_6); if (unlikely((__pyx_t_7 == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(0, 123, __pyx_L4_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_r = __pyx_t_7;
      goto __pyx_L8_try_return;

      /* "cprotobuf/internal.pyx":122
 *         if self.packed:
 *             return 2
 *         try:             # <<<<<<<<<<<<<<
 *             return wire_types[self.type]
 *         except KeyError:
 */
    }
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "cprotobuf/internal.pyx":124
 *         try:
 *             return wire_types[self.type]
 *         except KeyError:             # <<<<<<<<<<<<<<
 *             return 2
 * 
 */
    __pyx_t_8 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
    if (__pyx_t_8) {
      __Pyx_AddTraceback("cprotobuf.internal.Field.get_wire_type", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_5, &__pyx_t_9) < 0) __PYX_ERR(0, 124, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_9);

      /* "cprotobuf/internal.pyx":125
 *             return wire_types[self.type]
 *         except KeyError:
 *             return 2             # <<<<<<<<<<<<<<
 * 
 *     cdef Encoder get_encoder(self):
 */
      __pyx_r = 2;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L7_except_return;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;

    /* "cprotobuf/internal.pyx":122
 *         if self.packed:
 *             return 2
 *         try:             # <<<<<<<<<<<<<<
 *             return wire_types[self.type]
 *         except KeyError:
 */
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L1_error;
    __pyx_L8_try_return:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L0;
    __pyx_L7_except_return:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L0;
  }

  /* "cprotobuf/internal.pyx":119
 *         return value
 * 
 *     cdef unsigned char get_wire_type(self):             # <<<<<<<<<<<<<<
 *         if self.packed:
 *             return 2
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_WriteUnraisable("cprotobuf.internal.Field.get_wire_type", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":127
 *             return 2
 * 
 *     cdef Encoder get_encoder(self):             # <<<<<<<<<<<<<<
 *         if self.type == 'int32':
 *             return encode_int64     # compatible with official protobuf
 */

static __pyx_t_9cprotobuf_8internal_Encoder __pyx_f_9cprotobuf_8internal_5Field_get_encoder(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self) {
  __pyx_t_9cprotobuf_8internal_Encoder __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_encoder", 0);

  /* "cprotobuf/internal.pyx":128
 * 
 *     cdef Encoder get_encoder(self):
 *         if self.type == 'int32':             # <<<<<<<<<<<<<<
 *             return encode_int64     # compatible with official protobuf
 *         if self.type == 'int64':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_int32, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":129
 *     cdef Encoder get_encoder(self):
 *         if self.type == 'int32':
 *             return encode_int64     # compatible with official protobuf             # <<<<<<<<<<<<<<
 *         if self.type == 'int64':
 *             return encode_int64
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_encode_int64;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":128
 * 
 *     cdef Encoder get_encoder(self):
 *         if self.type == 'int32':             # <<<<<<<<<<<<<<
 *             return encode_int64     # compatible with official protobuf
 *         if self.type == 'int64':
 */
  }

  /* "cprotobuf/internal.pyx":130
 *         if self.type == 'int32':
 *             return encode_int64     # compatible with official protobuf
 *         if self.type == 'int64':             # <<<<<<<<<<<<<<
 *             return encode_int64
 *         if self.type == 'sint32':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_int64, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 130, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":131
 *             return encode_int64     # compatible with official protobuf
 *         if self.type == 'int64':
 *             return encode_int64             # <<<<<<<<<<<<<<
 *         if self.type == 'sint32':
 *             return encode_sint32
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_encode_int64;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":130
 *         if self.type == 'int32':
 *             return encode_int64     # compatible with official protobuf
 *         if self.type == 'int64':             # <<<<<<<<<<<<<<
 *             return encode_int64
 *         if self.type == 'sint32':
 */
  }

  /* "cprotobuf/internal.pyx":132
 *         if self.type == 'int64':
 *             return encode_int64
 *         if self.type == 'sint32':             # <<<<<<<<<<<<<<
 *             return encode_sint32
 *         if self.type == 'sint64':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_sint32, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 132, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":133
 *             return encode_int64
 *         if self.type == 'sint32':
 *             return encode_sint32             # <<<<<<<<<<<<<<
 *         if self.type == 'sint64':
 *             return encode_sint64
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_encode_sint32;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":132
 *         if self.type == 'int64':
 *             return encode_int64
 *         if self.type == 'sint32':             # <<<<<<<<<<<<<<
 *             return encode_sint32
 *         if self.type == 'sint64':
 */
  }

  /* "cprotobuf/internal.pyx":134
 *         if self.type == 'sint32':
 *             return encode_sint32
 *         if self.type == 'sint64':             # <<<<<<<<<<<<<<
 *             return encode_sint64
 *         if self.type == 'uint32':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_sint64, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 134, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":135
 *             return encode_sint32
 *         if self.type == 'sint64':
 *             return encode_sint64             # <<<<<<<<<<<<<<
 *         if self.type == 'uint32':
 *             return encode_uint32
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_encode_sint64;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":134
 *         if self.type == 'sint32':
 *             return encode_sint32
 *         if self.type == 'sint64':             # <<<<<<<<<<<<<<
 *             return encode_sint64
 *         if self.type == 'uint32':
 */
  }

  /* "cprotobuf/internal.pyx":136
 *         if self.type == 'sint64':
 *             return encode_sint64
 *         if self.type == 'uint32':             # <<<<<<<<<<<<<<
 *             return encode_uint32
 *         if self.type == 'uint64':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_uint32, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 136, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":137
 *             return encode_sint64
 *         if self.type == 'uint32':
 *             return encode_uint32             # <<<<<<<<<<<<<<
 *         if self.type == 'uint64':
 *             return encode_uint64
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_encode_uint32;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":136
 *         if self.type == 'sint64':
 *             return encode_sint64
 *         if self.type == 'uint32':             # <<<<<<<<<<<<<<
 *             return encode_uint32
 *         if self.type == 'uint64':
 */
  }

  /* "cprotobuf/internal.pyx":138
 *         if self.type == 'uint32':
 *             return encode_uint32
 *         if self.type == 'uint64':             # <<<<<<<<<<<<<<
 *             return encode_uint64
 *         if self.type == 'bool':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_uint64, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 138, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":139
 *             return encode_uint32
 *         if self.type == 'uint64':
 *             return encode_uint64             # <<<<<<<<<<<<<<
 *         if self.type == 'bool':
 *             return encode_bool
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_encode_uint64;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":138
 *         if self.type == 'uint32':
 *             return encode_uint32
 *         if self.type == 'uint64':             # <<<<<<<<<<<<<<
 *             return encode_uint64
 *         if self.type == 'bool':
 */
  }

  /* "cprotobuf/internal.pyx":140
 *         if self.type == 'uint64':
 *             return encode_uint64
 *         if self.type == 'bool':             # <<<<<<<<<<<<<<
 *             return encode_bool
 *         if self.type == 'enum':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_bool, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 140, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":141
 *             return encode_uint64
 *         if self.type == 'bool':
 *             return encode_bool             # <<<<<<<<<<<<<<
 *         if self.type == 'enum':
 *             return encode_int64     # compatible with official protobuf
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_encode_bool;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":140
 *         if self.type == 'uint64':
 *             return encode_uint64
 *         if self.type == 'bool':             # <<<<<<<<<<<<<<
 *             return encode_bool
 *         if self.type == 'enum':
 */
  }

  /* "cprotobuf/internal.pyx":142
 *         if self.type == 'bool':
 *             return encode_bool
 *         if self.type == 'enum':             # <<<<<<<<<<<<<<
 *             return encode_int64     # compatible with official protobuf
 *         if self.type == 'fixed64':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_enum, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 142, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":143
 *             return encode_bool
 *         if self.type == 'enum':
 *             return encode_int64     # compatible with official protobuf             # <<<<<<<<<<<<<<
 *         if self.type == 'fixed64':
 *             return encode_fixed64
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_encode_int64;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":142
 *         if self.type == 'bool':
 *             return encode_bool
 *         if self.type == 'enum':             # <<<<<<<<<<<<<<
 *             return encode_int64     # compatible with official protobuf
 *         if self.type == 'fixed64':
 */
  }

  /* "cprotobuf/internal.pyx":144
 *         if self.type == 'enum':
 *             return encode_int64     # compatible with official protobuf
 *         if self.type == 'fixed64':             # <<<<<<<<<<<<<<
 *             return encode_fixed64
 *         if self.type == 'sfixed64':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_fixed64, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 144, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":145
 *             return encode_int64     # compatible with official protobuf
 *         if self.type == 'fixed64':
 *             return encode_fixed64             # <<<<<<<<<<<<<<
 *         if self.type == 'sfixed64':
 *             return encode_sfixed64
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_encode_fixed64;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":144
 *         if self.type == 'enum':
 *             return encode_int64     # compatible with official protobuf
 *         if self.type == 'fixed64':             # <<<<<<<<<<<<<<
 *             return encode_fixed64
 *         if self.type == 'sfixed64':
 */
  }

  /* "cprotobuf/internal.pyx":146
 *         if self.type == 'fixed64':
 *             return encode_fixed64
 *         if self.type == 'sfixed64':             # <<<<<<<<<<<<<<
 *             return encode_sfixed64
 *         if self.type == 'fixed32':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_sfixed64, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 146, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":147
 *             return encode_fixed64
 *         if self.type == 'sfixed64':
 *             return encode_sfixed64             # <<<<<<<<<<<<<<
 *         if self.type == 'fixed32':
 *             return encode_fixed32
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_encode_sfixed64;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":146
 *         if self.type == 'fixed64':
 *             return encode_fixed64
 *         if self.type == 'sfixed64':             # <<<<<<<<<<<<<<
 *             return encode_sfixed64
 *         if self.type == 'fixed32':
 */
  }

  /* "cprotobuf/internal.pyx":148
 *         if self.type == 'sfixed64':
 *             return encode_sfixed64
 *         if self.type == 'fixed32':             # <<<<<<<<<<<<<<
 *             return encode_fixed32
 *         if self.type == 'sfixed32':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_fixed32, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 148, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":149
 *             return encode_sfixed64
 *         if self.type == 'fixed32':
 *             return encode_fixed32             # <<<<<<<<<<<<<<
 *         if self.type == 'sfixed32':
 *             return encode_sfixed32
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_encode_fixed32;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":148
 *         if self.type == 'sfixed64':
 *             return encode_sfixed64
 *         if self.type == 'fixed32':             # <<<<<<<<<<<<<<
 *             return encode_fixed32
 *         if self.type == 'sfixed32':
 */
  }

  /* "cprotobuf/internal.pyx":150
 *         if self.type == 'fixed32':
 *             return encode_fixed32
 *         if self.type == 'sfixed32':             # <<<<<<<<<<<<<<
 *             return encode_sfixed32
 *         if self.type == 'string':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_sfixed32, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 150, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":151
 *             return encode_fixed32
 *         if self.type == 'sfixed32':
 *             return encode_sfixed32             # <<<<<<<<<<<<<<
 *         if self.type == 'string':
 *             return encode_string
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_encode_sfixed32;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":150
 *         if self.type == 'fixed32':
 *             return encode_fixed32
 *         if self.type == 'sfixed32':             # <<<<<<<<<<<<<<
 *             return encode_sfixed32
 *         if self.type == 'string':
 */
  }

  /* "cprotobuf/internal.pyx":152
 *         if self.type == 'sfixed32':
 *             return encode_sfixed32
 *         if self.type == 'string':             # <<<<<<<<<<<<<<
 *             return encode_string
 *         if self.type == 'bytes':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_string, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 152, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":153
 *             return encode_sfixed32
 *         if self.type == 'string':
 *             return encode_string             # <<<<<<<<<<<<<<
 *         if self.type == 'bytes':
 *             return encode_bytes
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_encode_string;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":152
 *         if self.type == 'sfixed32':
 *             return encode_sfixed32
 *         if self.type == 'string':             # <<<<<<<<<<<<<<
 *             return encode_string
 *         if self.type == 'bytes':
 */
  }

  /* "cprotobuf/internal.pyx":154
 *         if self.type == 'string':
 *             return encode_string
 *         if self.type == 'bytes':             # <<<<<<<<<<<<<<
 *             return encode_bytes
 *         if self.type == 'float':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_bytes, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 154, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":155
 *             return encode_string
 *         if self.type == 'bytes':
 *             return encode_bytes             # <<<<<<<<<<<<<<
 *         if self.type == 'float':
 *             return encode_float
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_encode_bytes;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":154
 *         if self.type == 'string':
 *             return encode_string
 *         if self.type == 'bytes':             # <<<<<<<<<<<<<<
 *             return encode_bytes
 *         if self.type == 'float':
 */
  }

  /* "cprotobuf/internal.pyx":156
 *         if self.type == 'bytes':
 *             return encode_bytes
 *         if self.type == 'float':             # <<<<<<<<<<<<<<
 *             return encode_float
 *         if self.type == 'double':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_float, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 156, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":157
 *             return encode_bytes
 *         if self.type == 'float':
 *             return encode_float             # <<<<<<<<<<<<<<
 *         if self.type == 'double':
 *             return encode_double
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_encode_float;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":156
 *         if self.type == 'bytes':
 *             return encode_bytes
 *         if self.type == 'float':             # <<<<<<<<<<<<<<
 *             return encode_float
 *         if self.type == 'double':
 */
  }

  /* "cprotobuf/internal.pyx":158
 *         if self.type == 'float':
 *             return encode_float
 *         if self.type == 'double':             # <<<<<<<<<<<<<<
 *             return encode_double
 *         return encode_subobject
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_double, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 158, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":159
 *             return encode_float
 *         if self.type == 'double':
 *             return encode_double             # <<<<<<<<<<<<<<
 *         return encode_subobject
 * 
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_encode_double;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":158
 *         if self.type == 'float':
 *             return encode_float
 *         if self.type == 'double':             # <<<<<<<<<<<<<<
 *             return encode_double
 *         return encode_subobject
 */
  }

  /* "cprotobuf/internal.pyx":160
 *         if self.type == 'double':
 *             return encode_double
 *         return encode_subobject             # <<<<<<<<<<<<<<
 * 
 *     cdef Decoder get_decoder(self):
 */
  __pyx_r = __pyx_f_9cprotobuf_8internal_encode_subobject;
  goto __pyx_L0;

  /* "cprotobuf/internal.pyx":127
 *             return 2
 * 
 *     cdef Encoder get_encoder(self):             # <<<<<<<<<<<<<<
 *         if self.type == 'int32':
 *             return encode_int64     # compatible with official protobuf
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("cprotobuf.internal.Field.get_encoder", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":162
 *         return encode_subobject
 * 
 *     cdef Decoder get_decoder(self):             # <<<<<<<<<<<<<<
 *         if self.type == 'int32':
 *             return decode_int64     # compatible with official protobuf
 */

static __pyx_t_9cprotobuf_8internal_Decoder __pyx_f_9cprotobuf_8internal_5Field_get_decoder(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self) {
  __pyx_t_9cprotobuf_8internal_Decoder __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_decoder", 0);

  /* "cprotobuf/internal.pyx":163
 * 
 *     cdef Decoder get_decoder(self):
 *         if self.type == 'int32':             # <<<<<<<<<<<<<<
 *             return decode_int64     # compatible with official protobuf
 *         if self.type == 'int64':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_int32, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 163, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":164
 *     cdef Decoder get_decoder(self):
 *         if self.type == 'int32':
 *             return decode_int64     # compatible with official protobuf             # <<<<<<<<<<<<<<
 *         if self.type == 'int64':
 *             return decode_int64
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_decode_int64;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":163
 * 
 *     cdef Decoder get_decoder(self):
 *         if self.type == 'int32':             # <<<<<<<<<<<<<<
 *             return decode_int64     # compatible with official protobuf
 *         if self.type == 'int64':
 */
  }

  /* "cprotobuf/internal.pyx":165
 *         if self.type == 'int32':
 *             return decode_int64     # compatible with official protobuf
 *         if self.type == 'int64':             # <<<<<<<<<<<<<<
 *             return decode_int64
 *         if self.type == 'sint32':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_int64, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 165, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":166
 *             return decode_int64     # compatible with official protobuf
 *         if self.type == 'int64':
 *             return decode_int64             # <<<<<<<<<<<<<<
 *         if self.type == 'sint32':
 *             return decode_sint32
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_decode_int64;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":165
 *         if self.type == 'int32':
 *             return decode_int64     # compatible with official protobuf
 *         if self.type == 'int64':             # <<<<<<<<<<<<<<
 *             return decode_int64
 *         if self.type == 'sint32':
 */
  }

  /* "cprotobuf/internal.pyx":167
 *         if self.type == 'int64':
 *             return decode_int64
 *         if self.type == 'sint32':             # <<<<<<<<<<<<<<
 *             return decode_sint32
 *         if self.type == 'sint64':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_sint32, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 167, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":168
 *             return decode_int64
 *         if self.type == 'sint32':
 *             return decode_sint32             # <<<<<<<<<<<<<<
 *         if self.type == 'sint64':
 *             return decode_sint64
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_decode_sint32;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":167
 *         if self.type == 'int64':
 *             return decode_int64
 *         if self.type == 'sint32':             # <<<<<<<<<<<<<<
 *             return decode_sint32
 *         if self.type == 'sint64':
 */
  }

  /* "cprotobuf/internal.pyx":169
 *         if self.type == 'sint32':
 *             return decode_sint32
 *         if self.type == 'sint64':             # <<<<<<<<<<<<<<
 *             return decode_sint64
 *         if self.type == 'uint32':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_sint64, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 169, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":170
 *             return decode_sint32
 *         if self.type == 'sint64':
 *             return decode_sint64             # <<<<<<<<<<<<<<
 *         if self.type == 'uint32':
 *             return decode_uint32
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_decode_sint64;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":169
 *         if self.type == 'sint32':
 *             return decode_sint32
 *         if self.type == 'sint64':             # <<<<<<<<<<<<<<
 *             return decode_sint64
 *         if self.type == 'uint32':
 */
  }

  /* "cprotobuf/internal.pyx":171
 *         if self.type == 'sint64':
 *             return decode_sint64
 *         if self.type == 'uint32':             # <<<<<<<<<<<<<<
 *             return decode_uint32
 *         if self.type == 'uint64':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_uint32, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 171, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":172
 *             return decode_sint64
 *         if self.type == 'uint32':
 *             return decode_uint32             # <<<<<<<<<<<<<<
 *         if self.type == 'uint64':
 *             return decode_uint64
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_decode_uint32;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":171
 *         if self.type == 'sint64':
 *             return decode_sint64
 *         if self.type == 'uint32':             # <<<<<<<<<<<<<<
 *             return decode_uint32
 *         if self.type == 'uint64':
 */
  }

  /* "cprotobuf/internal.pyx":173
 *         if self.type == 'uint32':
 *             return decode_uint32
 *         if self.type == 'uint64':             # <<<<<<<<<<<<<<
 *             return decode_uint64
 *         if self.type == 'bool':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_uint64, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 173, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":174
 *             return decode_uint32
 *         if self.type == 'uint64':
 *             return decode_uint64             # <<<<<<<<<<<<<<
 *         if self.type == 'bool':
 *             return decode_bool
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_decode_uint64;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":173
 *         if self.type == 'uint32':
 *             return decode_uint32
 *         if self.type == 'uint64':             # <<<<<<<<<<<<<<
 *             return decode_uint64
 *         if self.type == 'bool':
 */
  }

  /* "cprotobuf/internal.pyx":175
 *         if self.type == 'uint64':
 *             return decode_uint64
 *         if self.type == 'bool':             # <<<<<<<<<<<<<<
 *             return decode_bool
 *         if self.type == 'enum':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_bool, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 175, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":176
 *             return decode_uint64
 *         if self.type == 'bool':
 *             return decode_bool             # <<<<<<<<<<<<<<
 *         if self.type == 'enum':
 *             return decode_int64     # compatible with official protobuf
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_decode_bool;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":175
 *         if self.type == 'uint64':
 *             return decode_uint64
 *         if self.type == 'bool':             # <<<<<<<<<<<<<<
 *             return decode_bool
 *         if self.type == 'enum':
 */
  }

  /* "cprotobuf/internal.pyx":177
 *         if self.type == 'bool':
 *             return decode_bool
 *         if self.type == 'enum':             # <<<<<<<<<<<<<<
 *             return decode_int64     # compatible with official protobuf
 *         if self.type == 'fixed32':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_enum, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 177, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":178
 *             return decode_bool
 *         if self.type == 'enum':
 *             return decode_int64     # compatible with official protobuf             # <<<<<<<<<<<<<<
 *         if self.type == 'fixed32':
 *             return decode_fixed32
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_decode_int64;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":177
 *         if self.type == 'bool':
 *             return decode_bool
 *         if self.type == 'enum':             # <<<<<<<<<<<<<<
 *             return decode_int64     # compatible with official protobuf
 *         if self.type == 'fixed32':
 */
  }

  /* "cprotobuf/internal.pyx":179
 *         if self.type == 'enum':
 *             return decode_int64     # compatible with official protobuf
 *         if self.type == 'fixed32':             # <<<<<<<<<<<<<<
 *             return decode_fixed32
 *         if self.type == 'fixed64':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_fixed32, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 179, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":180
 *             return decode_int64     # compatible with official protobuf
 *         if self.type == 'fixed32':
 *             return decode_fixed32             # <<<<<<<<<<<<<<
 *         if self.type == 'fixed64':
 *             return decode_fixed64
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_decode_fixed32;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":179
 *         if self.type == 'enum':
 *             return decode_int64     # compatible with official protobuf
 *         if self.type == 'fixed32':             # <<<<<<<<<<<<<<
 *             return decode_fixed32
 *         if self.type == 'fixed64':
 */
  }

  /* "cprotobuf/internal.pyx":181
 *         if self.type == 'fixed32':
 *             return decode_fixed32
 *         if self.type == 'fixed64':             # <<<<<<<<<<<<<<
 *             return decode_fixed64
 *         if self.type == 'sfixed32':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_fixed64, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 181, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":182
 *             return decode_fixed32
 *         if self.type == 'fixed64':
 *             return decode_fixed64             # <<<<<<<<<<<<<<
 *         if self.type == 'sfixed32':
 *             return decode_sfixed32
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_decode_fixed64;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":181
 *         if self.type == 'fixed32':
 *             return decode_fixed32
 *         if self.type == 'fixed64':             # <<<<<<<<<<<<<<
 *             return decode_fixed64
 *         if self.type == 'sfixed32':
 */
  }

  /* "cprotobuf/internal.pyx":183
 *         if self.type == 'fixed64':
 *             return decode_fixed64
 *         if self.type == 'sfixed32':             # <<<<<<<<<<<<<<
 *             return decode_sfixed32
 *         if self.type == 'sfixed64':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_sfixed32, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 183, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":184
 *             return decode_fixed64
 *         if self.type == 'sfixed32':
 *             return decode_sfixed32             # <<<<<<<<<<<<<<
 *         if self.type == 'sfixed64':
 *             return decode_sfixed64
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_decode_sfixed32;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":183
 *         if self.type == 'fixed64':
 *             return decode_fixed64
 *         if self.type == 'sfixed32':             # <<<<<<<<<<<<<<
 *             return decode_sfixed32
 *         if self.type == 'sfixed64':
 */
  }

  /* "cprotobuf/internal.pyx":185
 *         if self.type == 'sfixed32':
 *             return decode_sfixed32
 *         if self.type == 'sfixed64':             # <<<<<<<<<<<<<<
 *             return decode_sfixed64
 *         if self.type == 'string':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_sfixed64, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 185, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":186
 *             return decode_sfixed32
 *         if self.type == 'sfixed64':
 *             return decode_sfixed64             # <<<<<<<<<<<<<<
 *         if self.type == 'string':
 *             return decode_string
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_decode_sfixed64;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":185
 *         if self.type == 'sfixed32':
 *             return decode_sfixed32
 *         if self.type == 'sfixed64':             # <<<<<<<<<<<<<<
 *             return decode_sfixed64
 *         if self.type == 'string':
 */
  }

  /* "cprotobuf/internal.pyx":187
 *         if self.type == 'sfixed64':
 *             return decode_sfixed64
 *         if self.type == 'string':             # <<<<<<<<<<<<<<
 *             return decode_string
 *         if self.type == 'bytes':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_string, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 187, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":188
 *             return decode_sfixed64
 *         if self.type == 'string':
 *             return decode_string             # <<<<<<<<<<<<<<
 *         if self.type == 'bytes':
 *             return decode_bytes
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_decode_string;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":187
 *         if self.type == 'sfixed64':
 *             return decode_sfixed64
 *         if self.type == 'string':             # <<<<<<<<<<<<<<
 *             return decode_string
 *         if self.type == 'bytes':
 */
  }

  /* "cprotobuf/internal.pyx":189
 *         if self.type == 'string':
 *             return decode_string
 *         if self.type == 'bytes':             # <<<<<<<<<<<<<<
 *             return decode_bytes
 *         if self.type == 'float':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_bytes, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 189, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":190
 *             return decode_string
 *         if self.type == 'bytes':
 *             return decode_bytes             # <<<<<<<<<<<<<<
 *         if self.type == 'float':
 *             return decode_float
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_decode_bytes;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":189
 *         if self.type == 'string':
 *             return decode_string
 *         if self.type == 'bytes':             # <<<<<<<<<<<<<<
 *             return decode_bytes
 *         if self.type == 'float':
 */
  }

  /* "cprotobuf/internal.pyx":191
 *         if self.type == 'bytes':
 *             return decode_bytes
 *         if self.type == 'float':             # <<<<<<<<<<<<<<
 *             return decode_float
 *         if self.type == 'double':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_float, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 191, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":192
 *             return decode_bytes
 *         if self.type == 'float':
 *             return decode_float             # <<<<<<<<<<<<<<
 *         if self.type == 'double':
 *             return decode_double
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_decode_float;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":191
 *         if self.type == 'bytes':
 *             return decode_bytes
 *         if self.type == 'float':             # <<<<<<<<<<<<<<
 *             return decode_float
 *         if self.type == 'double':
 */
  }

  /* "cprotobuf/internal.pyx":193
 *         if self.type == 'float':
 *             return decode_float
 *         if self.type == 'double':             # <<<<<<<<<<<<<<
 *             return decode_double
 *         return NULL
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->type, __pyx_n_s_double, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 193, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":194
 *             return decode_float
 *         if self.type == 'double':
 *             return decode_double             # <<<<<<<<<<<<<<
 *         return NULL
 * 
 */
    __pyx_r = __pyx_f_9cprotobuf_8internal_decode_double;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":193
 *         if self.type == 'float':
 *             return decode_float
 *         if self.type == 'double':             # <<<<<<<<<<<<<<
 *             return decode_double
 *         return NULL
 */
  }

  /* "cprotobuf/internal.pyx":195
 *         if self.type == 'double':
 *             return decode_double
 *         return NULL             # <<<<<<<<<<<<<<
 * 
 * cdef dict _proto_classes = {}
 */
  __pyx_r = NULL;
  goto __pyx_L0;

  /* "cprotobuf/internal.pyx":162
 *         return encode_subobject
 * 
 *     cdef Decoder get_decoder(self):             # <<<<<<<<<<<<<<
 *         if self.type == 'int32':
 *             return decode_int64     # compatible with official protobuf
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("cprotobuf.internal.Field.get_decoder", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":56
 * cdef class Field(object):
 * 
 *     cdef public str name             # <<<<<<<<<<<<<<
 *     cdef object type
 *     cdef public int index
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_4name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_4name_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field_4name___get__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_4name___get__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->name);
  __pyx_r = __pyx_v_self->name;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9cprotobuf_8internal_5Field_4name_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9cprotobuf_8internal_5Field_4name_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field_4name_2__set__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9cprotobuf_8internal_5Field_4name_2__set__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyString_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(0, 56, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cprotobuf.internal.Field.name.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9cprotobuf_8internal_5Field_4name_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_9cprotobuf_8internal_5Field_4name_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field_4name_4__del__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9cprotobuf_8internal_5Field_4name_4__del__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":58
 *     cdef public str name
 *     cdef object type
 *     cdef public int index             # <<<<<<<<<<<<<<
 *     cdef public bint packed
 *     cdef public bint required
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_5index_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_5index_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field_5index___get__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_5index___get__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cprotobuf.internal.Field.index.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9cprotobuf_8internal_5Field_5index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9cprotobuf_8internal_5Field_5index_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field_5index_2__set__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9cprotobuf_8internal_5Field_5index_2__set__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 58, __pyx_L1_error)
  __pyx_v_self->index = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.Field.index.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":59
 *     cdef object type
 *     cdef public int index
 *     cdef public bint packed             # <<<<<<<<<<<<<<
 *     cdef public bint required
 *     cdef public bint repeated
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_6packed_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_6packed_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field_6packed___get__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_6packed___get__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->packed); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cprotobuf.internal.Field.packed.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9cprotobuf_8internal_5Field_6packed_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9cprotobuf_8internal_5Field_6packed_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field_6packed_2__set__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9cprotobuf_8internal_5Field_6packed_2__set__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 59, __pyx_L1_error)
  __pyx_v_self->packed = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.Field.packed.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":60
 *     cdef public int index
 *     cdef public bint packed
 *     cdef public bint required             # <<<<<<<<<<<<<<
 *     cdef public bint repeated
 *     cdef object klass
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_8required_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_8required_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field_8required___get__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_8required___get__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->required); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cprotobuf.internal.Field.required.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9cprotobuf_8internal_5Field_8required_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9cprotobuf_8internal_5Field_8required_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field_8required_2__set__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9cprotobuf_8internal_5Field_8required_2__set__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 60, __pyx_L1_error)
  __pyx_v_self->required = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.Field.required.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":61
 *     cdef public bint packed
 *     cdef public bint required
 *     cdef public bint repeated             # <<<<<<<<<<<<<<
 *     cdef object klass
 *     cdef object default
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_8repeated_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_8repeated_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field_8repeated___get__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_8repeated___get__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->repeated); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cprotobuf.internal.Field.repeated.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9cprotobuf_8internal_5Field_8repeated_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9cprotobuf_8internal_5Field_8repeated_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field_8repeated_2__set__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9cprotobuf_8internal_5Field_8repeated_2__set__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 61, __pyx_L1_error)
  __pyx_v_self->repeated = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.Field.repeated.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":65
 *     cdef object default
 * 
 *     cdef public unsigned char wire_type             # <<<<<<<<<<<<<<
 *     cdef Encoder encoder
 *     cdef Decoder decoder
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_9wire_type_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_9wire_type_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field_9wire_type___get__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_9wire_type___get__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_char(__pyx_v_self->wire_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cprotobuf.internal.Field.wire_type.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_9cprotobuf_8internal_5Field_9wire_type_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_9cprotobuf_8internal_5Field_9wire_type_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field_9wire_type_2__set__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9cprotobuf_8internal_5Field_9wire_type_2__set__(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned char __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_unsigned_char(__pyx_v_value); if (unlikely((__pyx_t_1 == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(0, 65, __pyx_L1_error)
  __pyx_v_self->wire_type = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.Field.wire_type.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.decoder,self.encoder cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field_4__reduce_cython__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.decoder,self.encoder cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.decoder,self.encoder cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.decoder,self.encoder cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cprotobuf.internal.Field.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.decoder,self.encoder cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.decoder,self.encoder cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_9cprotobuf_8internal_5Field_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_5Field_6__setstate_cython__(((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_5Field_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.decoder,self.encoder cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.decoder,self.encoder cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.decoder,self.encoder cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.decoder,self.encoder cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cprotobuf.internal.Field.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":198
 * 
 * cdef dict _proto_classes = {}
 * cpdef get_proto(name):             # <<<<<<<<<<<<<<
 *     return _proto_classes[name]
 * 
 */

static PyObject *__pyx_pw_9cprotobuf_8internal_1get_proto(PyObject *__pyx_self, PyObject *__pyx_v_name); /*proto*/
static PyObject *__pyx_f_9cprotobuf_8internal_get_proto(PyObject *__pyx_v_name, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("get_proto", 0);

  /* "cprotobuf/internal.pyx":199
 * cdef dict _proto_classes = {}
 * cpdef get_proto(name):
 *     return _proto_classes[name]             # <<<<<<<<<<<<<<
 * 
 * def register_proto(name, cls):
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_9cprotobuf_8internal__proto_classes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 199, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_9cprotobuf_8internal__proto_classes, __pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cprotobuf/internal.pyx":198
 * 
 * cdef dict _proto_classes = {}
 * cpdef get_proto(name):             # <<<<<<<<<<<<<<
 *     return _proto_classes[name]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cprotobuf.internal.get_proto", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_1get_proto(PyObject *__pyx_self, PyObject *__pyx_v_name); /*proto*/
static PyObject *__pyx_pw_9cprotobuf_8internal_1get_proto(PyObject *__pyx_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_proto (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_get_proto(__pyx_self, ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_get_proto(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("get_proto", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_9cprotobuf_8internal_get_proto(__pyx_v_name, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cprotobuf.internal.get_proto", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":201
 *     return _proto_classes[name]
 * 
 * def register_proto(name, cls):             # <<<<<<<<<<<<<<
 *     _proto_classes[name] = cls
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_3register_proto(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9cprotobuf_8internal_3register_proto = {"register_proto", (PyCFunction)__pyx_pw_9cprotobuf_8internal_3register_proto, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9cprotobuf_8internal_3register_proto(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_cls = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("register_proto (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_cls,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_cls)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("register_proto", 1, 2, 2, 1); __PYX_ERR(0, 201, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "register_proto") < 0)) __PYX_ERR(0, 201, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_name = values[0];
    __pyx_v_cls = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("register_proto", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 201, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cprotobuf.internal.register_proto", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9cprotobuf_8internal_2register_proto(__pyx_self, __pyx_v_name, __pyx_v_cls);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_2register_proto(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name, PyObject *__pyx_v_cls) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("register_proto", 0);

  /* "cprotobuf/internal.pyx":202
 * 
 * def register_proto(name, cls):
 *     _proto_classes[name] = cls             # <<<<<<<<<<<<<<
 * 
 * class MetaProtoEntity(type):
 */
  if (unlikely(__pyx_v_9cprotobuf_8internal__proto_classes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 202, __pyx_L1_error)
  }
  if (unlikely(PyDict_SetItem(__pyx_v_9cprotobuf_8internal__proto_classes, __pyx_v_name, __pyx_v_cls) < 0)) __PYX_ERR(0, 202, __pyx_L1_error)

  /* "cprotobuf/internal.pyx":201
 *     return _proto_classes[name]
 * 
 * def register_proto(name, cls):             # <<<<<<<<<<<<<<
 *     _proto_classes[name] = cls
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cprotobuf.internal.register_proto", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":205
 * 
 * class MetaProtoEntity(type):
 *     def __new__(cls, clsname, bases, attrs):             # <<<<<<<<<<<<<<
 *         if clsname == 'ProtoEntity':
 *             return super(MetaProtoEntity, cls).__new__(cls, clsname, bases, attrs)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_15MetaProtoEntity_1__new__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9cprotobuf_8internal_15MetaProtoEntity_1__new__ = {"__new__", (PyCFunction)__pyx_pw_9cprotobuf_8internal_15MetaProtoEntity_1__new__, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9cprotobuf_8internal_15MetaProtoEntity_1__new__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_cls = 0;
  PyObject *__pyx_v_clsname = 0;
  PyObject *__pyx_v_bases = 0;
  PyObject *__pyx_v_attrs = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__new__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_cls,&__pyx_n_s_clsname,&__pyx_n_s_bases,&__pyx_n_s_attrs,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_cls)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_clsname)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__new__", 1, 4, 4, 1); __PYX_ERR(0, 205, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_bases)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__new__", 1, 4, 4, 2); __PYX_ERR(0, 205, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_attrs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__new__", 1, 4, 4, 3); __PYX_ERR(0, 205, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__new__") < 0)) __PYX_ERR(0, 205, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_cls = values[0];
    __pyx_v_clsname = values[1];
    __pyx_v_bases = values[2];
    __pyx_v_attrs = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__new__", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 205, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cprotobuf.internal.MetaProtoEntity.__new__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9cprotobuf_8internal_15MetaProtoEntity___new__(__pyx_self, __pyx_v_cls, __pyx_v_clsname, __pyx_v_bases, __pyx_v_attrs);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":226
 *             _fieldsmap_by_name[name] = f
 *         newcls = super(MetaProtoEntity, cls).__new__(cls, clsname, bases, attrs)
 *         _fields.sort(key=lambda f:f.index)             # <<<<<<<<<<<<<<
 *         newcls._fields = _fields
 *         newcls._fieldsmap = _fieldsmap
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_15MetaProtoEntity_7__new___lambda(PyObject *__pyx_self, PyObject *__pyx_v_f); /*proto*/
static PyMethodDef __pyx_mdef_9cprotobuf_8internal_15MetaProtoEntity_7__new___lambda = {"lambda", (PyCFunction)__pyx_pw_9cprotobuf_8internal_15MetaProtoEntity_7__new___lambda, METH_O, 0};
static PyObject *__pyx_pw_9cprotobuf_8internal_15MetaProtoEntity_7__new___lambda(PyObject *__pyx_self, PyObject *__pyx_v_f) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lambda (wrapper)", 0);
  __pyx_r = __pyx_lambda_funcdef_lambda(__pyx_self, ((PyObject *)__pyx_v_f));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_lambda_funcdef_lambda(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_f) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("lambda", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cprotobuf.internal.MetaProtoEntity.__new__.lambda", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":205
 * 
 * class MetaProtoEntity(type):
 *     def __new__(cls, clsname, bases, attrs):             # <<<<<<<<<<<<<<
 *         if clsname == 'ProtoEntity':
 *             return super(MetaProtoEntity, cls).__new__(cls, clsname, bases, attrs)
 */

static PyObject *__pyx_pf_9cprotobuf_8internal_15MetaProtoEntity___new__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cls, PyObject *__pyx_v_clsname, PyObject *__pyx_v_bases, PyObject *__pyx_v_attrs) {
  PyObject *__pyx_v__fields = NULL;
  PyObject *__pyx_v__fieldsmap = NULL;
  PyObject *__pyx_v__fieldsmap_by_name = NULL;
  struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_f = 0;
  PyObject *__pyx_v_name = NULL;
  PyObject *__pyx_v_v = NULL;
  PyObject *__pyx_v_newcls = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  PyObject *(*__pyx_t_10)(PyObject *);
  int __pyx_t_11;
  int __pyx_t_12;
  __Pyx_RefNannySetupContext("__new__", 0);

  /* "cprotobuf/internal.pyx":206
 * class MetaProtoEntity(type):
 *     def __new__(cls, clsname, bases, attrs):
 *         if clsname == 'ProtoEntity':             # <<<<<<<<<<<<<<
 *             return super(MetaProtoEntity, cls).__new__(cls, clsname, bases, attrs)
 *         # _fields for encode
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_clsname, __pyx_n_s_ProtoEntity, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 206, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cprotobuf/internal.pyx":207
 *     def __new__(cls, clsname, bases, attrs):
 *         if clsname == 'ProtoEntity':
 *             return super(MetaProtoEntity, cls).__new__(cls, clsname, bases, attrs)             # <<<<<<<<<<<<<<
 *         # _fields for encode
 *         # _fieldsmap for decode
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_MetaProtoEntity); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_cls);
    __Pyx_GIVEREF(__pyx_v_cls);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_cls);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_new); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    __pyx_t_5 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_5 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[5] = {__pyx_t_3, __pyx_v_cls, __pyx_v_clsname, __pyx_v_bases, __pyx_v_attrs};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 207, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[5] = {__pyx_t_3, __pyx_v_cls, __pyx_v_clsname, __pyx_v_bases, __pyx_v_attrs};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 207, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    {
      __pyx_t_6 = PyTuple_New(4+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 207, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__pyx_t_3) {
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3); __pyx_t_3 = NULL;
      }
      __Pyx_INCREF(__pyx_v_cls);
      __Pyx_GIVEREF(__pyx_v_cls);
      PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_cls);
      __Pyx_INCREF(__pyx_v_clsname);
      __Pyx_GIVEREF(__pyx_v_clsname);
      PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_clsname);
      __Pyx_INCREF(__pyx_v_bases);
      __Pyx_GIVEREF(__pyx_v_bases);
      PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_bases);
      __Pyx_INCREF(__pyx_v_attrs);
      __Pyx_GIVEREF(__pyx_v_attrs);
      PyTuple_SET_ITEM(__pyx_t_6, 3+__pyx_t_5, __pyx_v_attrs);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 207, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "cprotobuf/internal.pyx":206
 * class MetaProtoEntity(type):
 *     def __new__(cls, clsname, bases, attrs):
 *         if clsname == 'ProtoEntity':             # <<<<<<<<<<<<<<
 *             return super(MetaProtoEntity, cls).__new__(cls, clsname, bases, attrs)
 *         # _fields for encode
 */
  }

  /* "cprotobuf/internal.pyx":210
 *         # _fields for encode
 *         # _fieldsmap for decode
 *         _fields = []             # <<<<<<<<<<<<<<
 *         _fieldsmap = {}
 *         _fieldsmap_by_name = {}
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__fields = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cprotobuf/internal.pyx":211
 *         # _fieldsmap for decode
 *         _fields = []
 *         _fieldsmap = {}             # <<<<<<<<<<<<<<
 *         _fieldsmap_by_name = {}
 *         cdef Field f
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__fieldsmap = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cprotobuf/internal.pyx":212
 *         _fields = []
 *         _fieldsmap = {}
 *         _fieldsmap_by_name = {}             # <<<<<<<<<<<<<<
 *         cdef Field f
 *         for name, v in attrs.items():
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__fieldsmap_by_name = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cprotobuf/internal.pyx":214
 *         _fieldsmap_by_name = {}
 *         cdef Field f
 *         for name, v in attrs.items():             # <<<<<<<<<<<<<<
 *             if name.startswith('__'):
 *                 continue
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_attrs, __pyx_n_s_items); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 214, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 214, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
    __pyx_t_4 = __pyx_t_2; __Pyx_INCREF(__pyx_t_4); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 214, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_8 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 214, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_7); __Pyx_INCREF(__pyx_t_2); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 214, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_4, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 214, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_7); __Pyx_INCREF(__pyx_t_2); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 214, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_4, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 214, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_8(__pyx_t_4);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 214, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
      PyObject* sequence = __pyx_t_2;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 214, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_6 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_3);
      #else
      __pyx_t_6 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 214, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 214, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_9 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 214, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = Py_TYPE(__pyx_t_9)->tp_iternext;
      index = 0; __pyx_t_6 = __pyx_t_10(__pyx_t_9); if (unlikely(!__pyx_t_6)) goto __pyx_L6_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      index = 1; __pyx_t_3 = __pyx_t_10(__pyx_t_9); if (unlikely(!__pyx_t_3)) goto __pyx_L6_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_3);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_10(__pyx_t_9), 2) < 0) __PYX_ERR(0, 214, __pyx_L1_error)
      __pyx_t_10 = NULL;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L7_unpacking_done;
      __pyx_L6_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_10 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 214, __pyx_L1_error)
      __pyx_L7_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_6);
    __pyx_t_6 = 0;
    __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "cprotobuf/internal.pyx":215
 *         cdef Field f
 *         for name, v in attrs.items():
 *             if name.startswith('__'):             # <<<<<<<<<<<<<<
 *                 continue
 *             if not isinstance(v, Field):
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_name, __pyx_n_s_startswith); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 215, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 215, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 215, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_1) {

      /* "cprotobuf/internal.pyx":216
 *         for name, v in attrs.items():
 *             if name.startswith('__'):
 *                 continue             # <<<<<<<<<<<<<<
 *             if not isinstance(v, Field):
 *                 continue
 */
      goto __pyx_L4_continue;

      /* "cprotobuf/internal.pyx":215
 *         cdef Field f
 *         for name, v in attrs.items():
 *             if name.startswith('__'):             # <<<<<<<<<<<<<<
 *                 continue
 *             if not isinstance(v, Field):
 */
    }

    /* "cprotobuf/internal.pyx":217
 *             if name.startswith('__'):
 *                 continue
 *             if not isinstance(v, Field):             # <<<<<<<<<<<<<<
 *                 continue
 *             f = v
 */
    __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_v, __pyx_ptype_9cprotobuf_8internal_Field); 
    __pyx_t_11 = ((!(__pyx_t_1 != 0)) != 0);
    if (__pyx_t_11) {

      /* "cprotobuf/internal.pyx":218
 *                 continue
 *             if not isinstance(v, Field):
 *                 continue             # <<<<<<<<<<<<<<
 *             f = v
 *             f.name = name
 */
      goto __pyx_L4_continue;

      /* "cprotobuf/internal.pyx":217
 *             if name.startswith('__'):
 *                 continue
 *             if not isinstance(v, Field):             # <<<<<<<<<<<<<<
 *                 continue
 *             f = v
 */
    }

    /* "cprotobuf/internal.pyx":219
 *             if not isinstance(v, Field):
 *                 continue
 *             f = v             # <<<<<<<<<<<<<<
 *             f.name = name
 *             assert f.index not in _fieldsmap, 'duplicate field index %s' % f.index
 */
    if (!(likely(((__pyx_v_v) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_v, __pyx_ptype_9cprotobuf_8internal_Field))))) __PYX_ERR(0, 219, __pyx_L1_error)
    __pyx_t_3 = __pyx_v_v;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_f, ((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "cprotobuf/internal.pyx":220
 *                 continue
 *             f = v
 *             f.name = name             # <<<<<<<<<<<<<<
 *             assert f.index not in _fieldsmap, 'duplicate field index %s' % f.index
 *             _fieldsmap[f.index] = f
 */
    if (!(likely(PyString_CheckExact(__pyx_v_name))||((__pyx_v_name) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_v_name)->tp_name), 0))) __PYX_ERR(0, 220, __pyx_L1_error)
    __pyx_t_3 = __pyx_v_name;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_f->name);
    __Pyx_DECREF(__pyx_v_f->name);
    __pyx_v_f->name = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "cprotobuf/internal.pyx":221
 *             f = v
 *             f.name = name
 *             assert f.index not in _fieldsmap, 'duplicate field index %s' % f.index             # <<<<<<<<<<<<<<
 *             _fieldsmap[f.index] = f
 *             _fields.append(f)
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_f->index); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 221, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_11 = (__Pyx_PyDict_ContainsTF(__pyx_t_3, __pyx_v__fieldsmap, Py_NE)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(0, 221, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!(__pyx_t_11 != 0))) {
        __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_f->index); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 221, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_duplicate_field_index_s, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 221, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        PyErr_SetObject(PyExc_AssertionError, __pyx_t_2);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __PYX_ERR(0, 221, __pyx_L1_error)
      }
    }
    #endif

    /* "cprotobuf/internal.pyx":222
 *             f.name = name
 *             assert f.index not in _fieldsmap, 'duplicate field index %s' % f.index
 *             _fieldsmap[f.index] = f             # <<<<<<<<<<<<<<
 *             _fields.append(f)
 *             _fieldsmap_by_name[name] = f
 */
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_f->index); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 222, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (unlikely(PyDict_SetItem(__pyx_v__fieldsmap, __pyx_t_2, ((PyObject *)__pyx_v_f)) < 0)) __PYX_ERR(0, 222, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cprotobuf/internal.pyx":223
 *             assert f.index not in _fieldsmap, 'duplicate field index %s' % f.index
 *             _fieldsmap[f.index] = f
 *             _fields.append(f)             # <<<<<<<<<<<<<<
 *             _fieldsmap_by_name[name] = f
 *         newcls = super(MetaProtoEntity, cls).__new__(cls, clsname, bases, attrs)
 */
    __pyx_t_12 = __Pyx_PyList_Append(__pyx_v__fields, ((PyObject *)__pyx_v_f)); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 223, __pyx_L1_error)

    /* "cprotobuf/internal.pyx":224
 *             _fieldsmap[f.index] = f
 *             _fields.append(f)
 *             _fieldsmap_by_name[name] = f             # <<<<<<<<<<<<<<
 *         newcls = super(MetaProtoEntity, cls).__new__(cls, clsname, bases, attrs)
 *         _fields.sort(key=lambda f:f.index)
 */
    if (unlikely(PyDict_SetItem(__pyx_v__fieldsmap_by_name, __pyx_v_name, ((PyObject *)__pyx_v_f)) < 0)) __PYX_ERR(0, 224, __pyx_L1_error)

    /* "cprotobuf/internal.pyx":214
 *         _fieldsmap_by_name = {}
 *         cdef Field f
 *         for name, v in attrs.items():             # <<<<<<<<<<<<<<
 *             if name.startswith('__'):
 *                 continue
 */
    __pyx_L4_continue:;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cprotobuf/internal.pyx":225
 *             _fields.append(f)
 *             _fieldsmap_by_name[name] = f
 *         newcls = super(MetaProtoEntity, cls).__new__(cls, clsname, bases, attrs)             # <<<<<<<<<<<<<<
 *         _fields.sort(key=lambda f:f.index)
 *         newcls._fields = _fields
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_MetaProtoEntity); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_cls);
  __Pyx_GIVEREF(__pyx_v_cls);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_cls);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_new); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[5] = {__pyx_t_2, __pyx_v_cls, __pyx_v_clsname, __pyx_v_bases, __pyx_v_attrs};
    __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 225, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_4);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[5] = {__pyx_t_2, __pyx_v_cls, __pyx_v_clsname, __pyx_v_bases, __pyx_v_attrs};
    __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 225, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_4);
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(4+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_2) {
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2); __pyx_t_2 = NULL;
    }
    __Pyx_INCREF(__pyx_v_cls);
    __Pyx_GIVEREF(__pyx_v_cls);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_cls);
    __Pyx_INCREF(__pyx_v_clsname);
    __Pyx_GIVEREF(__pyx_v_clsname);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_clsname);
    __Pyx_INCREF(__pyx_v_bases);
    __Pyx_GIVEREF(__pyx_v_bases);
    PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_bases);
    __Pyx_INCREF(__pyx_v_attrs);
    __Pyx_GIVEREF(__pyx_v_attrs);
    PyTuple_SET_ITEM(__pyx_t_6, 3+__pyx_t_5, __pyx_v_attrs);
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 225, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_newcls = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "cprotobuf/internal.pyx":226
 *             _fieldsmap_by_name[name] = f
 *         newcls = super(MetaProtoEntity, cls).__new__(cls, clsname, bases, attrs)
 *         _fields.sort(key=lambda f:f.index)             # <<<<<<<<<<<<<<
 *         newcls._fields = _fields
 *         newcls._fieldsmap = _fieldsmap
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v__fields, __pyx_n_s_sort); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_CyFunction_NewEx(&__pyx_mdef_9cprotobuf_8internal_15MetaProtoEntity_7__new___lambda, 0, __pyx_n_s_MetaProtoEntity___new___locals_l, NULL, __pyx_n_s_cprotobuf_internal, __pyx_d, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_key, __pyx_t_6) < 0) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "cprotobuf/internal.pyx":227
 *         newcls = super(MetaProtoEntity, cls).__new__(cls, clsname, bases, attrs)
 *         _fields.sort(key=lambda f:f.index)
 *         newcls._fields = _fields             # <<<<<<<<<<<<<<
 *         newcls._fieldsmap = _fieldsmap
 *         newcls._fieldsmap_by_name = _fieldsmap_by_name
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_newcls, __pyx_n_s_fields, __pyx_v__fields) < 0) __PYX_ERR(0, 227, __pyx_L1_error)

  /* "cprotobuf/internal.pyx":228
 *         _fields.sort(key=lambda f:f.index)
 *         newcls._fields = _fields
 *         newcls._fieldsmap = _fieldsmap             # <<<<<<<<<<<<<<
 *         newcls._fieldsmap_by_name = _fieldsmap_by_name
 *         register_proto(clsname, newcls)
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_newcls, __pyx_n_s_fieldsmap, __pyx_v__fieldsmap) < 0) __PYX_ERR(0, 228, __pyx_L1_error)

  /* "cprotobuf/internal.pyx":229
 *         newcls._fields = _fields
 *         newcls._fieldsmap = _fieldsmap
 *         newcls._fieldsmap_by_name = _fieldsmap_by_name             # <<<<<<<<<<<<<<
 *         register_proto(clsname, newcls)
 *         return newcls
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_newcls, __pyx_n_s_fieldsmap_by_name, __pyx_v__fieldsmap_by_name) < 0) __PYX_ERR(0, 229, __pyx_L1_error)

  /* "cprotobuf/internal.pyx":230
 *         newcls._fieldsmap = _fieldsmap
 *         newcls._fieldsmap_by_name = _fieldsmap_by_name
 *         register_proto(clsname, newcls)             # <<<<<<<<<<<<<<
 *         return newcls
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_register_proto); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_clsname, __pyx_v_newcls};
    __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 230, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_clsname, __pyx_v_newcls};
    __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 230, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else
  #endif
  {
    __pyx_t_2 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 230, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_clsname);
    __Pyx_GIVEREF(__pyx_v_clsname);
    PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_5, __pyx_v_clsname);
    __Pyx_INCREF(__pyx_v_newcls);
    __Pyx_GIVEREF(__pyx_v_newcls);
    PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_5, __pyx_v_newcls);
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_2, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 230, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "cprotobuf/internal.pyx":231
 *         newcls._fieldsmap_by_name = _fieldsmap_by_name
 *         register_proto(clsname, newcls)
 *         return newcls             # <<<<<<<<<<<<<<
 * 
 * class ProtoEntity(object):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_newcls);
  __pyx_r = __pyx_v_newcls;
  goto __pyx_L0;

  /* "cprotobuf/internal.pyx":205
 * 
 * class MetaProtoEntity(type):
 *     def __new__(cls, clsname, bases, attrs):             # <<<<<<<<<<<<<<
 *         if clsname == 'ProtoEntity':
 *             return super(MetaProtoEntity, cls).__new__(cls, clsname, bases, attrs)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("cprotobuf.internal.MetaProtoEntity.__new__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v__fields);
  __Pyx_XDECREF(__pyx_v__fieldsmap);
  __Pyx_XDECREF(__pyx_v__fieldsmap_by_name);
  __Pyx_XDECREF((PyObject *)__pyx_v_f);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_XDECREF(__pyx_v_newcls);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":236
 *     __metaclass__ = MetaProtoEntity
 * 
 *     def __init__(self, **kwargs):             # <<<<<<<<<<<<<<
 *         self.__dict__.update(kwargs)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_11ProtoEntity_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9cprotobuf_8internal_11ProtoEntity_1__init__ = {"__init__", (PyCFunction)__pyx_pw_9cprotobuf_8internal_11ProtoEntity_1__init__, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9cprotobuf_8internal_11ProtoEntity_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_kwargs = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return NULL;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 236, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_self = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 236, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("cprotobuf.internal.ProtoEntity.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9cprotobuf_8internal_11ProtoEntity___init__(__pyx_self, __pyx_v_self, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_11ProtoEntity___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cprotobuf/internal.pyx":237
 * 
 *     def __init__(self, **kwargs):
 *         self.__dict__.update(kwargs)             # <<<<<<<<<<<<<<
 * 
 *     def SerializeToString(self):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_dict); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_update); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_kwargs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_v_kwargs};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 237, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_v_kwargs};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 237, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 237, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2); __pyx_t_2 = NULL;
      __Pyx_INCREF(__pyx_v_kwargs);
      __Pyx_GIVEREF(__pyx_v_kwargs);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_kwargs);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 237, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":236
 *     __metaclass__ = MetaProtoEntity
 * 
 *     def __init__(self, **kwargs):             # <<<<<<<<<<<<<<
 *         self.__dict__.update(kwargs)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cprotobuf.internal.ProtoEntity.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":239
 *         self.__dict__.update(kwargs)
 * 
 *     def SerializeToString(self):             # <<<<<<<<<<<<<<
 *         return encode_data(type(self), self.__dict__)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_11ProtoEntity_3SerializeToString(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_9cprotobuf_8internal_11ProtoEntity_3SerializeToString = {"SerializeToString", (PyCFunction)__pyx_pw_9cprotobuf_8internal_11ProtoEntity_3SerializeToString, METH_O, 0};
static PyObject *__pyx_pw_9cprotobuf_8internal_11ProtoEntity_3SerializeToString(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("SerializeToString (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_11ProtoEntity_2SerializeToString(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_11ProtoEntity_2SerializeToString(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("SerializeToString", 0);

  /* "cprotobuf/internal.pyx":240
 * 
 *     def SerializeToString(self):
 *         return encode_data(type(self), self.__dict__)             # <<<<<<<<<<<<<<
 * 
 *     def ParseFromString(self, s, int offset=0, int count=-1):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_encode_data); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self);
  __Pyx_GIVEREF(__pyx_v_self);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self);
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7cpython_4type_type), __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_dict); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_3);
    __pyx_t_4 = 0;
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cprotobuf/internal.pyx":239
 *         self.__dict__.update(kwargs)
 * 
 *     def SerializeToString(self):             # <<<<<<<<<<<<<<
 *         return encode_data(type(self), self.__dict__)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("cprotobuf.internal.ProtoEntity.SerializeToString", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":242
 *         return encode_data(type(self), self.__dict__)
 * 
 *     def ParseFromString(self, s, int offset=0, int count=-1):             # <<<<<<<<<<<<<<
 *         cdef char *buff = <char*>s
 *         cdef char *start
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_11ProtoEntity_5ParseFromString(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9cprotobuf_8internal_11ProtoEntity_5ParseFromString = {"ParseFromString", (PyCFunction)__pyx_pw_9cprotobuf_8internal_11ProtoEntity_5ParseFromString, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9cprotobuf_8internal_11ProtoEntity_5ParseFromString(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_s = 0;
  int __pyx_v_offset;
  int __pyx_v_count;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ParseFromString (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_s,&__pyx_n_s_offset,&__pyx_n_s_count,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_s)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("ParseFromString", 0, 2, 4, 1); __PYX_ERR(0, 242, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_offset);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "ParseFromString") < 0)) __PYX_ERR(0, 242, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_s = values[1];
    if (values[2]) {
      __pyx_v_offset = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_offset == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 242, __pyx_L3_error)
    } else {
      __pyx_v_offset = ((int)((int)0));
    }
    if (values[3]) {
      __pyx_v_count = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_count == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 242, __pyx_L3_error)
    } else {
      __pyx_v_count = ((int)((int)-1));
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("ParseFromString", 0, 2, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 242, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cprotobuf.internal.ProtoEntity.ParseFromString", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9cprotobuf_8internal_11ProtoEntity_4ParseFromString(__pyx_self, __pyx_v_self, __pyx_v_s, __pyx_v_offset, __pyx_v_count);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_11ProtoEntity_4ParseFromString(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_s, int __pyx_v_offset, int __pyx_v_count) {
  char *__pyx_v_buff;
  char *__pyx_v_start;
  char *__pyx_v_end;
  Py_ssize_t __pyx_v_size;
  PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  __Pyx_RefNannySetupContext("ParseFromString", 0);

  /* "cprotobuf/internal.pyx":243
 * 
 *     def ParseFromString(self, s, int offset=0, int count=-1):
 *         cdef char *buff = <char*>s             # <<<<<<<<<<<<<<
 *         cdef char *start
 *         cdef char *end
 */
  __pyx_t_1 = __Pyx_PyObject_AsWritableString(__pyx_v_s); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) __PYX_ERR(0, 243, __pyx_L1_error)
  __pyx_v_buff = ((char *)__pyx_t_1);

  /* "cprotobuf/internal.pyx":246
 *         cdef char *start
 *         cdef char *end
 *         cdef Py_ssize_t size = len(s)             # <<<<<<<<<<<<<<
 * 
 *         assert offset < size, "Offset out of bound."
 */
  __pyx_t_2 = PyObject_Length(__pyx_v_s); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 246, __pyx_L1_error)
  __pyx_v_size = __pyx_t_2;

  /* "cprotobuf/internal.pyx":248
 *         cdef Py_ssize_t size = len(s)
 * 
 *         assert offset < size, "Offset out of bound."             # <<<<<<<<<<<<<<
 *         assert count < size-offset, "Count out of bound."
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_offset < __pyx_v_size) != 0))) {
      PyErr_SetObject(PyExc_AssertionError, __pyx_kp_s_Offset_out_of_bound);
      __PYX_ERR(0, 248, __pyx_L1_error)
    }
  }
  #endif

  /* "cprotobuf/internal.pyx":249
 * 
 *         assert offset < size, "Offset out of bound."
 *         assert count < size-offset, "Count out of bound."             # <<<<<<<<<<<<<<
 * 
 *         start = buff + offset
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_count < (__pyx_v_size - __pyx_v_offset)) != 0))) {
      PyErr_SetObject(PyExc_AssertionError, __pyx_kp_s_Count_out_of_bound);
      __PYX_ERR(0, 249, __pyx_L1_error)
    }
  }
  #endif

  /* "cprotobuf/internal.pyx":251
 *         assert count < size-offset, "Count out of bound."
 * 
 *         start = buff + offset             # <<<<<<<<<<<<<<
 *         if count < 0:
 *             end = buff + size
 */
  __pyx_v_start = (__pyx_v_buff + __pyx_v_offset);

  /* "cprotobuf/internal.pyx":252
 * 
 *         start = buff + offset
 *         if count < 0:             # <<<<<<<<<<<<<<
 *             end = buff + size
 *         else:
 */
  __pyx_t_3 = ((__pyx_v_count < 0) != 0);
  if (__pyx_t_3) {

    /* "cprotobuf/internal.pyx":253
 *         start = buff + offset
 *         if count < 0:
 *             end = buff + size             # <<<<<<<<<<<<<<
 *         else:
 *             end = start + count
 */
    __pyx_v_end = (__pyx_v_buff + __pyx_v_size);

    /* "cprotobuf/internal.pyx":252
 * 
 *         start = buff + offset
 *         if count < 0:             # <<<<<<<<<<<<<<
 *             end = buff + size
 *         else:
 */
    goto __pyx_L3;
  }

  /* "cprotobuf/internal.pyx":255
 *             end = buff + size
 *         else:
 *             end = start + count             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  /*else*/ {
    __pyx_v_end = (__pyx_v_start + __pyx_v_count);
  }
  __pyx_L3:;

  /* "cprotobuf/internal.pyx":257
 *             end = start + count
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             decode_object(self, &buff, end)
 *         except InternalDecodeError as e:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {

      /* "cprotobuf/internal.pyx":258
 * 
 *         try:
 *             decode_object(self, &buff, end)             # <<<<<<<<<<<<<<
 *         except InternalDecodeError as e:
 *             #traceback.print_exc()
 */
      __pyx_t_7 = __pyx_f_9cprotobuf_8internal_decode_object(__pyx_v_self, (&__pyx_v_buff), __pyx_v_end); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 258, __pyx_L4_error)

      /* "cprotobuf/internal.pyx":257
 *             end = start + count
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             decode_object(self, &buff, end)
 *         except InternalDecodeError as e:
 */
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L9_try_end;
    __pyx_L4_error:;

    /* "cprotobuf/internal.pyx":259
 *         try:
 *             decode_object(self, &buff, end)
 *         except InternalDecodeError as e:             # <<<<<<<<<<<<<<
 *             #traceback.print_exc()
 *             raise DecodeError(e.args[0] - <uint64_t>start, e.args[1])
 */
    __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_InternalDecodeError); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 259, __pyx_L6_except_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_7 = __Pyx_PyErr_ExceptionMatches(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_7) {
      __Pyx_AddTraceback("cprotobuf.internal.ProtoEntity.ParseFromString", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10) < 0) __PYX_ERR(0, 259, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_9);
      __pyx_v_e = __pyx_t_9;

      /* "cprotobuf/internal.pyx":261
 *         except InternalDecodeError as e:
 *             #traceback.print_exc()
 *             raise DecodeError(e.args[0] - <uint64_t>start, e.args[1])             # <<<<<<<<<<<<<<
 * 
 *     def __unicode__(self):
 */
      __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_DecodeError); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 261, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_e, __pyx_n_s_args); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 261, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_14 = __Pyx_GetItemInt(__pyx_t_13, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 261, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_13 = __Pyx_PyInt_From_uint64_t(((uint64_t)__pyx_v_start)); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 261, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_15 = PyNumber_Subtract(__pyx_t_14, __pyx_t_13); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 261, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_15);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_e, __pyx_n_s_args); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 261, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_14 = __Pyx_GetItemInt(__pyx_t_13, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 261, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_13 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_12);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_12, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_12)) {
        PyObject *__pyx_temp[3] = {__pyx_t_13, __pyx_t_15, __pyx_t_14};
        __pyx_t_11 = __Pyx_PyFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 261, __pyx_L6_except_error)
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_12)) {
        PyObject *__pyx_temp[3] = {__pyx_t_13, __pyx_t_15, __pyx_t_14};
        __pyx_t_11 = __Pyx_PyCFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 261, __pyx_L6_except_error)
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      } else
      #endif
      {
        __pyx_t_16 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 261, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_16);
        if (__pyx_t_13) {
          __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_t_13); __pyx_t_13 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_15);
        PyTuple_SET_ITEM(__pyx_t_16, 0+__pyx_t_7, __pyx_t_15);
        __Pyx_GIVEREF(__pyx_t_14);
        PyTuple_SET_ITEM(__pyx_t_16, 1+__pyx_t_7, __pyx_t_14);
        __pyx_t_15 = 0;
        __pyx_t_14 = 0;
        __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_16, NULL); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 261, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
      }
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_Raise(__pyx_t_11, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __PYX_ERR(0, 261, __pyx_L6_except_error)
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;

    /* "cprotobuf/internal.pyx":257
 *             end = start + count
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             decode_object(self, &buff, end)
 *         except InternalDecodeError as e:
 */
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L9_try_end:;
  }

  /* "cprotobuf/internal.pyx":242
 *         return encode_data(type(self), self.__dict__)
 * 
 *     def ParseFromString(self, s, int offset=0, int count=-1):             # <<<<<<<<<<<<<<
 *         cdef char *buff = <char*>s
 *         cdef char *start
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("cprotobuf.internal.ProtoEntity.ParseFromString", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":263
 *             raise DecodeError(e.args[0] - <uint64_t>start, e.args[1])
 * 
 *     def __unicode__(self):             # <<<<<<<<<<<<<<
 *         return str(self).decode('utf-8')
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_11ProtoEntity_7__unicode__(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_9cprotobuf_8internal_11ProtoEntity_7__unicode__ = {"__unicode__", (PyCFunction)__pyx_pw_9cprotobuf_8internal_11ProtoEntity_7__unicode__, METH_O, 0};
static PyObject *__pyx_pw_9cprotobuf_8internal_11ProtoEntity_7__unicode__(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__unicode__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_11ProtoEntity_6__unicode__(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_11ProtoEntity_6__unicode__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__unicode__", 0);

  /* "cprotobuf/internal.pyx":264
 * 
 *     def __unicode__(self):
 *         return str(self).decode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self);
  __Pyx_GIVEREF(__pyx_v_self);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self);
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)(&PyString_Type)), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_decode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cprotobuf/internal.pyx":263
 *             raise DecodeError(e.args[0] - <uint64_t>start, e.args[1])
 * 
 *     def __unicode__(self):             # <<<<<<<<<<<<<<
 *         return str(self).decode('utf-8')
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cprotobuf.internal.ProtoEntity.__unicode__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":266
 *         return str(self).decode('utf-8')
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return str(self.todict())
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_11ProtoEntity_9__str__(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_9cprotobuf_8internal_11ProtoEntity_9__str__ = {"__str__", (PyCFunction)__pyx_pw_9cprotobuf_8internal_11ProtoEntity_9__str__, METH_O, 0};
static PyObject *__pyx_pw_9cprotobuf_8internal_11ProtoEntity_9__str__(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_11ProtoEntity_8__str__(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_11ProtoEntity_8__str__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "cprotobuf/internal.pyx":267
 * 
 *     def __str__(self):
 *         return str(self.todict())             # <<<<<<<<<<<<<<
 * 
 *     def todict(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_todict); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 267, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 267, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)(&PyString_Type)), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 267, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cprotobuf/internal.pyx":266
 *         return str(self).decode('utf-8')
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return str(self.todict())
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cprotobuf.internal.ProtoEntity.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":269
 *         return str(self.todict())
 * 
 *     def todict(self):             # <<<<<<<<<<<<<<
 *         cdef Field f
 *         data = {}
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_11ProtoEntity_11todict(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_9cprotobuf_8internal_11ProtoEntity_11todict = {"todict", (PyCFunction)__pyx_pw_9cprotobuf_8internal_11ProtoEntity_11todict, METH_O, 0};
static PyObject *__pyx_pw_9cprotobuf_8internal_11ProtoEntity_11todict(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("todict (wrapper)", 0);
  __pyx_r = __pyx_pf_9cprotobuf_8internal_11ProtoEntity_10todict(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_11ProtoEntity_10todict(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_f = 0;
  PyObject *__pyx_v_data = NULL;
  PyObject *__pyx_v_d = NULL;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_v_v = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  int __pyx_t_10;
  PyObject *(*__pyx_t_11)(PyObject *);
  PyObject *__pyx_t_12 = NULL;
  __Pyx_RefNannySetupContext("todict", 0);

  /* "cprotobuf/internal.pyx":271
 *     def todict(self):
 *         cdef Field f
 *         data = {}             # <<<<<<<<<<<<<<
 *         d = self.__dict__
 *         for f in self._fields:
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_data = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":272
 *         cdef Field f
 *         data = {}
 *         d = self.__dict__             # <<<<<<<<<<<<<<
 *         for f in self._fields:
 *             value = d.get(f.name)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_dict); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_d = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":273
 *         data = {}
 *         d = self.__dict__
 *         for f in self._fields:             # <<<<<<<<<<<<<<
 *             value = d.get(f.name)
 *             if value == None:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_fields); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 273, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 273, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 273, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 273, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 273, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 273, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_9cprotobuf_8internal_Field))))) __PYX_ERR(0, 273, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_f, ((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "cprotobuf/internal.pyx":274
 *         d = self.__dict__
 *         for f in self._fields:
 *             value = d.get(f.name)             # <<<<<<<<<<<<<<
 *             if value == None:
 *                 continue
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_d, __pyx_n_s_get); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 274, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (!__pyx_t_6) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_f->name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 274, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_f->name};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 274, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_f->name};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 274, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      {
        __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 274, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
        __Pyx_INCREF(__pyx_v_f->name);
        __Pyx_GIVEREF(__pyx_v_f->name);
        PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_v_f->name);
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 274, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "cprotobuf/internal.pyx":275
 *         for f in self._fields:
 *             value = d.get(f.name)
 *             if value == None:             # <<<<<<<<<<<<<<
 *                 continue
 *             if f.repeated:
 */
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_value, Py_None, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 275, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_8) {

      /* "cprotobuf/internal.pyx":276
 *             value = d.get(f.name)
 *             if value == None:
 *                 continue             # <<<<<<<<<<<<<<
 *             if f.repeated:
 *                 if len(value) < 1:
 */
      goto __pyx_L3_continue;

      /* "cprotobuf/internal.pyx":275
 *         for f in self._fields:
 *             value = d.get(f.name)
 *             if value == None:             # <<<<<<<<<<<<<<
 *                 continue
 *             if f.repeated:
 */
    }

    /* "cprotobuf/internal.pyx":277
 *             if value == None:
 *                 continue
 *             if f.repeated:             # <<<<<<<<<<<<<<
 *                 if len(value) < 1:
 *                     continue
 */
    __pyx_t_8 = (__pyx_v_f->repeated != 0);
    if (__pyx_t_8) {

      /* "cprotobuf/internal.pyx":278
 *                 continue
 *             if f.repeated:
 *                 if len(value) < 1:             # <<<<<<<<<<<<<<
 *                     continue
 *                 if isinstance(value[0], ProtoEntity):
 */
      __pyx_t_9 = PyObject_Length(__pyx_v_value); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 278, __pyx_L1_error)
      __pyx_t_8 = ((__pyx_t_9 < 1) != 0);
      if (__pyx_t_8) {

        /* "cprotobuf/internal.pyx":279
 *             if f.repeated:
 *                 if len(value) < 1:
 *                     continue             # <<<<<<<<<<<<<<
 *                 if isinstance(value[0], ProtoEntity):
 *                     data[f.name] = [v.todict() for v in value]
 */
        goto __pyx_L3_continue;

        /* "cprotobuf/internal.pyx":278
 *                 continue
 *             if f.repeated:
 *                 if len(value) < 1:             # <<<<<<<<<<<<<<
 *                     continue
 *                 if isinstance(value[0], ProtoEntity):
 */
      }

      /* "cprotobuf/internal.pyx":280
 *                 if len(value) < 1:
 *                     continue
 *                 if isinstance(value[0], ProtoEntity):             # <<<<<<<<<<<<<<
 *                     data[f.name] = [v.todict() for v in value]
 *                 else:
 */
      __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_value, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 280, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_ProtoEntity); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 280, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_8 = PyObject_IsInstance(__pyx_t_1, __pyx_t_5); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 280, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_10 = (__pyx_t_8 != 0);
      if (__pyx_t_10) {

        /* "cprotobuf/internal.pyx":281
 *                     continue
 *                 if isinstance(value[0], ProtoEntity):
 *                     data[f.name] = [v.todict() for v in value]             # <<<<<<<<<<<<<<
 *                 else:
 *                     data[f.name] = value
 */
        __pyx_t_5 = PyList_New(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 281, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        if (likely(PyList_CheckExact(__pyx_v_value)) || PyTuple_CheckExact(__pyx_v_value)) {
          __pyx_t_1 = __pyx_v_value; __Pyx_INCREF(__pyx_t_1); __pyx_t_9 = 0;
          __pyx_t_11 = NULL;
        } else {
          __pyx_t_9 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 281, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_11 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 281, __pyx_L1_error)
        }
        for (;;) {
          if (likely(!__pyx_t_11)) {
            if (likely(PyList_CheckExact(__pyx_t_1))) {
              if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_1)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_7 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_7); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 281, __pyx_L1_error)
              #else
              __pyx_t_7 = PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 281, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_7);
              #endif
            } else {
              if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_9); __Pyx_INCREF(__pyx_t_7); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 281, __pyx_L1_error)
              #else
              __pyx_t_7 = PySequence_ITEM(__pyx_t_1, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 281, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_7);
              #endif
            }
          } else {
            __pyx_t_7 = __pyx_t_11(__pyx_t_1);
            if (unlikely(!__pyx_t_7)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 281, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_7);
          }
          __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_7);
          __pyx_t_7 = 0;
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_v, __pyx_n_s_todict); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 281, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_12 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
            __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_6);
            if (likely(__pyx_t_12)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
              __Pyx_INCREF(__pyx_t_12);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_6, function);
            }
          }
          if (__pyx_t_12) {
            __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 281, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          } else {
            __pyx_t_7 = __Pyx_PyObject_CallNoArg(__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 281, __pyx_L1_error)
          }
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(__Pyx_ListComp_Append(__pyx_t_5, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 281, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(PyDict_SetItem(__pyx_v_data, __pyx_v_f->name, __pyx_t_5) < 0)) __PYX_ERR(0, 281, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "cprotobuf/internal.pyx":280
 *                 if len(value) < 1:
 *                     continue
 *                 if isinstance(value[0], ProtoEntity):             # <<<<<<<<<<<<<<
 *                     data[f.name] = [v.todict() for v in value]
 *                 else:
 */
        goto __pyx_L8;
      }

      /* "cprotobuf/internal.pyx":283
 *                     data[f.name] = [v.todict() for v in value]
 *                 else:
 *                     data[f.name] = value             # <<<<<<<<<<<<<<
 *             else:
 *                 if isinstance(value, ProtoEntity):
 */
      /*else*/ {
        if (unlikely(PyDict_SetItem(__pyx_v_data, __pyx_v_f->name, __pyx_v_value) < 0)) __PYX_ERR(0, 283, __pyx_L1_error)
      }
      __pyx_L8:;

      /* "cprotobuf/internal.pyx":277
 *             if value == None:
 *                 continue
 *             if f.repeated:             # <<<<<<<<<<<<<<
 *                 if len(value) < 1:
 *                     continue
 */
      goto __pyx_L6;
    }

    /* "cprotobuf/internal.pyx":285
 *                     data[f.name] = value
 *             else:
 *                 if isinstance(value, ProtoEntity):             # <<<<<<<<<<<<<<
 *                     data[f.name] = value.todict()
 *                 else:
 */
    /*else*/ {
      __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_ProtoEntity); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 285, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_10 = PyObject_IsInstance(__pyx_v_value, __pyx_t_5); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 285, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_8 = (__pyx_t_10 != 0);
      if (__pyx_t_8) {

        /* "cprotobuf/internal.pyx":286
 *             else:
 *                 if isinstance(value, ProtoEntity):
 *                     data[f.name] = value.todict()             # <<<<<<<<<<<<<<
 *                 else:
 *                     data[f.name] = value
 */
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_todict); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 286, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_1);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_1, function);
          }
        }
        if (__pyx_t_7) {
          __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 286, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else {
          __pyx_t_5 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 286, __pyx_L1_error)
        }
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(PyDict_SetItem(__pyx_v_data, __pyx_v_f->name, __pyx_t_5) < 0)) __PYX_ERR(0, 286, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "cprotobuf/internal.pyx":285
 *                     data[f.name] = value
 *             else:
 *                 if isinstance(value, ProtoEntity):             # <<<<<<<<<<<<<<
 *                     data[f.name] = value.todict()
 *                 else:
 */
        goto __pyx_L11;
      }

      /* "cprotobuf/internal.pyx":288
 *                     data[f.name] = value.todict()
 *                 else:
 *                     data[f.name] = value             # <<<<<<<<<<<<<<
 *         return data
 * 
 */
      /*else*/ {
        if (unlikely(PyDict_SetItem(__pyx_v_data, __pyx_v_f->name, __pyx_v_value) < 0)) __PYX_ERR(0, 288, __pyx_L1_error)
      }
      __pyx_L11:;
    }
    __pyx_L6:;

    /* "cprotobuf/internal.pyx":273
 *         data = {}
 *         d = self.__dict__
 *         for f in self._fields:             # <<<<<<<<<<<<<<
 *             value = d.get(f.name)
 *             if value == None:
 */
    __pyx_L3_continue:;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cprotobuf/internal.pyx":289
 *                 else:
 *                     data[f.name] = value
 *         return data             # <<<<<<<<<<<<<<
 * 
 * def encode_data(cls, dict d):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_data);
  __pyx_r = __pyx_v_data;
  goto __pyx_L0;

  /* "cprotobuf/internal.pyx":269
 *         return str(self.todict())
 * 
 *     def todict(self):             # <<<<<<<<<<<<<<
 *         cdef Field f
 *         data = {}
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("cprotobuf.internal.ProtoEntity.todict", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_f);
  __Pyx_XDECREF(__pyx_v_data);
  __Pyx_XDECREF(__pyx_v_d);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":291
 *         return data
 * 
 * def encode_data(cls, dict d):             # <<<<<<<<<<<<<<
 *     cdef bytearray buf = bytearray()
 *     cdef bytearray buf1
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_5encode_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9cprotobuf_8internal_5encode_data = {"encode_data", (PyCFunction)__pyx_pw_9cprotobuf_8internal_5encode_data, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9cprotobuf_8internal_5encode_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_cls = 0;
  PyObject *__pyx_v_d = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("encode_data (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_cls,&__pyx_n_s_d,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_cls)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("encode_data", 1, 2, 2, 1); __PYX_ERR(0, 291, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "encode_data") < 0)) __PYX_ERR(0, 291, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_cls = values[0];
    __pyx_v_d = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("encode_data", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 291, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cprotobuf.internal.encode_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_d), (&PyDict_Type), 1, "d", 1))) __PYX_ERR(0, 291, __pyx_L1_error)
  __pyx_r = __pyx_pf_9cprotobuf_8internal_4encode_data(__pyx_self, __pyx_v_cls, __pyx_v_d);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_4encode_data(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cls, PyObject *__pyx_v_d) {
  PyObject *__pyx_v_buf = 0;
  PyObject *__pyx_v_buf1 = 0;
  struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_f = 0;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_v_item = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("encode_data", 0);

  /* "cprotobuf/internal.pyx":292
 * 
 * def encode_data(cls, dict d):
 *     cdef bytearray buf = bytearray()             # <<<<<<<<<<<<<<
 *     cdef bytearray buf1
 *     cdef Field f
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)(&PyByteArray_Type)), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 292, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_buf = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":295
 *     cdef bytearray buf1
 *     cdef Field f
 *     for f in <list>cls._fields:             # <<<<<<<<<<<<<<
 *         value = d.get(f.name)
 *         if value is None:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_cls, __pyx_n_s_fields); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 295, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 295, __pyx_L1_error)
  }
  __pyx_t_2 = ((PyObject*)__pyx_t_1); __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 295, __pyx_L1_error)
    #else
    __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    #endif
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_9cprotobuf_8internal_Field))))) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_f, ((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "cprotobuf/internal.pyx":296
 *     cdef Field f
 *     for f in <list>cls._fields:
 *         value = d.get(f.name)             # <<<<<<<<<<<<<<
 *         if value is None:
 *             continue
 */
    if (unlikely(__pyx_v_d == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
      __PYX_ERR(0, 296, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItemDefault(__pyx_v_d, __pyx_v_f->name, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "cprotobuf/internal.pyx":297
 *     for f in <list>cls._fields:
 *         value = d.get(f.name)
 *         if value is None:             # <<<<<<<<<<<<<<
 *             continue
 *         if f.packed:
 */
    __pyx_t_4 = (__pyx_v_value == Py_None);
    __pyx_t_5 = (__pyx_t_4 != 0);
    if (__pyx_t_5) {

      /* "cprotobuf/internal.pyx":298
 *         value = d.get(f.name)
 *         if value is None:
 *             continue             # <<<<<<<<<<<<<<
 *         if f.packed:
 *             encode_type(buf, f.wire_type, f.index)
 */
      goto __pyx_L3_continue;

      /* "cprotobuf/internal.pyx":297
 *     for f in <list>cls._fields:
 *         value = d.get(f.name)
 *         if value is None:             # <<<<<<<<<<<<<<
 *             continue
 *         if f.packed:
 */
    }

    /* "cprotobuf/internal.pyx":299
 *         if value is None:
 *             continue
 *         if f.packed:             # <<<<<<<<<<<<<<
 *             encode_type(buf, f.wire_type, f.index)
 *             buf1 = bytearray()
 */
    __pyx_t_5 = (__pyx_v_f->packed != 0);
    if (__pyx_t_5) {

      /* "cprotobuf/internal.pyx":300
 *             continue
 *         if f.packed:
 *             encode_type(buf, f.wire_type, f.index)             # <<<<<<<<<<<<<<
 *             buf1 = bytearray()
 *             for item in value:
 */
      __pyx_t_1 = __pyx_f_9cprotobuf_8internal_encode_type(__pyx_v_buf, __pyx_v_f->wire_type, __pyx_v_f->index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 300, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "cprotobuf/internal.pyx":301
 *         if f.packed:
 *             encode_type(buf, f.wire_type, f.index)
 *             buf1 = bytearray()             # <<<<<<<<<<<<<<
 *             for item in value:
 *                 f.encoder(f, buf1, item)
 */
      __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)(&PyByteArray_Type)), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 301, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_buf1, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "cprotobuf/internal.pyx":302
 *             encode_type(buf, f.wire_type, f.index)
 *             buf1 = bytearray()
 *             for item in value:             # <<<<<<<<<<<<<<
 *                 f.encoder(f, buf1, item)
 *             encode_bytes(f, buf, buf1)
 */
      if (likely(PyList_CheckExact(__pyx_v_value)) || PyTuple_CheckExact(__pyx_v_value)) {
        __pyx_t_1 = __pyx_v_value; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
        __pyx_t_7 = NULL;
      } else {
        __pyx_t_6 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 302, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_7 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 302, __pyx_L1_error)
      }
      for (;;) {
        if (likely(!__pyx_t_7)) {
          if (likely(PyList_CheckExact(__pyx_t_1))) {
            if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_8 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_8); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 302, __pyx_L1_error)
            #else
            __pyx_t_8 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 302, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_8);
            #endif
          } else {
            if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_8 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_8); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 302, __pyx_L1_error)
            #else
            __pyx_t_8 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 302, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_8);
            #endif
          }
        } else {
          __pyx_t_8 = __pyx_t_7(__pyx_t_1);
          if (unlikely(!__pyx_t_8)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 302, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_8);
        }
        __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_8);
        __pyx_t_8 = 0;

        /* "cprotobuf/internal.pyx":303
 *             buf1 = bytearray()
 *             for item in value:
 *                 f.encoder(f, buf1, item)             # <<<<<<<<<<<<<<
 *             encode_bytes(f, buf, buf1)
 *         else:
 */
        __pyx_t_8 = __pyx_v_f->encoder(__pyx_v_f, __pyx_v_buf1, __pyx_v_item); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 303, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

        /* "cprotobuf/internal.pyx":302
 *             encode_type(buf, f.wire_type, f.index)
 *             buf1 = bytearray()
 *             for item in value:             # <<<<<<<<<<<<<<
 *                 f.encoder(f, buf1, item)
 *             encode_bytes(f, buf, buf1)
 */
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "cprotobuf/internal.pyx":304
 *             for item in value:
 *                 f.encoder(f, buf1, item)
 *             encode_bytes(f, buf, buf1)             # <<<<<<<<<<<<<<
 *         else:
 *             if f.repeated:
 */
      __pyx_t_1 = __pyx_f_9cprotobuf_8internal_encode_bytes(__pyx_v_f, __pyx_v_buf, __pyx_v_buf1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 304, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "cprotobuf/internal.pyx":299
 *         if value is None:
 *             continue
 *         if f.packed:             # <<<<<<<<<<<<<<
 *             encode_type(buf, f.wire_type, f.index)
 *             buf1 = bytearray()
 */
      goto __pyx_L6;
    }

    /* "cprotobuf/internal.pyx":306
 *             encode_bytes(f, buf, buf1)
 *         else:
 *             if f.repeated:             # <<<<<<<<<<<<<<
 *                 for item in value:
 *                     encode_type(buf, f.wire_type, f.index)
 */
    /*else*/ {
      __pyx_t_5 = (__pyx_v_f->repeated != 0);
      if (__pyx_t_5) {

        /* "cprotobuf/internal.pyx":307
 *         else:
 *             if f.repeated:
 *                 for item in value:             # <<<<<<<<<<<<<<
 *                     encode_type(buf, f.wire_type, f.index)
 *                     f.encoder(f, buf, item)
 */
        if (likely(PyList_CheckExact(__pyx_v_value)) || PyTuple_CheckExact(__pyx_v_value)) {
          __pyx_t_1 = __pyx_v_value; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
          __pyx_t_7 = NULL;
        } else {
          __pyx_t_6 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 307, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_7 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 307, __pyx_L1_error)
        }
        for (;;) {
          if (likely(!__pyx_t_7)) {
            if (likely(PyList_CheckExact(__pyx_t_1))) {
              if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_8 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_8); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 307, __pyx_L1_error)
              #else
              __pyx_t_8 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 307, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_8);
              #endif
            } else {
              if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_8 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_8); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 307, __pyx_L1_error)
              #else
              __pyx_t_8 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 307, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_8);
              #endif
            }
          } else {
            __pyx_t_8 = __pyx_t_7(__pyx_t_1);
            if (unlikely(!__pyx_t_8)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 307, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_8);
          }
          __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_8);
          __pyx_t_8 = 0;

          /* "cprotobuf/internal.pyx":308
 *             if f.repeated:
 *                 for item in value:
 *                     encode_type(buf, f.wire_type, f.index)             # <<<<<<<<<<<<<<
 *                     f.encoder(f, buf, item)
 *             else:
 */
          __pyx_t_8 = __pyx_f_9cprotobuf_8internal_encode_type(__pyx_v_buf, __pyx_v_f->wire_type, __pyx_v_f->index); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 308, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

          /* "cprotobuf/internal.pyx":309
 *                 for item in value:
 *                     encode_type(buf, f.wire_type, f.index)
 *                     f.encoder(f, buf, item)             # <<<<<<<<<<<<<<
 *             else:
 *                 encode_type(buf, f.wire_type, f.index)
 */
          __pyx_t_8 = __pyx_v_f->encoder(__pyx_v_f, __pyx_v_buf, __pyx_v_item); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 309, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

          /* "cprotobuf/internal.pyx":307
 *         else:
 *             if f.repeated:
 *                 for item in value:             # <<<<<<<<<<<<<<
 *                     encode_type(buf, f.wire_type, f.index)
 *                     f.encoder(f, buf, item)
 */
        }
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "cprotobuf/internal.pyx":306
 *             encode_bytes(f, buf, buf1)
 *         else:
 *             if f.repeated:             # <<<<<<<<<<<<<<
 *                 for item in value:
 *                     encode_type(buf, f.wire_type, f.index)
 */
        goto __pyx_L9;
      }

      /* "cprotobuf/internal.pyx":311
 *                     f.encoder(f, buf, item)
 *             else:
 *                 encode_type(buf, f.wire_type, f.index)             # <<<<<<<<<<<<<<
 *                 f.encoder(f, buf, value)
 *     return buf
 */
      /*else*/ {
        __pyx_t_1 = __pyx_f_9cprotobuf_8internal_encode_type(__pyx_v_buf, __pyx_v_f->wire_type, __pyx_v_f->index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 311, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "cprotobuf/internal.pyx":312
 *             else:
 *                 encode_type(buf, f.wire_type, f.index)
 *                 f.encoder(f, buf, value)             # <<<<<<<<<<<<<<
 *     return buf
 * 
 */
        __pyx_t_1 = __pyx_v_f->encoder(__pyx_v_f, __pyx_v_buf, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 312, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }
      __pyx_L9:;
    }
    __pyx_L6:;

    /* "cprotobuf/internal.pyx":295
 *     cdef bytearray buf1
 *     cdef Field f
 *     for f in <list>cls._fields:             # <<<<<<<<<<<<<<
 *         value = d.get(f.name)
 *         if value is None:
 */
    __pyx_L3_continue:;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cprotobuf/internal.pyx":313
 *                 encode_type(buf, f.wire_type, f.index)
 *                 f.encoder(f, buf, value)
 *     return buf             # <<<<<<<<<<<<<<
 * 
 * cdef inline encode_subobject(Field f, bytearray array, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_buf);
  __pyx_r = __pyx_v_buf;
  goto __pyx_L0;

  /* "cprotobuf/internal.pyx":291
 *         return data
 * 
 * def encode_data(cls, dict d):             # <<<<<<<<<<<<<<
 *     cdef bytearray buf = bytearray()
 *     cdef bytearray buf1
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("cprotobuf.internal.encode_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_buf);
  __Pyx_XDECREF(__pyx_v_buf1);
  __Pyx_XDECREF((PyObject *)__pyx_v_f);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":315
 *     return buf
 * 
 * cdef inline encode_subobject(Field f, bytearray array, value):             # <<<<<<<<<<<<<<
 *     cdef object sub_buf
 *     f.resolve_klass()
 */

static CYTHON_INLINE PyObject *__pyx_f_9cprotobuf_8internal_encode_subobject(struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_f, PyObject *__pyx_v_array, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_sub_buf = 0;
  PyObject *__pyx_v_cls = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("encode_subobject", 0);

  /* "cprotobuf/internal.pyx":317
 * cdef inline encode_subobject(Field f, bytearray array, value):
 *     cdef object sub_buf
 *     f.resolve_klass()             # <<<<<<<<<<<<<<
 *     cdef object cls = f.klass
 *     if isinstance(value, dict):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_9cprotobuf_8internal_Field *)__pyx_v_f->__pyx_vtab)->resolve_klass(__pyx_v_f); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":318
 *     cdef object sub_buf
 *     f.resolve_klass()
 *     cdef object cls = f.klass             # <<<<<<<<<<<<<<
 *     if isinstance(value, dict):
 *         sub_buf = encode_data(cls, value)
 */
  __pyx_t_1 = __pyx_v_f->klass;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_cls = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":319
 *     f.resolve_klass()
 *     cdef object cls = f.klass
 *     if isinstance(value, dict):             # <<<<<<<<<<<<<<
 *         sub_buf = encode_data(cls, value)
 *     else:
 */
  __pyx_t_2 = PyDict_Check(__pyx_v_value); 
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "cprotobuf/internal.pyx":320
 *     cdef object cls = f.klass
 *     if isinstance(value, dict):
 *         sub_buf = encode_data(cls, value)             # <<<<<<<<<<<<<<
 *     else:
 *         sub_buf = encode_data(cls, value.__dict__)
 */
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_encode_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 320, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_cls, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 320, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_cls, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 320, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 320, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_cls);
      __Pyx_GIVEREF(__pyx_v_cls);
      PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_cls);
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 320, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_sub_buf = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "cprotobuf/internal.pyx":319
 *     f.resolve_klass()
 *     cdef object cls = f.klass
 *     if isinstance(value, dict):             # <<<<<<<<<<<<<<
 *         sub_buf = encode_data(cls, value)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "cprotobuf/internal.pyx":322
 *         sub_buf = encode_data(cls, value)
 *     else:
 *         sub_buf = encode_data(cls, value.__dict__)             # <<<<<<<<<<<<<<
 *     encode_bytes(f, array, sub_buf)
 * 
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_encode_data); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 322, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 322, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_5 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_cls, __pyx_t_7};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 322, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_cls, __pyx_t_7};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 322, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 322, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_cls);
      __Pyx_GIVEREF(__pyx_v_cls);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_6, __pyx_v_cls);
      __Pyx_GIVEREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_6, __pyx_t_7);
      __pyx_t_7 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 322, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_sub_buf = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "cprotobuf/internal.pyx":323
 *     else:
 *         sub_buf = encode_data(cls, value.__dict__)
 *     encode_bytes(f, array, sub_buf)             # <<<<<<<<<<<<<<
 * 
 * cdef inline int decode_object(object self, char **pointer, char *end) except -1:
 */
  __pyx_t_1 = __pyx_f_9cprotobuf_8internal_encode_bytes(__pyx_v_f, __pyx_v_array, __pyx_v_sub_buf); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":315
 *     return buf
 * 
 * cdef inline encode_subobject(Field f, bytearray array, value):             # <<<<<<<<<<<<<<
 *     cdef object sub_buf
 *     f.resolve_klass()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("cprotobuf.internal.encode_subobject", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_sub_buf);
  __Pyx_XDECREF(__pyx_v_cls);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cprotobuf/internal.pyx":325
 *     encode_bytes(f, array, sub_buf)
 * 
 * cdef inline int decode_object(object self, char **pointer, char *end) except -1:             # <<<<<<<<<<<<<<
 *     cdef dict fieldsmap = self._fieldsmap
 *     cdef uint32_t tag, wtype, findex
 */

static CYTHON_INLINE int __pyx_f_9cprotobuf_8internal_decode_object(PyObject *__pyx_v_self, char **__pyx_v_pointer, char *__pyx_v_end) {
  PyObject *__pyx_v_fieldsmap = 0;
  uint32_t __pyx_v_tag;
  uint32_t __pyx_v_wtype;
  uint32_t __pyx_v_findex;
  char *__pyx_v_sub_buff;
  char *__pyx_v_sub_end;
  uint64_t __pyx_v_sub_size;
  struct __pyx_obj_9cprotobuf_8internal_Field *__pyx_v_f = 0;
  PyObject *__pyx_v_d = 0;
  PyObject *__pyx_v_l = 0;
  PyObject *__pyx_v_value = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("decode_object", 0);

  /* "cprotobuf/internal.pyx":326
 * 
 * cdef inline int decode_object(object self, char **pointer, char *end) except -1:
 *     cdef dict fieldsmap = self._fieldsmap             # <<<<<<<<<<<<<<
 *     cdef uint32_t tag, wtype, findex
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_fieldsmap); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 326, __pyx_L1_error)
  __pyx_v_fieldsmap = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":334
 * 
 *     cdef Field f
 *     cdef dict d = self.__dict__             # <<<<<<<<<<<<<<
 *     cdef list l
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_dict); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 334, __pyx_L1_error)
  __pyx_v_d = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":337
 *     cdef list l
 * 
 *     while pointer[0] < end:             # <<<<<<<<<<<<<<
 *         if raw_decode_uint32(pointer, end, &tag):
 *             raise makeDecodeError(pointer[0], "Can't deserialize type tag at [{0}] for value")
 */
  while (1) {
    __pyx_t_2 = (((__pyx_v_pointer[0]) < __pyx_v_end) != 0);
    if (!__pyx_t_2) break;

    /* "cprotobuf/internal.pyx":338
 * 
 *     while pointer[0] < end:
 *         if raw_decode_uint32(pointer, end, &tag):             # <<<<<<<<<<<<<<
 *             raise makeDecodeError(pointer[0], "Can't deserialize type tag at [{0}] for value")
 *         findex = tag >> 3
 */
    __pyx_t_2 = (__pyx_f_9cprotobuf_8internal_raw_decode_uint32(__pyx_v_pointer, __pyx_v_end, (&__pyx_v_tag)) != 0);
    if (__pyx_t_2) {

      /* "cprotobuf/internal.pyx":339
 *     while pointer[0] < end:
 *         if raw_decode_uint32(pointer, end, &tag):
 *             raise makeDecodeError(pointer[0], "Can't deserialize type tag at [{0}] for value")             # <<<<<<<<<<<<<<
 *         findex = tag >> 3
 *         f = fieldsmap.get(findex)
 */
      __pyx_t_1 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_deserialize_type_tag_at_0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 339, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 339, __pyx_L1_error)

      /* "cprotobuf/internal.pyx":338
 * 
 *     while pointer[0] < end:
 *         if raw_decode_uint32(pointer, end, &tag):             # <<<<<<<<<<<<<<
 *             raise makeDecodeError(pointer[0], "Can't deserialize type tag at [{0}] for value")
 *         findex = tag >> 3
 */
    }

    /* "cprotobuf/internal.pyx":340
 *         if raw_decode_uint32(pointer, end, &tag):
 *             raise makeDecodeError(pointer[0], "Can't deserialize type tag at [{0}] for value")
 *         findex = tag >> 3             # <<<<<<<<<<<<<<
 *         f = fieldsmap.get(findex)
 *         if f is None:
 */
    __pyx_v_findex = (__pyx_v_tag >> 3);

    /* "cprotobuf/internal.pyx":341
 *             raise makeDecodeError(pointer[0], "Can't deserialize type tag at [{0}] for value")
 *         findex = tag >> 3
 *         f = fieldsmap.get(findex)             # <<<<<<<<<<<<<<
 *         if f is None:
 *             wtype= tag & 0x07
 */
    if (unlikely(__pyx_v_fieldsmap == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
      __PYX_ERR(0, 341, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyInt_From_uint32_t(__pyx_v_findex); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 341, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyDict_GetItemDefault(__pyx_v_fieldsmap, __pyx_t_1, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 341, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_9cprotobuf_8internal_Field))))) __PYX_ERR(0, 341, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_f, ((struct __pyx_obj_9cprotobuf_8internal_Field *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "cprotobuf/internal.pyx":342
 *         findex = tag >> 3
 *         f = fieldsmap.get(findex)
 *         if f is None:             # <<<<<<<<<<<<<<
 *             wtype= tag & 0x07
 *             if skip_unknown_field(pointer, end, wtype):
 */
    __pyx_t_2 = (((PyObject *)__pyx_v_f) == Py_None);
    __pyx_t_4 = (__pyx_t_2 != 0);
    if (__pyx_t_4) {

      /* "cprotobuf/internal.pyx":343
 *         f = fieldsmap.get(findex)
 *         if f is None:
 *             wtype= tag & 0x07             # <<<<<<<<<<<<<<
 *             if skip_unknown_field(pointer, end, wtype):
 *                 raise makeDecodeError(pointer[0], "Can't skip enough bytes for wire_type %s at [{0}] for value" % wtype)
 */
      __pyx_v_wtype = (__pyx_v_tag & 0x07);

      /* "cprotobuf/internal.pyx":344
 *         if f is None:
 *             wtype= tag & 0x07
 *             if skip_unknown_field(pointer, end, wtype):             # <<<<<<<<<<<<<<
 *                 raise makeDecodeError(pointer[0], "Can't skip enough bytes for wire_type %s at [{0}] for value" % wtype)
 *         else:
 */
      __pyx_t_4 = (__pyx_f_9cprotobuf_8internal_skip_unknown_field(__pyx_v_pointer, __pyx_v_end, __pyx_v_wtype) != 0);
      if (__pyx_t_4) {

        /* "cprotobuf/internal.pyx":345
 *             wtype= tag & 0x07
 *             if skip_unknown_field(pointer, end, wtype):
 *                 raise makeDecodeError(pointer[0], "Can't skip enough bytes for wire_type %s at [{0}] for value" % wtype)             # <<<<<<<<<<<<<<
 *         else:
 *             if f.packed:
 */
        __pyx_t_3 = __Pyx_PyInt_From_uint32_t(__pyx_v_wtype); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 345, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_Can_t_skip_enough_bytes_for_wire, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 345, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 345, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(0, 345, __pyx_L1_error)

        /* "cprotobuf/internal.pyx":344
 *         if f is None:
 *             wtype= tag & 0x07
 *             if skip_unknown_field(pointer, end, wtype):             # <<<<<<<<<<<<<<
 *                 raise makeDecodeError(pointer[0], "Can't skip enough bytes for wire_type %s at [{0}] for value" % wtype)
 *         else:
 */
      }

      /* "cprotobuf/internal.pyx":342
 *         findex = tag >> 3
 *         f = fieldsmap.get(findex)
 *         if f is None:             # <<<<<<<<<<<<<<
 *             wtype= tag & 0x07
 *             if skip_unknown_field(pointer, end, wtype):
 */
      goto __pyx_L6;
    }

    /* "cprotobuf/internal.pyx":347
 *                 raise makeDecodeError(pointer[0], "Can't skip enough bytes for wire_type %s at [{0}] for value" % wtype)
 *         else:
 *             if f.packed:             # <<<<<<<<<<<<<<
 *                 if raw_decode_delimited(pointer, end, &sub_buff, &sub_size):
 *                     raise makeDecodeError(pointer[0], "Can't decode value of type `packed` at [{0}]")
 */
    /*else*/ {
      __pyx_t_4 = (__pyx_v_f->packed != 0);
      if (__pyx_t_4) {

        /* "cprotobuf/internal.pyx":348
 *         else:
 *             if f.packed:
 *                 if raw_decode_delimited(pointer, end, &sub_buff, &sub_size):             # <<<<<<<<<<<<<<
 *                     raise makeDecodeError(pointer[0], "Can't decode value of type `packed` at [{0}]")
 *                 sub_end = sub_buff + sub_size
 */
        __pyx_t_4 = (__pyx_f_9cprotobuf_8internal_raw_decode_delimited(__pyx_v_pointer, __pyx_v_end, (&__pyx_v_sub_buff), (&__pyx_v_sub_size)) != 0);
        if (__pyx_t_4) {

          /* "cprotobuf/internal.pyx":349
 *             if f.packed:
 *                 if raw_decode_delimited(pointer, end, &sub_buff, &sub_size):
 *                     raise makeDecodeError(pointer[0], "Can't decode value of type `packed` at [{0}]")             # <<<<<<<<<<<<<<
 *                 sub_end = sub_buff + sub_size
 *                 l = d.setdefault(f.name, [])
 */
          __pyx_t_3 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_decode_value_of_type_packe); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 349, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_Raise(__pyx_t_3, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __PYX_ERR(0, 349, __pyx_L1_error)

          /* "cprotobuf/internal.pyx":348
 *         else:
 *             if f.packed:
 *                 if raw_decode_delimited(pointer, end, &sub_buff, &sub_size):             # <<<<<<<<<<<<<<
 *                     raise makeDecodeError(pointer[0], "Can't decode value of type `packed` at [{0}]")
 *                 sub_end = sub_buff + sub_size
 */
        }

        /* "cprotobuf/internal.pyx":350
 *                 if raw_decode_delimited(pointer, end, &sub_buff, &sub_size):
 *                     raise makeDecodeError(pointer[0], "Can't decode value of type `packed` at [{0}]")
 *                 sub_end = sub_buff + sub_size             # <<<<<<<<<<<<<<
 *                 l = d.setdefault(f.name, [])
 *                 while sub_buff < sub_end:
 */
        __pyx_v_sub_end = (__pyx_v_sub_buff + __pyx_v_sub_size);

        /* "cprotobuf/internal.pyx":351
 *                     raise makeDecodeError(pointer[0], "Can't decode value of type `packed` at [{0}]")
 *                 sub_end = sub_buff + sub_size
 *                 l = d.setdefault(f.name, [])             # <<<<<<<<<<<<<<
 *                 while sub_buff < sub_end:
 *                     l.append(f.decoder(&sub_buff, sub_end))
 */
        if (unlikely(__pyx_v_d == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "setdefault");
          __PYX_ERR(0, 351, __pyx_L1_error)
        }
        __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 351, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_1 = __Pyx_PyDict_SetDefault(__pyx_v_d, __pyx_v_f->name, __pyx_t_3, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 351, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 351, __pyx_L1_error)
        __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "cprotobuf/internal.pyx":352
 *                 sub_end = sub_buff + sub_size
 *                 l = d.setdefault(f.name, [])
 *                 while sub_buff < sub_end:             # <<<<<<<<<<<<<<
 *                     l.append(f.decoder(&sub_buff, sub_end))
 *             else:
 */
        while (1) {
          __pyx_t_4 = ((__pyx_v_sub_buff < __pyx_v_sub_end) != 0);
          if (!__pyx_t_4) break;

          /* "cprotobuf/internal.pyx":353
 *                 l = d.setdefault(f.name, [])
 *                 while sub_buff < sub_end:
 *                     l.append(f.decoder(&sub_buff, sub_end))             # <<<<<<<<<<<<<<
 *             else:
 *                 if f.klass is None:
 */
          if (unlikely(__pyx_v_l == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
            __PYX_ERR(0, 353, __pyx_L1_error)
          }
          __pyx_t_1 = __pyx_v_f->decoder((&__pyx_v_sub_buff), __pyx_v_sub_end); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 353, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_l, __pyx_t_1); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 353, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        }

        /* "cprotobuf/internal.pyx":347
 *                 raise makeDecodeError(pointer[0], "Can't skip enough bytes for wire_type %s at [{0}] for value" % wtype)
 *         else:
 *             if f.packed:             # <<<<<<<<<<<<<<
 *                 if raw_decode_delimited(pointer, end, &sub_buff, &sub_size):
 *                     raise makeDecodeError(pointer[0], "Can't decode value of type `packed` at [{0}]")
 */
        goto __pyx_L8;
      }

      /* "cprotobuf/internal.pyx":355
 *                     l.append(f.decoder(&sub_buff, sub_end))
 *             else:
 *                 if f.klass is None:             # <<<<<<<<<<<<<<
 *                     value = f.decoder(pointer, end)
 *                 else:
 */
      /*else*/ {
        __pyx_t_4 = (__pyx_v_f->klass == Py_None);
        __pyx_t_2 = (__pyx_t_4 != 0);
        if (__pyx_t_2) {

          /* "cprotobuf/internal.pyx":356
 *             else:
 *                 if f.klass is None:
 *                     value = f.decoder(pointer, end)             # <<<<<<<<<<<<<<
 *                 else:
 *                     if raw_decode_delimited(pointer, end, &sub_buff, &sub_size):
 */
          __pyx_t_1 = __pyx_v_f->decoder(__pyx_v_pointer, __pyx_v_end); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 356, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_1);
          __pyx_t_1 = 0;

          /* "cprotobuf/internal.pyx":355
 *                     l.append(f.decoder(&sub_buff, sub_end))
 *             else:
 *                 if f.klass is None:             # <<<<<<<<<<<<<<
 *                     value = f.decoder(pointer, end)
 *                 else:
 */
          goto __pyx_L12;
        }

        /* "cprotobuf/internal.pyx":358
 *                     value = f.decoder(pointer, end)
 *                 else:
 *                     if raw_decode_delimited(pointer, end, &sub_buff, &sub_size):             # <<<<<<<<<<<<<<
 *                         raise makeDecodeError(pointer[0], "Can't decode value of sub message at [{0}]")
 *                     f.resolve_klass()
 */
        /*else*/ {
          __pyx_t_2 = (__pyx_f_9cprotobuf_8internal_raw_decode_delimited(__pyx_v_pointer, __pyx_v_end, (&__pyx_v_sub_buff), (&__pyx_v_sub_size)) != 0);
          if (__pyx_t_2) {

            /* "cprotobuf/internal.pyx":359
 *                 else:
 *                     if raw_decode_delimited(pointer, end, &sub_buff, &sub_size):
 *                         raise makeDecodeError(pointer[0], "Can't decode value of sub message at [{0}]")             # <<<<<<<<<<<<<<
 *                     f.resolve_klass()
 *                     value = f.klass()
 */
            __pyx_t_1 = __pyx_f_9cprotobuf_8internal_makeDecodeError((__pyx_v_pointer[0]), __pyx_kp_s_Can_t_decode_value_of_sub_messag); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 359, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_Raise(__pyx_t_1, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __PYX_ERR(0, 359, __pyx_L1_error)

            /* "cprotobuf/internal.pyx":358
 *                     value = f.decoder(pointer, end)
 *                 else:
 *                     if raw_decode_delimited(pointer, end, &sub_buff, &sub_size):             # <<<<<<<<<<<<<<
 *                         raise makeDecodeError(pointer[0], "Can't decode value of sub message at [{0}]")
 *                     f.resolve_klass()
 */
          }

          /* "cprotobuf/internal.pyx":360
 *                     if raw_decode_delimited(pointer, end, &sub_buff, &sub_size):
 *                         raise makeDecodeError(pointer[0], "Can't decode value of sub message at [{0}]")
 *                     f.resolve_klass()             # <<<<<<<<<<<<<<
 *                     value = f.klass()
 *                     decode_object(value, &sub_buff, sub_buff+sub_size)
 */
          __pyx_t_1 = ((struct __pyx_vtabstruct_9cprotobuf_8internal_Field *)__pyx_v_f->__pyx_vtab)->resolve_klass(__pyx_v_f); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 360, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "cprotobuf/internal.pyx":361
 *                         raise makeDecodeError(pointer[0], "Can't decode value of sub message at [{0}]")
 *                     f.resolve_klass()
 *                     value = f.klass()             # <<<<<<<<<<<<<<
 *                     decode_object(value, &sub_buff, sub_buff+sub_size)
 *                 if f.repeated:
 */
          __Pyx_INCREF(__pyx_v_f->klass);
          __pyx_t_3 = __pyx_v_f->klass; __pyx_t_6 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
            if (likely(__pyx_t_6)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
              __Pyx_INCREF(__pyx_t_6);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_3, function);
            }
          }
          if (__pyx_t_6) {
            __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 361, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          } else {
            __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 361, __pyx_L1_error)
          }
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_1);
          __pyx_t_1 = 0;

          /* "cprotobuf/internal.pyx":362
 *                     f.resolve_klass()
 *                     value = f.klass()
 *                     decode_object(value, &sub_buff, sub_buff+sub_size)             # <<<<<<<<<<<<<<
 *                 if f.repeated:
 *                     l = d.setdefault(f.name, [])
 */
          __pyx_t_7 = __pyx_f_9cprotobuf_8internal_decode_object(__pyx_v_value, (&__pyx_v_sub_buff), (__pyx_v_sub_buff + __pyx_v_sub_size)); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 362, __pyx_L1_error)
        }
        __pyx_L12:;

        /* "cprotobuf/internal.pyx":363
 *                     value = f.klass()
 *                     decode_object(value, &sub_buff, sub_buff+sub_size)
 *                 if f.repeated:             # <<<<<<<<<<<<<<
 *                     l = d.setdefault(f.name, [])
 *                     l.append(value)
 */
        __pyx_t_2 = (__pyx_v_f->repeated != 0);
        if (__pyx_t_2) {

          /* "cprotobuf/internal.pyx":364
 *                     decode_object(value, &sub_buff, sub_buff+sub_size)
 *                 if f.repeated:
 *                     l = d.setdefault(f.name, [])             # <<<<<<<<<<<<<<
 *                     l.append(value)
 *                 else:
 */
          if (unlikely(__pyx_v_d == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "setdefault");
            __PYX_ERR(0, 364, __pyx_L1_error)
          }
          __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 364, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_3 = __Pyx_PyDict_SetDefault(__pyx_v_d, __pyx_v_f->name, __pyx_t_1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 364, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 364, __pyx_L1_error)
          __Pyx_XDECREF_SET(__pyx_v_l, ((PyObject*)__pyx_t_3));
          __pyx_t_3 = 0;

          /* "cprotobuf/internal.pyx":365
 *                 if f.repeated:
 *                     l = d.setdefault(f.name, [])
 *                     l.append(value)             # <<<<<<<<<<<<<<
 *                 else:
 *                     d[f.name] = value
 */
          if (unlikely(__pyx_v_l == Py_None)) {
            PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
            __PYX_ERR(0, 365, __pyx_L1_error)
          }
          __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_l, __pyx_v_value); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 365, __pyx_L1_error)

          /* "cprotobuf/internal.pyx":363
 *                     value = f.klass()
 *                     decode_object(value, &sub_buff, sub_buff+sub_size)
 *                 if f.repeated:             # <<<<<<<<<<<<<<
 *                     l = d.setdefault(f.name, [])
 *                     l.append(value)
 */
          goto __pyx_L14;
        }

        /* "cprotobuf/internal.pyx":367
 *                     l.append(value)
 *                 else:
 *                     d[f.name] = value             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
        /*else*/ {
          if (unlikely(__pyx_v_d == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
            __PYX_ERR(0, 367, __pyx_L1_error)
          }
          if (unlikely(PyDict_SetItem(__pyx_v_d, __pyx_v_f->name, __pyx_v_value) < 0)) __PYX_ERR(0, 367, __pyx_L1_error)
        }
        __pyx_L14:;
      }
      __pyx_L8:;
    }
    __pyx_L6:;
  }

  /* "cprotobuf/internal.pyx":369
 *                     d[f.name] = value
 * 
 *     return 0             # <<<<<<<<<<<<<<
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "cprotobuf/internal.pyx":325
 *     encode_bytes(f, array, sub_buf)
 * 
 * cdef inline int decode_object(object self, char **pointer, char *end) except -1:             # <<<<<<<<<<<<<<
 *     cdef dict fieldsmap = self._fieldsmap
 *     cdef uint32_t tag, wtype, findex
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cprotobuf.internal.decode_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_fieldsmap);
  __Pyx_XDECREF((PyObject *)__pyx_v_f);
  __Pyx_XDECREF(__pyx_v_d);
  __Pyx_XDECREF(__pyx_v_l);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_RepeatedContainer(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x64a34c4:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_9cprotobuf_8internal_7__pyx_unpickle_RepeatedContainer(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9cprotobuf_8internal_7__pyx_unpickle_RepeatedContainer = {"__pyx_unpickle_RepeatedContainer", (PyCFunction)__pyx_pw_9cprotobuf_8internal_7__pyx_unpickle_RepeatedContainer, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9cprotobuf_8internal_7__pyx_unpickle_RepeatedContainer(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_RepeatedContainer (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_RepeatedContainer", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_RepeatedContainer", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_RepeatedContainer") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_RepeatedContainer", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cprotobuf.internal.__pyx_unpickle_RepeatedContainer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9cprotobuf_8internal_6__pyx_unpickle_RepeatedContainer(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9cprotobuf_8internal_6__pyx_unpickle_RepeatedContainer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__pyx_unpickle_RepeatedContainer", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle_RepeatedContainer(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x64a34c4:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x64a34c4 = (klass))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x64a34c4) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle_RepeatedContainer(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x64a34c4:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x64a34c4 = (klass))" % __pyx_checksum)
 *     __pyx_result = RepeatedContainer.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0x64a34c4:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x64a34c4 = (klass))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = RepeatedContainer.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x64, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle_RepeatedContainer(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x64a34c4:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x64a34c4 = (klass))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x64a34c4 = (klass))" % __pyx_checksum)
 *     __pyx_result = RepeatedContainer.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_RepeatedContainer__set_state(<RepeatedContainer> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9cprotobuf_8internal_RepeatedContainer), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_INCREF(__pyx_v___pyx_type);
      __Pyx_GIVEREF(__pyx_v___pyx_type);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v___pyx_type);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x64a34c4 = (klass))" % __pyx_checksum)
 *     __pyx_result = RepeatedContainer.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_RepeatedContainer__set_state(<RepeatedContainer> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":7
 *     __pyx_result = RepeatedContainer.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_RepeatedContainer__set_state(<RepeatedContainer> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_RepeatedContainer__set_state(RepeatedContainer __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_9cprotobuf_8internal___pyx_unpickle_RepeatedContainer__set_state(((struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x64a34c4 = (klass))" % __pyx_checksum)
 *     __pyx_result = RepeatedContainer.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_RepeatedContainer__set_state(<RepeatedContainer> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle_RepeatedContainer__set_state(<RepeatedContainer> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_RepeatedContainer__set_state(RepeatedContainer __pyx_result, tuple __pyx_state):
 *     __pyx_result.klass = __pyx_state[0]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_RepeatedContainer(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x64a34c4:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cprotobuf.internal.__pyx_unpickle_RepeatedContainer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle_RepeatedContainer__set_state(<RepeatedContainer> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_RepeatedContainer__set_state(RepeatedContainer __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.klass = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_9cprotobuf_8internal___pyx_unpickle_RepeatedContainer__set_state(struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_RepeatedContainer__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_RepeatedContainer__set_state(RepeatedContainer __pyx_result, tuple __pyx_state):
 *     __pyx_result.klass = __pyx_state[0]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->klass);
  __Pyx_DECREF(__pyx_v___pyx_result->klass);
  __pyx_v___pyx_result->klass = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":11
 * cdef __pyx_unpickle_RepeatedContainer__set_state(RepeatedContainer __pyx_result, tuple __pyx_state):
 *     __pyx_result.klass = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 11, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 1) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":12
 *     __pyx_result.klass = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[1])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 12, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    if (!__pyx_t_8) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_6};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_6};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_6);
        __pyx_t_6 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":11
 * cdef __pyx_unpickle_RepeatedContainer__set_state(RepeatedContainer __pyx_result, tuple __pyx_state):
 *     __pyx_result.klass = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle_RepeatedContainer__set_state(<RepeatedContainer> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_RepeatedContainer__set_state(RepeatedContainer __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.klass = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("cprotobuf.internal.__pyx_unpickle_RepeatedContainer__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_tp_new_9cprotobuf_8internal_RepeatedContainer(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *p;
  PyObject *o = (&PyList_Type)->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *)o);
  p->klass = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9cprotobuf_8internal_RepeatedContainer(PyObject *o) {
  struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *p = (struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->klass);
  PyObject_GC_Track(o);
  (&PyList_Type)->tp_dealloc(o);
}

static int __pyx_tp_traverse_9cprotobuf_8internal_RepeatedContainer(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *p = (struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *)o;
  if (!(&PyList_Type)->tp_traverse); else { e = (&PyList_Type)->tp_traverse(o,v,a); if (e) return e; }
  if (p->klass) {
    e = (*v)(p->klass, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9cprotobuf_8internal_RepeatedContainer(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *p = (struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer *)o;
  if (!(&PyList_Type)->tp_clear); else (&PyList_Type)->tp_clear(o);
  tmp = ((PyObject*)p->klass);
  p->klass = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_9cprotobuf_8internal_RepeatedContainer[] = {
  {"add", (PyCFunction)__pyx_pw_9cprotobuf_8internal_17RepeatedContainer_3add, METH_VARARGS|METH_KEYWORDS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9cprotobuf_8internal_17RepeatedContainer_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9cprotobuf_8internal_17RepeatedContainer_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9cprotobuf_8internal_RepeatedContainer = {
  PyVarObject_HEAD_INIT(0, 0)
  "cprotobuf.internal.RepeatedContainer", /*tp_name*/
  sizeof(struct __pyx_obj_9cprotobuf_8internal_RepeatedContainer), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9cprotobuf_8internal_RepeatedContainer, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_9cprotobuf_8internal_RepeatedContainer, /*tp_traverse*/
  __pyx_tp_clear_9cprotobuf_8internal_RepeatedContainer, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9cprotobuf_8internal_RepeatedContainer, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9cprotobuf_8internal_17RepeatedContainer_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9cprotobuf_8internal_RepeatedContainer, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_9cprotobuf_8internal_Field __pyx_vtable_9cprotobuf_8internal_Field;

static PyObject *__pyx_tp_new_9cprotobuf_8internal_Field(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_9cprotobuf_8internal_Field *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9cprotobuf_8internal_Field *)o);
  p->__pyx_vtab = __pyx_vtabptr_9cprotobuf_8internal_Field;
  p->name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->type = Py_None; Py_INCREF(Py_None);
  p->klass = Py_None; Py_INCREF(Py_None);
  p->__pyx_default = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_9cprotobuf_8internal_Field(PyObject *o) {
  struct __pyx_obj_9cprotobuf_8internal_Field *p = (struct __pyx_obj_9cprotobuf_8internal_Field *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->name);
  Py_CLEAR(p->type);
  Py_CLEAR(p->klass);
  Py_CLEAR(p->__pyx_default);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_9cprotobuf_8internal_Field(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9cprotobuf_8internal_Field *p = (struct __pyx_obj_9cprotobuf_8internal_Field *)o;
  if (p->type) {
    e = (*v)(p->type, a); if (e) return e;
  }
  if (p->klass) {
    e = (*v)(p->klass, a); if (e) return e;
  }
  if (p->__pyx_default) {
    e = (*v)(p->__pyx_default, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9cprotobuf_8internal_Field(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9cprotobuf_8internal_Field *p = (struct __pyx_obj_9cprotobuf_8internal_Field *)o;
  tmp = ((PyObject*)p->type);
  p->type = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->klass);
  p->klass = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_default);
  p->__pyx_default = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_tp_descr_get_9cprotobuf_8internal_Field(PyObject *o, PyObject *i, PyObject *c) {
  PyObject *r = 0;
  if (!i) i = Py_None;
  if (!c) c = Py_None;
  r = __pyx_pw_9cprotobuf_8internal_5Field_3__get__(o, i, c);
  return r;
}

static PyObject *__pyx_getprop_9cprotobuf_8internal_5Field_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9cprotobuf_8internal_5Field_4name_1__get__(o);
}

static int __pyx_setprop_9cprotobuf_8internal_5Field_name(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9cprotobuf_8internal_5Field_4name_3__set__(o, v);
  }
  else {
    return __pyx_pw_9cprotobuf_8internal_5Field_4name_5__del__(o);
  }
}

static PyObject *__pyx_getprop_9cprotobuf_8internal_5Field_index(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9cprotobuf_8internal_5Field_5index_1__get__(o);
}

static int __pyx_setprop_9cprotobuf_8internal_5Field_index(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9cprotobuf_8internal_5Field_5index_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_9cprotobuf_8internal_5Field_packed(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9cprotobuf_8internal_5Field_6packed_1__get__(o);
}

static int __pyx_setprop_9cprotobuf_8internal_5Field_packed(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9cprotobuf_8internal_5Field_6packed_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_9cprotobuf_8internal_5Field_required(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9cprotobuf_8internal_5Field_8required_1__get__(o);
}

static int __pyx_setprop_9cprotobuf_8internal_5Field_required(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9cprotobuf_8internal_5Field_8required_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_9cprotobuf_8internal_5Field_repeated(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9cprotobuf_8internal_5Field_8repeated_1__get__(o);
}

static int __pyx_setprop_9cprotobuf_8internal_5Field_repeated(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9cprotobuf_8internal_5Field_8repeated_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_9cprotobuf_8internal_5Field_wire_type(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9cprotobuf_8internal_5Field_9wire_type_1__get__(o);
}

static int __pyx_setprop_9cprotobuf_8internal_5Field_wire_type(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_9cprotobuf_8internal_5Field_9wire_type_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_9cprotobuf_8internal_Field[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_9cprotobuf_8internal_5Field_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_9cprotobuf_8internal_5Field_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9cprotobuf_8internal_Field[] = {
  {(char *)"name", __pyx_getprop_9cprotobuf_8internal_5Field_name, __pyx_setprop_9cprotobuf_8internal_5Field_name, (char *)0, 0},
  {(char *)"index", __pyx_getprop_9cprotobuf_8internal_5Field_index, __pyx_setprop_9cprotobuf_8internal_5Field_index, (char *)0, 0},
  {(char *)"packed", __pyx_getprop_9cprotobuf_8internal_5Field_packed, __pyx_setprop_9cprotobuf_8internal_5Field_packed, (char *)0, 0},
  {(char *)"required", __pyx_getprop_9cprotobuf_8internal_5Field_required, __pyx_setprop_9cprotobuf_8internal_5Field_required, (char *)0, 0},
  {(char *)"repeated", __pyx_getprop_9cprotobuf_8internal_5Field_repeated, __pyx_setprop_9cprotobuf_8internal_5Field_repeated, (char *)0, 0},
  {(char *)"wire_type", __pyx_getprop_9cprotobuf_8internal_5Field_wire_type, __pyx_setprop_9cprotobuf_8internal_5Field_wire_type, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9cprotobuf_8internal_Field = {
  PyVarObject_HEAD_INIT(0, 0)
  "cprotobuf.internal.Field", /*tp_name*/
  sizeof(struct __pyx_obj_9cprotobuf_8internal_Field), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9cprotobuf_8internal_Field, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_9cprotobuf_8internal_Field, /*tp_traverse*/
  __pyx_tp_clear_9cprotobuf_8internal_Field, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9cprotobuf_8internal_Field, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9cprotobuf_8internal_Field, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  __pyx_tp_descr_get_9cprotobuf_8internal_Field, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9cprotobuf_8internal_5Field_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9cprotobuf_8internal_Field, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {"get_proto", (PyCFunction)__pyx_pw_9cprotobuf_8internal_1get_proto, METH_O, 0},
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_internal(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_internal},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "internal",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_Can_t_decode_size_for_value_of_t, __pyx_k_Can_t_decode_size_for_value_of_t, sizeof(__pyx_k_Can_t_decode_size_for_value_of_t), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_size_for_value_of_t_2, __pyx_k_Can_t_decode_size_for_value_of_t_2, sizeof(__pyx_k_Can_t_decode_size_for_value_of_t_2), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_value_of_sub_messag, __pyx_k_Can_t_decode_value_of_sub_messag, sizeof(__pyx_k_Can_t_decode_value_of_sub_messag), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_value_of_type_bytes, __pyx_k_Can_t_decode_value_of_type_bytes, sizeof(__pyx_k_Can_t_decode_value_of_type_bytes), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_value_of_type_doubl, __pyx_k_Can_t_decode_value_of_type_doubl, sizeof(__pyx_k_Can_t_decode_value_of_type_doubl), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_value_of_type_fixed, __pyx_k_Can_t_decode_value_of_type_fixed, sizeof(__pyx_k_Can_t_decode_value_of_type_fixed), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_value_of_type_fixed_2, __pyx_k_Can_t_decode_value_of_type_fixed_2, sizeof(__pyx_k_Can_t_decode_value_of_type_fixed_2), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_value_of_type_float, __pyx_k_Can_t_decode_value_of_type_float, sizeof(__pyx_k_Can_t_decode_value_of_type_float), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_value_of_type_int32, __pyx_k_Can_t_decode_value_of_type_int32, sizeof(__pyx_k_Can_t_decode_value_of_type_int32), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_value_of_type_int64, __pyx_k_Can_t_decode_value_of_type_int64, sizeof(__pyx_k_Can_t_decode_value_of_type_int64), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_value_of_type_packe, __pyx_k_Can_t_decode_value_of_type_packe, sizeof(__pyx_k_Can_t_decode_value_of_type_packe), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_value_of_type_sfixe, __pyx_k_Can_t_decode_value_of_type_sfixe, sizeof(__pyx_k_Can_t_decode_value_of_type_sfixe), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_value_of_type_sfixe_2, __pyx_k_Can_t_decode_value_of_type_sfixe_2, sizeof(__pyx_k_Can_t_decode_value_of_type_sfixe_2), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_value_of_type_sint3, __pyx_k_Can_t_decode_value_of_type_sint3, sizeof(__pyx_k_Can_t_decode_value_of_type_sint3), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_value_of_type_sint6, __pyx_k_Can_t_decode_value_of_type_sint6, sizeof(__pyx_k_Can_t_decode_value_of_type_sint6), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_value_of_type_strin, __pyx_k_Can_t_decode_value_of_type_strin, sizeof(__pyx_k_Can_t_decode_value_of_type_strin), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_value_of_type_uint3, __pyx_k_Can_t_decode_value_of_type_uint3, sizeof(__pyx_k_Can_t_decode_value_of_type_uint3), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_decode_value_of_type_uint6, __pyx_k_Can_t_decode_value_of_type_uint6, sizeof(__pyx_k_Can_t_decode_value_of_type_uint6), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_deserialize_type_tag_at_0, __pyx_k_Can_t_deserialize_type_tag_at_0, sizeof(__pyx_k_Can_t_deserialize_type_tag_at_0), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_skip_enough_bytes_for_wire, __pyx_k_Can_t_skip_enough_bytes_for_wire, sizeof(__pyx_k_Can_t_skip_enough_bytes_for_wire), 0, 0, 1, 0},
  {&__pyx_kp_s_Count_out_of_bound, __pyx_k_Count_out_of_bound, sizeof(__pyx_k_Count_out_of_bound), 0, 0, 1, 0},
  {&__pyx_n_s_DecodeError, __pyx_k_DecodeError, sizeof(__pyx_k_DecodeError), 0, 0, 1, 1},
  {&__pyx_n_s_DecodeError___init, __pyx_k_DecodeError___init, sizeof(__pyx_k_DecodeError___init), 0, 0, 1, 1},
  {&__pyx_n_s_DecodeError___str, __pyx_k_DecodeError___str, sizeof(__pyx_k_DecodeError___str), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x64, __pyx_k_Incompatible_checksums_s_vs_0x64, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x64), 0, 0, 1, 0},
  {&__pyx_n_s_InternalDecodeError, __pyx_k_InternalDecodeError, sizeof(__pyx_k_InternalDecodeError), 0, 0, 1, 1},
  {&__pyx_n_s_KeyError, __pyx_k_KeyError, sizeof(__pyx_k_KeyError), 0, 0, 1, 1},
  {&__pyx_n_s_MetaProtoEntity, __pyx_k_MetaProtoEntity, sizeof(__pyx_k_MetaProtoEntity), 0, 0, 1, 1},
  {&__pyx_n_s_MetaProtoEntity___new, __pyx_k_MetaProtoEntity___new, sizeof(__pyx_k_MetaProtoEntity___new), 0, 0, 1, 1},
  {&__pyx_n_s_MetaProtoEntity___new___locals_l, __pyx_k_MetaProtoEntity___new___locals_l, sizeof(__pyx_k_MetaProtoEntity___new___locals_l), 0, 0, 1, 1},
  {&__pyx_kp_s_Offset_out_of_bound, __pyx_k_Offset_out_of_bound, sizeof(__pyx_k_Offset_out_of_bound), 0, 0, 1, 0},
  {&__pyx_n_s_ParseFromString, __pyx_k_ParseFromString, sizeof(__pyx_k_ParseFromString), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_ProtoEntity, __pyx_k_ProtoEntity, sizeof(__pyx_k_ProtoEntity), 0, 0, 1, 1},
  {&__pyx_n_s_ProtoEntity_ParseFromString, __pyx_k_ProtoEntity_ParseFromString, sizeof(__pyx_k_ProtoEntity_ParseFromString), 0, 0, 1, 1},
  {&__pyx_n_s_ProtoEntity_SerializeToString, __pyx_k_ProtoEntity_SerializeToString, sizeof(__pyx_k_ProtoEntity_SerializeToString), 0, 0, 1, 1},
  {&__pyx_n_s_ProtoEntity___init, __pyx_k_ProtoEntity___init, sizeof(__pyx_k_ProtoEntity___init), 0, 0, 1, 1},
  {&__pyx_n_s_ProtoEntity___str, __pyx_k_ProtoEntity___str, sizeof(__pyx_k_ProtoEntity___str), 0, 0, 1, 1},
  {&__pyx_n_s_ProtoEntity___unicode, __pyx_k_ProtoEntity___unicode, sizeof(__pyx_k_ProtoEntity___unicode), 0, 0, 1, 1},
  {&__pyx_n_s_ProtoEntity_todict, __pyx_k_ProtoEntity_todict, sizeof(__pyx_k_ProtoEntity_todict), 0, 0, 1, 1},
  {&__pyx_n_s_SerializeToString, __pyx_k_SerializeToString, sizeof(__pyx_k_SerializeToString), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_s__10, __pyx_k__10, sizeof(__pyx_k__10), 0, 0, 1, 0},
  {&__pyx_kp_u__10, __pyx_k__10, sizeof(__pyx_k__10), 0, 1, 0, 0},
  {&__pyx_n_s__3, __pyx_k__3, sizeof(__pyx_k__3), 0, 0, 1, 1},
  {&__pyx_n_s_append, __pyx_k_append, sizeof(__pyx_k_append), 0, 0, 1, 1},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_attrs, __pyx_k_attrs, sizeof(__pyx_k_attrs), 0, 0, 1, 1},
  {&__pyx_n_s_bases, __pyx_k_bases, sizeof(__pyx_k_bases), 0, 0, 1, 1},
  {&__pyx_n_s_bool, __pyx_k_bool, sizeof(__pyx_k_bool), 0, 0, 1, 1},
  {&__pyx_n_s_buf, __pyx_k_buf, sizeof(__pyx_k_buf), 0, 0, 1, 1},
  {&__pyx_n_s_buf1, __pyx_k_buf1, sizeof(__pyx_k_buf1), 0, 0, 1, 1},
  {&__pyx_n_s_buff, __pyx_k_buff, sizeof(__pyx_k_buff), 0, 0, 1, 1},
  {&__pyx_n_s_bytes, __pyx_k_bytes, sizeof(__pyx_k_bytes), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_cls, __pyx_k_cls, sizeof(__pyx_k_cls), 0, 0, 1, 1},
  {&__pyx_n_s_clsname, __pyx_k_clsname, sizeof(__pyx_k_clsname), 0, 0, 1, 1},
  {&__pyx_n_s_count, __pyx_k_count, sizeof(__pyx_k_count), 0, 0, 1, 1},
  {&__pyx_n_s_cprotobuf_internal, __pyx_k_cprotobuf_internal, sizeof(__pyx_k_cprotobuf_internal), 0, 0, 1, 1},
  {&__pyx_kp_s_cprotobuf_internal_pyx, __pyx_k_cprotobuf_internal_pyx, sizeof(__pyx_k_cprotobuf_internal_pyx), 0, 0, 1, 0},
  {&__pyx_kp_s_cprotobuf_utils_pxi, __pyx_k_cprotobuf_utils_pxi, sizeof(__pyx_k_cprotobuf_utils_pxi), 0, 0, 1, 0},
  {&__pyx_n_s_d, __pyx_k_d, sizeof(__pyx_k_d), 0, 0, 1, 1},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_s_decode, __pyx_k_decode, sizeof(__pyx_k_decode), 0, 0, 1, 1},
  {&__pyx_n_s_default, __pyx_k_default, sizeof(__pyx_k_default), 0, 0, 1, 1},
  {&__pyx_n_s_default_objects, __pyx_k_default_objects, sizeof(__pyx_k_default_objects), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_double, __pyx_k_double, sizeof(__pyx_k_double), 0, 0, 1, 1},
  {&__pyx_kp_s_duplicate_field_index_s, __pyx_k_duplicate_field_index_s, sizeof(__pyx_k_duplicate_field_index_s), 0, 0, 1, 0},
  {&__pyx_n_s_e, __pyx_k_e, sizeof(__pyx_k_e), 0, 0, 1, 1},
  {&__pyx_n_s_encode_data, __pyx_k_encode_data, sizeof(__pyx_k_encode_data), 0, 0, 1, 1},
  {&__pyx_n_s_end, __pyx_k_end, sizeof(__pyx_k_end), 0, 0, 1, 1},
  {&__pyx_n_s_enum, __pyx_k_enum, sizeof(__pyx_k_enum), 0, 0, 1, 1},
  {&__pyx_n_s_f, __pyx_k_f, sizeof(__pyx_k_f), 0, 0, 1, 1},
  {&__pyx_n_s_fields, __pyx_k_fields, sizeof(__pyx_k_fields), 0, 0, 1, 1},
  {&__pyx_n_s_fieldsmap, __pyx_k_fieldsmap, sizeof(__pyx_k_fieldsmap), 0, 0, 1, 1},
  {&__pyx_n_s_fieldsmap_by_name, __pyx_k_fieldsmap_by_name, sizeof(__pyx_k_fieldsmap_by_name), 0, 0, 1, 1},
  {&__pyx_n_s_fixed32, __pyx_k_fixed32, sizeof(__pyx_k_fixed32), 0, 0, 1, 1},
  {&__pyx_n_s_fixed64, __pyx_k_fixed64, sizeof(__pyx_k_fixed64), 0, 0, 1, 1},
  {&__pyx_n_s_float, __pyx_k_float, sizeof(__pyx_k_float), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_inspect, __pyx_k_inspect, sizeof(__pyx_k_inspect), 0, 0, 1, 1},
  {&__pyx_n_s_int32, __pyx_k_int32, sizeof(__pyx_k_int32), 0, 0, 1, 1},
  {&__pyx_n_s_int64, __pyx_k_int64, sizeof(__pyx_k_int64), 0, 0, 1, 1},
  {&__pyx_kp_s_invalid_type_s, __pyx_k_invalid_type_s, sizeof(__pyx_k_invalid_type_s), 0, 0, 1, 0},
  {&__pyx_n_s_isclass, __pyx_k_isclass, sizeof(__pyx_k_isclass), 0, 0, 1, 1},
  {&__pyx_n_s_item, __pyx_k_item, sizeof(__pyx_k_item), 0, 0, 1, 1},
  {&__pyx_n_s_items, __pyx_k_items, sizeof(__pyx_k_items), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_kwargs, __pyx_k_kwargs, sizeof(__pyx_k_kwargs), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_message, __pyx_k_message, sizeof(__pyx_k_message), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_newcls, __pyx_k_newcls, sizeof(__pyx_k_newcls), 0, 0, 1, 1},
  {&__pyx_n_s_object, __pyx_k_object, sizeof(__pyx_k_object), 0, 0, 1, 1},
  {&__pyx_n_s_offset, __pyx_k_offset, sizeof(__pyx_k_offset), 0, 0, 1, 1},
  {&__pyx_n_s_packed, __pyx_k_packed, sizeof(__pyx_k_packed), 0, 0, 1, 1},
  {&__pyx_kp_s_packed_must_be_used_with_repeate, __pyx_k_packed_must_be_used_with_repeate, sizeof(__pyx_k_packed_must_be_used_with_repeate), 0, 0, 1, 0},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_pointer, __pyx_k_pointer, sizeof(__pyx_k_pointer), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_RepeatedContainer, __pyx_k_pyx_unpickle_RepeatedContainer, sizeof(__pyx_k_pyx_unpickle_RepeatedContainer), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_register_proto, __pyx_k_register_proto, sizeof(__pyx_k_register_proto), 0, 0, 1, 1},
  {&__pyx_n_s_repeated, __pyx_k_repeated, sizeof(__pyx_k_repeated), 0, 0, 1, 1},
  {&__pyx_n_s_required, __pyx_k_required, sizeof(__pyx_k_required), 0, 0, 1, 1},
  {&__pyx_n_s_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_kp_s_self_decoder_self_encoder_cannot, __pyx_k_self_decoder_self_encoder_cannot, sizeof(__pyx_k_self_decoder_self_encoder_cannot), 0, 0, 1, 0},
  {&__pyx_n_s_setdefault, __pyx_k_setdefault, sizeof(__pyx_k_setdefault), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_sfixed32, __pyx_k_sfixed32, sizeof(__pyx_k_sfixed32), 0, 0, 1, 1},
  {&__pyx_n_s_sfixed64, __pyx_k_sfixed64, sizeof(__pyx_k_sfixed64), 0, 0, 1, 1},
  {&__pyx_n_s_sint32, __pyx_k_sint32, sizeof(__pyx_k_sint32), 0, 0, 1, 1},
  {&__pyx_n_s_sint64, __pyx_k_sint64, sizeof(__pyx_k_sint64), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_sort, __pyx_k_sort, sizeof(__pyx_k_sort), 0, 0, 1, 1},
  {&__pyx_n_s_start, __pyx_k_start, sizeof(__pyx_k_start), 0, 0, 1, 1},
  {&__pyx_n_s_startswith, __pyx_k_startswith, sizeof(__pyx_k_startswith), 0, 0, 1, 1},
  {&__pyx_n_s_str, __pyx_k_str, sizeof(__pyx_k_str), 0, 0, 1, 1},
  {&__pyx_n_s_string, __pyx_k_string, sizeof(__pyx_k_string), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_super, __pyx_k_super, sizeof(__pyx_k_super), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_todict, __pyx_k_todict, sizeof(__pyx_k_todict), 0, 0, 1, 1},
  {&__pyx_n_s_type, __pyx_k_type, sizeof(__pyx_k_type), 0, 0, 1, 1},
  {&__pyx_n_s_uint32, __pyx_k_uint32, sizeof(__pyx_k_uint32), 0, 0, 1, 1},
  {&__pyx_n_s_uint64, __pyx_k_uint64, sizeof(__pyx_k_uint64), 0, 0, 1, 1},
  {&__pyx_n_s_unicode, __pyx_k_unicode, sizeof(__pyx_k_unicode), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_kp_s_utf_8, __pyx_k_utf_8, sizeof(__pyx_k_utf_8), 0, 0, 1, 0},
  {&__pyx_n_s_v, __pyx_k_v, sizeof(__pyx_k_v), 0, 0, 1, 1},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_wire_types, __pyx_k_wire_types, sizeof(__pyx_k_wire_types), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_object = __Pyx_GetBuiltinName(__pyx_n_s_object); if (!__pyx_builtin_object) __PYX_ERR(0, 233, __pyx_L1_error)
  __pyx_builtin_KeyError = __Pyx_GetBuiltinName(__pyx_n_s_KeyError); if (!__pyx_builtin_KeyError) __PYX_ERR(0, 124, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_n_s_super); if (!__pyx_builtin_super) __PYX_ERR(0, 207, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.decoder,self.encoder cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.decoder,self.encoder cannot be converted to a Python object for pickling")
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_self_decoder_self_encoder_cannot); if (unlikely(!__pyx_tuple_)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "(tree fragment)":4
 *     raise TypeError("self.decoder,self.encoder cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.decoder,self.encoder cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_self_decoder_self_encoder_cannot); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "cprotobuf/internal.pyx":215
 *         cdef Field f
 *         for name, v in attrs.items():
 *             if name.startswith('__'):             # <<<<<<<<<<<<<<
 *                 continue
 *             if not isinstance(v, Field):
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_n_s__3); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "cprotobuf/internal.pyx":264
 * 
 *     def __unicode__(self):
 *         return str(self).decode('utf-8')             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_utf_8); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "cprotobuf/utils.pxi":23
 * 
 * class DecodeError(Exception):
 *     def __init__(self, pointer, message):             # <<<<<<<<<<<<<<
 *         self.pointer = pointer
 *         self.message = message
 */
  __pyx_tuple__6 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_pointer, __pyx_n_s_message); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(2, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);
  __pyx_codeobj__7 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__6, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cprotobuf_utils_pxi, __pyx_n_s_init, 23, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__7)) __PYX_ERR(2, 23, __pyx_L1_error)

  /* "cprotobuf/utils.pxi":26
 *         self.pointer = pointer
 *         self.message = message
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message.format(self.pointer)
 * 
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(2, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);
  __pyx_codeobj__9 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__8, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cprotobuf_utils_pxi, __pyx_n_s_str, 26, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__9)) __PYX_ERR(2, 26, __pyx_L1_error)

  /* "cprotobuf/internal.pyx":201
 *     return _proto_classes[name]
 * 
 * def register_proto(name, cls):             # <<<<<<<<<<<<<<
 *     _proto_classes[name] = cls
 * 
 */
  __pyx_tuple__11 = PyTuple_Pack(2, __pyx_n_s_name, __pyx_n_s_cls); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);
  __pyx_codeobj__12 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cprotobuf_internal_pyx, __pyx_n_s_register_proto, 201, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__12)) __PYX_ERR(0, 201, __pyx_L1_error)

  /* "cprotobuf/internal.pyx":205
 * 
 * class MetaProtoEntity(type):
 *     def __new__(cls, clsname, bases, attrs):             # <<<<<<<<<<<<<<
 *         if clsname == 'ProtoEntity':
 *             return super(MetaProtoEntity, cls).__new__(cls, clsname, bases, attrs)
 */
  __pyx_tuple__13 = PyTuple_Pack(11, __pyx_n_s_cls, __pyx_n_s_clsname, __pyx_n_s_bases, __pyx_n_s_attrs, __pyx_n_s_fields, __pyx_n_s_fieldsmap, __pyx_n_s_fieldsmap_by_name, __pyx_n_s_f, __pyx_n_s_name, __pyx_n_s_v, __pyx_n_s_newcls); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);
  __pyx_codeobj__14 = (PyObject*)__Pyx_PyCode_New(4, 0, 11, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__13, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cprotobuf_internal_pyx, __pyx_n_s_new, 205, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__14)) __PYX_ERR(0, 205, __pyx_L1_error)

  /* "cprotobuf/internal.pyx":233
 *         return newcls
 * 
 * class ProtoEntity(object):             # <<<<<<<<<<<<<<
 *     __metaclass__ = MetaProtoEntity
 * 
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "cprotobuf/internal.pyx":236
 *     __metaclass__ = MetaProtoEntity
 * 
 *     def __init__(self, **kwargs):             # <<<<<<<<<<<<<<
 *         self.__dict__.update(kwargs)
 * 
 */
  __pyx_tuple__16 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_kwargs); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);
  __pyx_codeobj__17 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS|CO_VARKEYWORDS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__16, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cprotobuf_internal_pyx, __pyx_n_s_init, 236, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__17)) __PYX_ERR(0, 236, __pyx_L1_error)

  /* "cprotobuf/internal.pyx":239
 *         self.__dict__.update(kwargs)
 * 
 *     def SerializeToString(self):             # <<<<<<<<<<<<<<
 *         return encode_data(type(self), self.__dict__)
 * 
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);
  __pyx_codeobj__19 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__18, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cprotobuf_internal_pyx, __pyx_n_s_SerializeToString, 239, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__19)) __PYX_ERR(0, 239, __pyx_L1_error)

  /* "cprotobuf/internal.pyx":242
 *         return encode_data(type(self), self.__dict__)
 * 
 *     def ParseFromString(self, s, int offset=0, int count=-1):             # <<<<<<<<<<<<<<
 *         cdef char *buff = <char*>s
 *         cdef char *start
 */
  __pyx_tuple__20 = PyTuple_Pack(9, __pyx_n_s_self, __pyx_n_s_s, __pyx_n_s_offset, __pyx_n_s_count, __pyx_n_s_buff, __pyx_n_s_start, __pyx_n_s_end, __pyx_n_s_size, __pyx_n_s_e); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);
  __pyx_codeobj__21 = (PyObject*)__Pyx_PyCode_New(4, 0, 9, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__20, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cprotobuf_internal_pyx, __pyx_n_s_ParseFromString, 242, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__21)) __PYX_ERR(0, 242, __pyx_L1_error)

  /* "cprotobuf/internal.pyx":263
 *             raise DecodeError(e.args[0] - <uint64_t>start, e.args[1])
 * 
 *     def __unicode__(self):             # <<<<<<<<<<<<<<
 *         return str(self).decode('utf-8')
 * 
 */
  __pyx_tuple__22 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);
  __pyx_codeobj__23 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__22, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cprotobuf_internal_pyx, __pyx_n_s_unicode, 263, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__23)) __PYX_ERR(0, 263, __pyx_L1_error)

  /* "cprotobuf/internal.pyx":266
 *         return str(self).decode('utf-8')
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return str(self.todict())
 * 
 */
  __pyx_tuple__24 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);
  __pyx_codeobj__25 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__24, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cprotobuf_internal_pyx, __pyx_n_s_str, 266, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__25)) __PYX_ERR(0, 266, __pyx_L1_error)

  /* "cprotobuf/internal.pyx":269
 *         return str(self.todict())
 * 
 *     def todict(self):             # <<<<<<<<<<<<<<
 *         cdef Field f
 *         data = {}
 */
  __pyx_tuple__26 = PyTuple_Pack(6, __pyx_n_s_self, __pyx_n_s_f, __pyx_n_s_data, __pyx_n_s_d, __pyx_n_s_value, __pyx_n_s_v); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(0, 269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);
  __pyx_codeobj__27 = (PyObject*)__Pyx_PyCode_New(1, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__26, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cprotobuf_internal_pyx, __pyx_n_s_todict, 269, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__27)) __PYX_ERR(0, 269, __pyx_L1_error)

  /* "cprotobuf/internal.pyx":291
 *         return data
 * 
 * def encode_data(cls, dict d):             # <<<<<<<<<<<<<<
 *     cdef bytearray buf = bytearray()
 *     cdef bytearray buf1
 */
  __pyx_tuple__28 = PyTuple_Pack(7, __pyx_n_s_cls, __pyx_n_s_d, __pyx_n_s_buf, __pyx_n_s_buf1, __pyx_n_s_f, __pyx_n_s_value, __pyx_n_s_item); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(0, 291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);
  __pyx_codeobj__29 = (PyObject*)__Pyx_PyCode_New(2, 0, 7, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__28, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cprotobuf_internal_pyx, __pyx_n_s_encode_data, 291, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__29)) __PYX_ERR(0, 291, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_RepeatedContainer(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x64a34c4:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_tuple__30 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);
  __pyx_codeobj__31 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__30, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_RepeatedContainer, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__31)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_0_0 = PyFloat_FromDouble(0.0); if (unlikely(!__pyx_float_0_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_5 = PyInt_FromLong(5); if (unlikely(!__pyx_int_5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_105526468 = PyInt_FromLong(105526468L); if (unlikely(!__pyx_int_105526468)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initinternal(void); /*proto*/
PyMODINIT_FUNC initinternal(void)
#else
PyMODINIT_FUNC PyInit_internal(void); /*proto*/
PyMODINIT_FUNC PyInit_internal(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_internal(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #endif
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_internal(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("internal", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_cprotobuf__internal) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "cprotobuf.internal")) {
      if (unlikely(PyDict_SetItemString(modules, "cprotobuf.internal", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  __pyx_v_9cprotobuf_8internal__proto_classes = ((PyObject*)Py_None); Py_INCREF(Py_None);
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  __pyx_type_9cprotobuf_8internal_RepeatedContainer.tp_base = (&PyList_Type);
  if (PyType_Ready(&__pyx_type_9cprotobuf_8internal_RepeatedContainer) < 0) __PYX_ERR(0, 43, __pyx_L1_error)
  __pyx_type_9cprotobuf_8internal_RepeatedContainer.tp_print = 0;
  if (PyObject_SetAttrString(__pyx_m, "RepeatedContainer", (PyObject *)&__pyx_type_9cprotobuf_8internal_RepeatedContainer) < 0) __PYX_ERR(0, 43, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9cprotobuf_8internal_RepeatedContainer) < 0) __PYX_ERR(0, 43, __pyx_L1_error)
  __pyx_ptype_9cprotobuf_8internal_RepeatedContainer = &__pyx_type_9cprotobuf_8internal_RepeatedContainer;
  __pyx_vtabptr_9cprotobuf_8internal_Field = &__pyx_vtable_9cprotobuf_8internal_Field;
  __pyx_vtable_9cprotobuf_8internal_Field.resolve_klass = (PyObject *(*)(struct __pyx_obj_9cprotobuf_8internal_Field *))__pyx_f_9cprotobuf_8internal_5Field_resolve_klass;
  __pyx_vtable_9cprotobuf_8internal_Field.get_wire_type = (unsigned char (*)(struct __pyx_obj_9cprotobuf_8internal_Field *))__pyx_f_9cprotobuf_8internal_5Field_get_wire_type;
  __pyx_vtable_9cprotobuf_8internal_Field.get_encoder = (__pyx_t_9cprotobuf_8internal_Encoder (*)(struct __pyx_obj_9cprotobuf_8internal_Field *))__pyx_f_9cprotobuf_8internal_5Field_get_encoder;
  __pyx_vtable_9cprotobuf_8internal_Field.get_decoder = (__pyx_t_9cprotobuf_8internal_Decoder (*)(struct __pyx_obj_9cprotobuf_8internal_Field *))__pyx_f_9cprotobuf_8internal_5Field_get_decoder;
  if (PyType_Ready(&__pyx_type_9cprotobuf_8internal_Field) < 0) __PYX_ERR(0, 54, __pyx_L1_error)
  __pyx_type_9cprotobuf_8internal_Field.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_9cprotobuf_8internal_Field.tp_dict, __pyx_vtabptr_9cprotobuf_8internal_Field) < 0) __PYX_ERR(0, 54, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Field", (PyObject *)&__pyx_type_9cprotobuf_8internal_Field) < 0) __PYX_ERR(0, 54, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_9cprotobuf_8internal_Field) < 0) __PYX_ERR(0, 54, __pyx_L1_error)
  __pyx_ptype_9cprotobuf_8internal_Field = &__pyx_type_9cprotobuf_8internal_Field;
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) __PYX_ERR(3, 9, __pyx_L1_error)
  __pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "bool", sizeof(PyBoolObject), 0); if (unlikely(!__pyx_ptype_7cpython_4bool_bool)) __PYX_ERR(4, 8, __pyx_L1_error)
  __pyx_ptype_7cpython_7complex_complex = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "complex", sizeof(PyComplexObject), 0); if (unlikely(!__pyx_ptype_7cpython_7complex_complex)) __PYX_ERR(5, 15, __pyx_L1_error)
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "cprotobuf/utils.pxi":15
 * ctypedef object(*Encoder)(Field _f, bytearray array, object value)
 * 
 * class InternalDecodeError(Exception):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __Pyx_GIVEREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_InternalDecodeError, __pyx_n_s_InternalDecodeError, (PyObject *) NULL, __pyx_n_s_cprotobuf_internal, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_InternalDecodeError, __pyx_t_1, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_InternalDecodeError, __pyx_t_4) < 0) __PYX_ERR(2, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cprotobuf/utils.pxi":22
 *     return InternalDecodeError(locator, message)
 * 
 * class DecodeError(Exception):             # <<<<<<<<<<<<<<
 *     def __init__(self, pointer, message):
 *         self.pointer = pointer
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __Pyx_GIVEREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_DecodeError, __pyx_n_s_DecodeError, (PyObject *) NULL, __pyx_n_s_cprotobuf_internal, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "cprotobuf/utils.pxi":23
 * 
 * class DecodeError(Exception):
 *     def __init__(self, pointer, message):             # <<<<<<<<<<<<<<
 *         self.pointer = pointer
 *         self.message = message
 */
  __pyx_t_4 = __Pyx_CyFunction_NewEx(&__pyx_mdef_9cprotobuf_8internal_11DecodeError_1__init__, 0, __pyx_n_s_DecodeError___init, NULL, __pyx_n_s_cprotobuf_internal, __pyx_d, ((PyObject *)__pyx_codeobj__7)); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetItem(__pyx_t_3, __pyx_n_s_init, __pyx_t_4) < 0) __PYX_ERR(2, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cprotobuf/utils.pxi":26
 *         self.pointer = pointer
 *         self.message = message
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.message.format(self.pointer)
 * 
 */
  __pyx_t_4 = __Pyx_CyFunction_NewEx(&__pyx_mdef_9cprotobuf_8internal_11DecodeError_3__str__, 0, __pyx_n_s_DecodeError___str, NULL, __pyx_n_s_cprotobuf_internal, __pyx_d, ((PyObject *)__pyx_codeobj__9)); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetItem(__pyx_t_3, __pyx_n_s_str, __pyx_t_4) < 0) __PYX_ERR(2, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cprotobuf/utils.pxi":22
 *     return InternalDecodeError(locator, message)
 * 
 * class DecodeError(Exception):             # <<<<<<<<<<<<<<
 *     def __init__(self, pointer, message):
 *         self.pointer = pointer
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_DecodeError, __pyx_t_1, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DecodeError, __pyx_t_4) < 0) __PYX_ERR(2, 22, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":2
 * include "utils.pxi"
 * import inspect             # <<<<<<<<<<<<<<
 * #import traceback
 * 
 */
  __pyx_t_1 = __Pyx_patch_inspect(__Pyx_Import(__pyx_n_s_inspect, 0, -1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_inspect, __pyx_t_1) < 0) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":6
 * 
 * wire_types = {
 *     'int32': 0,             # <<<<<<<<<<<<<<
 *     'int64': 0,
 *     'sint32': 0,
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(16); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_int32, __pyx_int_0) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_int64, __pyx_int_0) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_sint32, __pyx_int_0) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_sint64, __pyx_int_0) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_uint32, __pyx_int_0) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_uint64, __pyx_int_0) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_bool, __pyx_int_0) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_enum, __pyx_int_0) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_fixed64, __pyx_int_1) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_sfixed64, __pyx_int_1) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_double, __pyx_int_1) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_string, __pyx_int_2) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_bytes, __pyx_int_2) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_fixed32, __pyx_int_5) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_sfixed32, __pyx_int_5) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_float, __pyx_int_5) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_wire_types, __pyx_t_1) < 0) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":25
 * 
 * default_objects = {
 *     'int32': 0,             # <<<<<<<<<<<<<<
 *     'int64': 0,
 *     'sint32': 0,
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(16); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_int32, __pyx_int_0) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_int64, __pyx_int_0) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_sint32, __pyx_int_0) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_sint64, __pyx_int_0) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_uint32, __pyx_int_0) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_uint64, __pyx_int_0) < 0) __PYX_ERR(0, 25, __pyx_L1_error)

  /* "cprotobuf/internal.pyx":31
 *     'uint32': 0,
 *     'uint64': 0,
 *     'bool': False,             # <<<<<<<<<<<<<<
 *     'enum': 0,
 *     'fixed64': 0,
 */
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_bool, Py_False) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_enum, __pyx_int_0) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_fixed64, __pyx_int_0) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_sfixed64, __pyx_int_0) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_double, __pyx_float_0_0) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_string, __pyx_kp_u__10) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_bytes, __pyx_kp_s__10) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_fixed32, __pyx_int_0) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_sfixed32, __pyx_int_0) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_float, __pyx_float_0_0) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_default_objects, __pyx_t_1) < 0) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":197
 *         return NULL
 * 
 * cdef dict _proto_classes = {}             # <<<<<<<<<<<<<<
 * cpdef get_proto(name):
 *     return _proto_classes[name]
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(__pyx_v_9cprotobuf_8internal__proto_classes);
  __Pyx_DECREF_SET(__pyx_v_9cprotobuf_8internal__proto_classes, ((PyObject*)__pyx_t_1));
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":201
 *     return _proto_classes[name]
 * 
 * def register_proto(name, cls):             # <<<<<<<<<<<<<<
 *     _proto_classes[name] = cls
 * 
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_9cprotobuf_8internal_3register_proto, NULL, __pyx_n_s_cprotobuf_internal); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_register_proto, __pyx_t_1) < 0) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":204
 *     _proto_classes[name] = cls
 * 
 * class MetaProtoEntity(type):             # <<<<<<<<<<<<<<
 *     def __new__(cls, clsname, bases, attrs):
 *         if clsname == 'ProtoEntity':
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_7cpython_4type_type));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_7cpython_4type_type));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_7cpython_4type_type));
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_MetaProtoEntity, __pyx_n_s_MetaProtoEntity, (PyObject *) NULL, __pyx_n_s_cprotobuf_internal, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "cprotobuf/internal.pyx":205
 * 
 * class MetaProtoEntity(type):
 *     def __new__(cls, clsname, bases, attrs):             # <<<<<<<<<<<<<<
 *         if clsname == 'ProtoEntity':
 *             return super(MetaProtoEntity, cls).__new__(cls, clsname, bases, attrs)
 */
  __pyx_t_4 = __Pyx_CyFunction_NewEx(&__pyx_mdef_9cprotobuf_8internal_15MetaProtoEntity_1__new__, __Pyx_CYFUNCTION_STATICMETHOD, __pyx_n_s_MetaProtoEntity___new, NULL, __pyx_n_s_cprotobuf_internal, __pyx_d, ((PyObject *)__pyx_codeobj__14)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetItem(__pyx_t_3, __pyx_n_s_new, __pyx_t_4) < 0) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cprotobuf/internal.pyx":204
 *     _proto_classes[name] = cls
 * 
 * class MetaProtoEntity(type):             # <<<<<<<<<<<<<<
 *     def __new__(cls, clsname, bases, attrs):
 *         if clsname == 'ProtoEntity':
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_MetaProtoEntity, __pyx_t_1, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_MetaProtoEntity, __pyx_t_4) < 0) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":233
 *         return newcls
 * 
 * class ProtoEntity(object):             # <<<<<<<<<<<<<<
 *     __metaclass__ = MetaProtoEntity
 * 
 */
  __pyx_t_1 = __Pyx_CalculateMetaclass(NULL, __pyx_tuple__15); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare(__pyx_t_1, __pyx_tuple__15, __pyx_n_s_ProtoEntity, __pyx_n_s_ProtoEntity, (PyObject *) NULL, __pyx_n_s_cprotobuf_internal, (PyObject *) NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "cprotobuf/internal.pyx":234
 * 
 * class ProtoEntity(object):
 *     __metaclass__ = MetaProtoEntity             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, **kwargs):
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_MetaProtoEntity); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_metaclass, __pyx_t_3) < 0) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cprotobuf/internal.pyx":236
 *     __metaclass__ = MetaProtoEntity
 * 
 *     def __init__(self, **kwargs):             # <<<<<<<<<<<<<<
 *         self.__dict__.update(kwargs)
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_NewEx(&__pyx_mdef_9cprotobuf_8internal_11ProtoEntity_1__init__, 0, __pyx_n_s_ProtoEntity___init, NULL, __pyx_n_s_cprotobuf_internal, __pyx_d, ((PyObject *)__pyx_codeobj__17)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_init, __pyx_t_3) < 0) __PYX_ERR(0, 236, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cprotobuf/internal.pyx":239
 *         self.__dict__.update(kwargs)
 * 
 *     def SerializeToString(self):             # <<<<<<<<<<<<<<
 *         return encode_data(type(self), self.__dict__)
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_NewEx(&__pyx_mdef_9cprotobuf_8internal_11ProtoEntity_3SerializeToString, 0, __pyx_n_s_ProtoEntity_SerializeToString, NULL, __pyx_n_s_cprotobuf_internal, __pyx_d, ((PyObject *)__pyx_codeobj__19)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_SerializeToString, __pyx_t_3) < 0) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cprotobuf/internal.pyx":242
 *         return encode_data(type(self), self.__dict__)
 * 
 *     def ParseFromString(self, s, int offset=0, int count=-1):             # <<<<<<<<<<<<<<
 *         cdef char *buff = <char*>s
 *         cdef char *start
 */
  __pyx_t_3 = __Pyx_PyInt_From_int(((int)0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_int(((int)-1)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_CyFunction_NewEx(&__pyx_mdef_9cprotobuf_8internal_11ProtoEntity_5ParseFromString, 0, __pyx_n_s_ProtoEntity_ParseFromString, NULL, __pyx_n_s_cprotobuf_internal, __pyx_d, ((PyObject *)__pyx_codeobj__21)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_t_5);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_ParseFromString, __pyx_t_4) < 0) __PYX_ERR(0, 242, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cprotobuf/internal.pyx":263
 *             raise DecodeError(e.args[0] - <uint64_t>start, e.args[1])
 * 
 *     def __unicode__(self):             # <<<<<<<<<<<<<<
 *         return str(self).decode('utf-8')
 * 
 */
  __pyx_t_4 = __Pyx_CyFunction_NewEx(&__pyx_mdef_9cprotobuf_8internal_11ProtoEntity_7__unicode__, 0, __pyx_n_s_ProtoEntity___unicode, NULL, __pyx_n_s_cprotobuf_internal, __pyx_d, ((PyObject *)__pyx_codeobj__23)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_unicode, __pyx_t_4) < 0) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cprotobuf/internal.pyx":266
 *         return str(self).decode('utf-8')
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return str(self.todict())
 * 
 */
  __pyx_t_4 = __Pyx_CyFunction_NewEx(&__pyx_mdef_9cprotobuf_8internal_11ProtoEntity_9__str__, 0, __pyx_n_s_ProtoEntity___str, NULL, __pyx_n_s_cprotobuf_internal, __pyx_d, ((PyObject *)__pyx_codeobj__25)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_str, __pyx_t_4) < 0) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cprotobuf/internal.pyx":269
 *         return str(self.todict())
 * 
 *     def todict(self):             # <<<<<<<<<<<<<<
 *         cdef Field f
 *         data = {}
 */
  __pyx_t_4 = __Pyx_CyFunction_NewEx(&__pyx_mdef_9cprotobuf_8internal_11ProtoEntity_11todict, 0, __pyx_n_s_ProtoEntity_todict, NULL, __pyx_n_s_cprotobuf_internal, __pyx_d, ((PyObject *)__pyx_codeobj__27)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetItem(__pyx_t_2, __pyx_n_s_todict, __pyx_t_4) < 0) __PYX_ERR(0, 269, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cprotobuf/internal.pyx":233
 *         return newcls
 * 
 * class ProtoEntity(object):             # <<<<<<<<<<<<<<
 *     __metaclass__ = MetaProtoEntity
 * 
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_1, __pyx_n_s_ProtoEntity, __pyx_tuple__15, __pyx_t_2, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ProtoEntity, __pyx_t_4) < 0) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":291
 *         return data
 * 
 * def encode_data(cls, dict d):             # <<<<<<<<<<<<<<
 *     cdef bytearray buf = bytearray()
 *     cdef bytearray buf1
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_9cprotobuf_8internal_5encode_data, NULL, __pyx_n_s_cprotobuf_internal); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_encode_data, __pyx_t_1) < 0) __PYX_ERR(0, 291, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_RepeatedContainer(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x64a34c4:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_9cprotobuf_8internal_7__pyx_unpickle_RepeatedContainer, NULL, __pyx_n_s_cprotobuf_internal); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_RepeatedContainer, __pyx_t_1) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cprotobuf/internal.pyx":1
 * include "utils.pxi"             # <<<<<<<<<<<<<<
 * import inspect
 * #import traceback
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init cprotobuf.internal", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init cprotobuf.internal");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* GetModuleGlobalName */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* PyFunctionFastCall */
  #if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyCFunctionFastCall */
  #if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCall */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* RaiseArgTupleInvalid */
  static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
  static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
  static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyObjectCallMethO */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyErrFetchRestore */
  #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
  #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* KeywordStringCheck */
  static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* PyObjectCallMethod1 */
  static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = NULL;
#if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(method))) {
        PyObject *self = PyMethod_GET_SELF(method);
        if (likely(self)) {
            PyObject *args;
            PyObject *function = PyMethod_GET_FUNCTION(method);
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyCFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            args = PyTuple_New(2);
            if (unlikely(!args)) goto done;
            Py_INCREF(self);
            PyTuple_SET_ITEM(args, 0, self);
            Py_INCREF(arg);
            PyTuple_SET_ITEM(args, 1, arg);
            Py_INCREF(function);
            result = __Pyx_PyObject_Call(function, args, NULL);
            Py_DECREF(args);
            Py_DECREF(function);
            return result;
        }
    }
#endif
    result = __Pyx_PyObject_CallOneArg(method, arg);
    goto done;
done:
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method, *result = NULL;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) goto done;
    result = __Pyx__PyObject_CallMethod1(method, arg);
done:
    Py_XDECREF(method);
    return result;
}

/* append */
  static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x) {
    if (likely(PyList_CheckExact(L))) {
        if (unlikely(__Pyx_PyList_Append(L, x) < 0)) return -1;
    } else {
        PyObject* retval = __Pyx_PyObject_CallMethod1(L, __pyx_n_s_append, x);
        if (unlikely(!retval))
            return -1;
        Py_DECREF(retval);
    }
    return 0;
}

/* PyErrExceptionMatches */
  #if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetAttr */
  static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
  static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* PyObjectCallNoArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* SaveResetException */
    #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if PY_VERSION_HEX >= 0x030700A2
    *type = tstate->exc_state.exc_type;
    *value = tstate->exc_state.exc_value;
    *tb = tstate->exc_state.exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if PY_VERSION_HEX >= 0x030700A2
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = type;
    tstate->exc_state.exc_value = value;
    tstate->exc_state.exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* GetException */
    #if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
#endif
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if PY_VERSION_HEX >= 0x030700A2
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = local_type;
    tstate->exc_state.exc_value = local_value;
    tstate->exc_state.exc_traceback = local_tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* WriteUnraisableException */
      static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* BytesEquals */
      static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
      static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* RaiseTooManyValuesToUnpack */
      static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
      static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
      static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
      static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* ExtTypeTest */
      static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* FetchCommonType */
      static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunction */
      static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromString(m->func.m_ml->ml_name);
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_New(PyTypeObject *type, PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    __pyx_CyFunctionObject *op = PyObject_GC_New(__pyx_CyFunctionObject, type);
    if (op == NULL)
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    PyObject_GC_Track(op);
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* GetItemInt */
          static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ArgTypeTest */
          static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* dict_getitem_default */
          static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value) {
    PyObject* value;
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (unlikely(PyErr_Occurred()))
            return NULL;
        value = default_value;
    }
    Py_INCREF(value);
#else
    if (PyString_CheckExact(key) || PyUnicode_CheckExact(key) || PyInt_CheckExact(key)) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            value = default_value;
        }
        Py_INCREF(value);
    } else {
        if (default_value == Py_None)
            default_value = NULL;
        value = PyObject_CallMethodObjArgs(
            d, __pyx_n_s_get, key, default_value, NULL);
    }
#endif
    return value;
}

/* PyObjectCallMethod2 */
          static PyObject* __Pyx_PyObject_CallMethod2(PyObject* obj, PyObject* method_name, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *method, *result = NULL;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) return NULL;
#if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(method)) && likely(PyMethod_GET_SELF(method))) {
        PyObject *self, *function;
        self = PyMethod_GET_SELF(method);
        function = PyMethod_GET_FUNCTION(method);
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(function)) {
            PyObject *args[3] = {self, arg1, arg2};
            result = __Pyx_PyFunction_FastCall(function, args, 3);
            goto done;
        }
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(function)) {
            PyObject *args[3] = {self, arg1, arg2};
            result = __Pyx_PyFunction_FastCall(function, args, 3);
            goto done;
        }
        #endif
        args = PyTuple_New(3);
        if (unlikely(!args)) goto done;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 1, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 2, arg2);
        Py_INCREF(function);
        Py_DECREF(method);
        method = function;
    } else
#endif
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(method)) {
        PyObject *args[2] = {arg1, arg2};
        result = __Pyx_PyFunction_FastCall(method, args, 2);
        goto done;
    } else
#endif
#if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(method)) {
        PyObject *args[2] = {arg1, arg2};
        result = __Pyx_PyCFunction_FastCall(method, args, 2);
        goto done;
    } else
#endif
    {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto done;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
    }
    result = __Pyx_PyObject_Call(method, args, NULL);
    Py_DECREF(args);
done:
    Py_DECREF(method);
    return result;
}

/* dict_setdefault */
          static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value,
                                                       CYTHON_UNUSED int is_safe_type) {
    PyObject* value;
#if PY_VERSION_HEX >= 0x030400A0
    if ((1)) {
        value = PyDict_SetDefault(d, key, default_value);
        if (unlikely(!value)) return NULL;
        Py_INCREF(value);
#else
    if (is_safe_type == 1 || (is_safe_type == -1 &&
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
            (PyUnicode_CheckExact(key) || PyString_CheckExact(key) || PyLong_CheckExact(key)))) {
        value = PyDict_GetItemWithError(d, key);
        if (unlikely(!value)) {
            if (unlikely(PyErr_Occurred()))
                return NULL;
            if (unlikely(PyDict_SetItem(d, key, default_value) == -1))
                return NULL;
            value = default_value;
        }
        Py_INCREF(value);
#else
            (PyString_CheckExact(key) || PyUnicode_CheckExact(key) || PyInt_CheckExact(key) || PyLong_CheckExact(key)))) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            if (unlikely(PyDict_SetItem(d, key, default_value) == -1))
                return NULL;
            value = default_value;
        }
        Py_INCREF(value);
#endif
#endif
    } else {
        value = __Pyx_PyObject_CallMethod2(d, __pyx_n_s_setdefault, key, default_value);
    }
    return value;
}

/* Import */
              static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
              static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
              static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* SetupReduce */
              static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* SetVTable */
              static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* CalculateMetaclass */
              static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* Py3ClassCreate */
              static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* PatchModuleWithCoroutine */
              static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchInspect */
              static PyObject* __Pyx_patch_inspect(PyObject* module) {
#if defined(__Pyx_Generator_USED) && (!defined(CYTHON_PATCH_INSPECT) || CYTHON_PATCH_INSPECT)
    static int inspect_patched = 0;
    if (unlikely((!inspect_patched) && module)) {
        module = __Pyx_Coroutine_patch_module(
            module, ""
"old_types = getattr(_module.isgenerator, '_cython_generator_types', None)\n"
"if old_types is None or not isinstance(old_types, set):\n"
"    old_types = set()\n"
"    def cy_wrap(orig_func, type=type, cython_generator_types=old_types):\n"
"        def cy_isgenerator(obj): return type(obj) in cython_generator_types or orig_func(obj)\n"
"        cy_isgenerator._cython_generator_types = cython_generator_types\n"
"        return cy_isgenerator\n"
"    _module.isgenerator = cy_wrap(_module.isgenerator)\n"
"old_types.add(_cython_generator_type)\n"
        );
        inspect_patched = 1;
    }
#else
    if ((0)) return __Pyx_Coroutine_patch_module(module, NULL);
#endif
    return module;
}

/* CLineInTraceback */
              #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = PyDict_GetItem(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
              static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
              #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
              #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint64_t(uint64_t value) {
    const uint64_t neg_one = (uint64_t) -1, const_zero = (uint64_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint64_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint64_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint64_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint64_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint64_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint64_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint32_t(uint32_t value) {
    const uint32_t neg_one = (uint32_t) -1, const_zero = (uint32_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint32_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint32_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint32_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint32_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint32_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int32_t(int32_t value) {
    const int32_t neg_one = (int32_t) -1, const_zero = (int32_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int32_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int32_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int32_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int32_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int32_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int32_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int64_t(int64_t value) {
    const int64_t neg_one = (int64_t) -1, const_zero = (int64_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int64_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int64_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int64_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int64_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int64_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int64_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_char(unsigned char value) {
    const unsigned char neg_one = (unsigned char) -1, const_zero = (unsigned char) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned char) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned char) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned char) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned char) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned char) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned char),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
              static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
              static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
              static CYTHON_INLINE uint32_t __Pyx_PyInt_As_uint32_t(PyObject *x) {
    const uint32_t neg_one = (uint32_t) -1, const_zero = (uint32_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint32_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint32_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint32_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint32_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 2 * PyLong_SHIFT) {
                            return (uint32_t) (((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 3 * PyLong_SHIFT) {
                            return (uint32_t) (((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 4 * PyLong_SHIFT) {
                            return (uint32_t) (((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint32_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint32_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint32_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint32_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint32_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32_t) ((((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32_t) ((((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32_t) ((((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint32_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint32_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint32_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint32_t) -1;
        }
    } else {
        uint32_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint32_t) -1;
        val = __Pyx_PyInt_As_uint32_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint32_t");
    return (uint32_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint32_t");
    return (uint32_t) -1;
}

/* CIntFromPy */
              static CYTHON_INLINE int32_t __Pyx_PyInt_As_int32_t(PyObject *x) {
    const int32_t neg_one = (int32_t) -1, const_zero = (int32_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int32_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int32_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int32_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int32_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int32_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(int32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) >= 2 * PyLong_SHIFT) {
                            return (int32_t) (((((int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) >= 3 * PyLong_SHIFT) {
                            return (int32_t) (((((((int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) >= 4 * PyLong_SHIFT) {
                            return (int32_t) (((((((((int32_t)digits[3]) << PyLong_SHIFT) | (int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int32_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int32_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int32_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int32_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int32_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int32_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int32_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (int32_t) (((int32_t)-1)*(((((int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (int32_t) ((((((int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int32_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (int32_t) (((int32_t)-1)*(((((((int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (int32_t) ((((((((int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int32_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (int32_t) (((int32_t)-1)*(((((((((int32_t)digits[3]) << PyLong_SHIFT) | (int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (int32_t) ((((((((((int32_t)digits[3]) << PyLong_SHIFT) | (int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int32_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int32_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int32_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int32_t) -1;
        }
    } else {
        int32_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int32_t) -1;
        val = __Pyx_PyInt_As_int32_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int32_t");
    return (int32_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int32_t");
    return (int32_t) -1;
}

/* CIntFromPy */
              static CYTHON_INLINE uint64_t __Pyx_PyInt_As_uint64_t(PyObject *x) {
    const uint64_t neg_one = (uint64_t) -1, const_zero = (uint64_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint64_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint64_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint64_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint64_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint64_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint64_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) >= 2 * PyLong_SHIFT) {
                            return (uint64_t) (((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint64_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) >= 3 * PyLong_SHIFT) {
                            return (uint64_t) (((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint64_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) >= 4 * PyLong_SHIFT) {
                            return (uint64_t) (((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint64_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint64_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint64_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint64_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint64_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint64_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint64_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint64_t) (((uint64_t)-1)*(((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint64_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint64_t) ((((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint64_t) (((uint64_t)-1)*(((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint64_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint64_t) ((((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint64_t) (((uint64_t)-1)*(((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint64_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint64_t) ((((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint64_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint64_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint64_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint64_t) -1;
        }
    } else {
        uint64_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint64_t) -1;
        val = __Pyx_PyInt_As_uint64_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint64_t");
    return (uint64_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint64_t");
    return (uint64_t) -1;
}

/* CIntFromPy */
              static CYTHON_INLINE int64_t __Pyx_PyInt_As_int64_t(PyObject *x) {
    const int64_t neg_one = (int64_t) -1, const_zero = (int64_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int64_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int64_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int64_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int64_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int64_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(int64_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) >= 2 * PyLong_SHIFT) {
                            return (int64_t) (((((int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int64_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) >= 3 * PyLong_SHIFT) {
                            return (int64_t) (((((((int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int64_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) >= 4 * PyLong_SHIFT) {
                            return (int64_t) (((((((((int64_t)digits[3]) << PyLong_SHIFT) | (int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int64_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int64_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int64_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int64_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int64_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int64_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int64_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int64_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int64_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) - 1 > 2 * PyLong_SHIFT) {
                            return (int64_t) (((int64_t)-1)*(((((int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int64_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) - 1 > 2 * PyLong_SHIFT) {
                            return (int64_t) ((((((int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int64_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) - 1 > 3 * PyLong_SHIFT) {
                            return (int64_t) (((int64_t)-1)*(((((((int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int64_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) - 1 > 3 * PyLong_SHIFT) {
                            return (int64_t) ((((((((int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int64_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) - 1 > 4 * PyLong_SHIFT) {
                            return (int64_t) (((int64_t)-1)*(((((((((int64_t)digits[3]) << PyLong_SHIFT) | (int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int64_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) - 1 > 4 * PyLong_SHIFT) {
                            return (int64_t) ((((((((((int64_t)digits[3]) << PyLong_SHIFT) | (int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int64_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int64_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int64_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int64_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int64_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int64_t) -1;
        }
    } else {
        int64_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int64_t) -1;
        val = __Pyx_PyInt_As_int64_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int64_t");
    return (int64_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int64_t");
    return (int64_t) -1;
}

/* CIntFromPy */
              static CYTHON_INLINE unsigned char __Pyx_PyInt_As_unsigned_char(PyObject *x) {
    const unsigned char neg_one = (unsigned char) -1, const_zero = (unsigned char) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned char) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned char, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned char) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned char) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned char, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) >= 2 * PyLong_SHIFT) {
                            return (unsigned char) (((((unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) >= 3 * PyLong_SHIFT) {
                            return (unsigned char) (((((((unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) >= 4 * PyLong_SHIFT) {
                            return (unsigned char) (((((((((unsigned char)digits[3]) << PyLong_SHIFT) | (unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned char) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned char) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned char, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned char) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned char, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned char) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned char, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned char,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned char) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned char) (((unsigned char)-1)*(((((unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned char) ((((((unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned char) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned char) (((unsigned char)-1)*(((((((unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned char) ((((((((unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned char) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned char) (((unsigned char)-1)*(((((((((unsigned char)digits[3]) << PyLong_SHIFT) | (unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned char) ((((((((((unsigned char)digits[3]) << PyLong_SHIFT) | (unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned char) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned char, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned char) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned char, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned char val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned char) -1;
        }
    } else {
        unsigned char val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned char) -1;
        val = __Pyx_PyInt_As_unsigned_char(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned char");
    return (unsigned char) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned char");
    return (unsigned char) -1;
}

/* FastTypeChecks */
              #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
              static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* ModuleImport */
              #ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
              #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* InitStrings */
              static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            PyErr_Clear();
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
